@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */ 
 
 #include "mutt.h"
@@ -153,7 +153,7 @@ void ci_start_color (void)
 #ifdef HAVE_COLOR
 
 #ifdef USE_SLANG_CURSES
-static char * get_color_name (int val)
+static char *get_color_name (char *dest, size_t destlen, int val)
 {
   static char * missing[3] = {"brown", "lightgray", ""};
   int i;
@@ -161,21 +161,33 @@ static char * get_color_name (int val)
   switch (val)
   {
     case COLOR_YELLOW:
-      return (missing[0]);
+      strfcpy (dest, missing[0], destlen);
+      return dest;
 
     case COLOR_WHITE:
-      return (missing[1]);
+      strfcpy (dest, missing[1], destlen);
+      return dest;
       
     case COLOR_DEFAULT:
-      return (missing[2]);
+      strfcpy (dest, missing[2], destlen);
+      return dest;
   }
 
   for (i = 0; Colors[i].name; i++)
   {
     if (Colors[i].value == val)
-      return (Colors[i].name);
+    {
+      strfcpy (dest, Colors[i].name, destlen);
+      return dest;
+    }
   }
-  return (Colors[0].name);
+
+  /* Sigh. If we got this far, the color is of the form 'colorN'
+   * Slang can handle this itself, so just return 'colorN'
+   */
+
+  snprintf (dest, destlen, "color%d", val);
+  return dest;
 }
 #endif
 
@@ -183,6 +195,10 @@ int mutt_alloc_color (int fg, int bg)
 {
   COLOR_LIST *p = ColorList;
   int i;
+  
+#if defined (USE_SLANG_CURSES)
+  char fgc[SHORT_STRING], bgc[SHORT_STRING];
+#endif
 
   /* check to see if this color is already allocated to save space */
   while (p)
@@ -223,7 +239,7 @@ int mutt_alloc_color (int fg, int bg)
 
 #if defined (USE_SLANG_CURSES)
   if (fg == COLOR_DEFAULT || bg == COLOR_DEFAULT)
-    SLtt_set_color (i, NULL, get_color_name (fg), get_color_name (bg));
+    SLtt_set_color (i, NULL, get_color_name (fgc, sizeof (fgc), fg), get_color_name (bgc, sizeof (bgc), bg));
   else
 #elif defined (HAVE_USE_DEFAULT_COLORS)
   if (fg == COLOR_DEFAULT)
@@ -376,13 +392,17 @@ _mutt_parse_uncolor (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err,
 
   if(
 #ifdef HAVE_COLOR
-     (parse_uncolor && !has_colors())
+     (parse_uncolor && !option(OPTNOCURSES) && !has_colors())
      || (!parse_uncolor && has_colors())
 #else
      parse_uncolor
 #endif
      )
   {
+    do
+      mutt_extract_token (buf, s, 0);
+    while (MoreArgs (s));
+
     return 0;
   }
      
@@ -687,8 +707,10 @@ _mutt_parse_color (BUFFER *buf, BUFFER *s, BUFFER *err,
   
 #ifdef HAVE_COLOR
 # ifdef HAVE_USE_DEFAULT_COLORS
-  if (has_colors() && use_default_colors () != OK 
-      && (fg == COLOR_DEFAULT || bg == COLOR_DEFAULT))
+  if (has_colors()
+    /* delay use_default_colors() until needed, since it initializes things */
+    && (fg == COLOR_DEFAULT || bg == COLOR_DEFAULT)
+    && use_default_colors () != OK)
   {
     strfcpy (err->data, _("default colors not supported"), err->dsize);
     return (-1);
