@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */ 
 
 #include "mutt.h"
@@ -24,9 +24,14 @@
 #include "sort.h"
 #include "buffy.h"
 
+#ifdef USE_IMAP
+#include "mx.h"
+#include "imap.h"
+#endif
 
 
-#ifdef _PGPPATH
+
+#ifdef HAVE_PGP
 #include "pgp.h"
 #endif
 
@@ -47,30 +52,35 @@
 #include <sys/stat.h>
 #include <errno.h>
 
+static const char *No_mailbox_is_open = N_("No mailbox is open.");
+static const char *There_are_no_messages = N_("There are no messages.");
+static const char *Mailbox_is_read_only = N_("Mailbox is read-only.");
+static const char *Function_not_permitted_in_attach_message_mode = N_("Function not permitted in attach-message mode.");
+
 #define CHECK_MSGCOUNT if (!Context) \
 	{ \
 	  	mutt_flushinp (); \
-		mutt_error _("No mailbox is open."); \
+		mutt_error _(No_mailbox_is_open); \
 		break; \
 	} \
 	else if (!Context->msgcount) \
 	{ \
 	  	mutt_flushinp (); \
-		mutt_error _("There are no messages."); \
+		mutt_error _(There_are_no_messages); \
 		break; \
 	}
 
 #define CHECK_READONLY if (Context->readonly) \
 			{ \
 			  	mutt_flushinp (); \
-				mutt_error _("Mailbox is read-only."); \
+				mutt_error _(Mailbox_is_read_only); \
 				break; \
 			}
 
 #define CHECK_ATTACH if(option(OPTATTACHMSG)) \
 		     {\
 			mutt_flushinp (); \
-			mutt_error ("Function not permitted in attach-message mode."); \
+			mutt_error _(Function_not_permitted_in_attach_message_mode); \
 			break; \
 		     }
 
@@ -230,6 +240,113 @@ static int mx_toggle_write (CONTEXT *ctx)
   return 0;
 }
 
+static void update_index (MUTTMENU *menu, CONTEXT *ctx, int check,
+			  int oldcount, int index_hint)
+{
+  /* store pointers to the newly added messages */
+  HEADER  **save_new = NULL;
+  int j;
+  
+  /* take note of the current message */
+  if (oldcount)
+  {
+    if (menu->current < Context->vcount)
+      menu->oldcurrent = index_hint;
+    else
+      oldcount = 0; /* invalid message number! */
+  }
+  
+  /* We are in a limited view. Check if the new message(s) satisfy
+   * the limit criteria. If they do, set their virtual msgno so that
+   * they will be visible in the limited view */
+  if (Context->pattern)
+  {
+#define THIS_BODY Context->hdrs[j]->content
+    if (oldcount || check == M_REOPENED)
+    {
+      for (j = (check == M_REOPENED) ? 0 : oldcount; j < Context->msgcount; j++)
+      {
+	if (mutt_pattern_exec (Context->limit_pattern,
+			       M_MATCH_FULL_ADDRESS, 
+			       Context, Context->hdrs[j]))
+	{
+	  Context->hdrs[j]->virtual = Context->vcount;
+	  Context->v2r[Context->vcount] = j;
+	  Context->hdrs[j]->limited = 1;
+	  Context->vcount++;
+	  Context->vsize += THIS_BODY->length + THIS_BODY->offset - THIS_BODY->hdr_offset;
+	}
+      }
+    }
+#undef THIS_BODY
+  }
+    
+  /* save the list of new messages */
+  if (oldcount && check != M_REOPENED
+      && ((Sort & SORT_MASK) == SORT_THREADS))
+  {
+    save_new = (HEADER **) safe_malloc (sizeof (HEADER *) * (Context->msgcount - oldcount));
+    for (j = oldcount; j < Context->msgcount; j++)
+      save_new[j-oldcount] = Context->hdrs[j];
+  }
+  
+  /* if the mailbox was reopened, need to rethread from scratch */
+  mutt_sort_headers (Context, (check == M_REOPENED));
+  
+  /* uncollapse threads with new mail */
+  if ((Sort & SORT_MASK) == SORT_THREADS)
+  {
+    if (check == M_REOPENED)
+    {
+      HEADER *h;
+      
+      h = Context->tree;
+      Context->collapsed = 0;
+      
+      while (h)
+      {
+	mutt_uncollapse_thread (Context, h);
+	h = h->next;
+      }
+      mutt_set_virtual (Context);
+    }
+    else if (oldcount)
+    {
+      for (j = 0; j < Context->msgcount - oldcount; j++)
+      {
+	int k;
+	
+	for (k = 0; k < Context->msgcount; k++)
+	{
+	  HEADER *h = Context->hdrs[k];
+	  if (h == save_new[j] && (!Context->pattern || h->limited))
+	    mutt_uncollapse_thread (Context, h);
+	}
+      }
+      FREE (&save_new);
+      mutt_set_virtual (Context);
+    }
+  }
+  
+  menu->current = -1;
+  if (oldcount)
+  {
+    /* restore the current message to the message it was pointing to */
+    for (j = 0; j < Context->vcount; j++)
+    {
+      if (Context->hdrs[Context->v2r[j]]->index == menu->oldcurrent)
+      {
+	menu->current = j;
+	break;
+      }
+    }
+  }
+  
+  if (menu->current < 0)
+    menu->current = ci_first_message ();
+  
+}
+
 static void resort_index (MUTTMENU *menu)
 {
   int i;
@@ -279,7 +396,7 @@ int mutt_index_menu (void)
   int do_buffy_notify = 1;
   int close = 0; /* did we OP_QUIT or OP_EXIT out of this menu? */
   int attach_msg = option(OPTATTACHMSG);
-
+  
   menu = mutt_new_menu ();
   menu->menu = MENU_MAIN;
   menu->offset = 1;
@@ -299,6 +416,13 @@ int mutt_index_menu (void)
     menu->max = Context ? Context->vcount : 0;
     oldcount = Context ? Context->msgcount : 0;
 
+    /* check if we need to resort the index because just about
+     * any 'op' below could do mutt_enter_command(), either here or
+     * from any new menu launched, and change $sort/$sort_aux
+     */
+    if (option (OPTNEEDRESORT) && Context && Context->msgcount)
+      resort_index (menu);
+
     if (Context && !attach_msg)
     {
       int check;
@@ -306,9 +430,14 @@ int mutt_index_menu (void)
        * changed about the file (either we got new mail or the file was
        * modified underneath us.)
        */
-      index_hint = (Context->vcount) ? CURHDR->index : 0;
 
-      if ((check = mx_check_mailbox (Context, &index_hint)) < 0)
+#ifdef USE_IMAP
+      imap_allow_reopen (Context);
+#endif
+    
+      index_hint = (Context->vcount && menu->current < Context->vcount) ? CURHDR->index : 0;
+
+      if ((check = mx_check_mailbox (Context, &index_hint, 0)) < 0)
       {
 	if (!Context->path)
 	{
@@ -321,102 +450,8 @@ int mutt_index_menu (void)
       }
       else if (check == M_NEW_MAIL || check == M_REOPENED)
       {
-	/* store pointers to the newly added messages */
-	HEADER  **save_new = NULL;
-
-	/* take note of the current message */
-	if (oldcount)
-	{
-	  if (menu->current < Context->vcount)
-	    menu->oldcurrent = index_hint;
-	  else
-	    oldcount = 0; /* invalid message number! */
-	}
-
-	/* We are in a limited view. Check if the new message(s) satisfy
-	 * the limit criteria. If they do, set their virtual msgno so that
-	 * they will be visible in the limited view */
-        if (Context->pattern)
-	{
-#define THIS_BODY Context->hdrs[j]->content
-	  if (oldcount || check == M_REOPENED)
-	    for (j = (check == M_REOPENED) ? 0 : oldcount; j < Context->msgcount; j++)
-	  {
-            if (mutt_pattern_exec (Context->limit_pattern,
-	                           M_MATCH_FULL_ADDRESS, 
-				    Context, Context->hdrs[j]))
-            {
-		Context->hdrs[j]->virtual = Context->vcount;
-		Context->v2r[Context->vcount] = j;
-		Context->hdrs[j]->limited = 1;
-		Context->vcount++;
-		Context->vsize += THIS_BODY->length + THIS_BODY->offset - THIS_BODY->hdr_offset;
-            }
-	  }
-#undef THIS_BODY
-        }
-
-	/* save the list of new messages */
-	if (oldcount && check != M_REOPENED)
-	{
-	  save_new = (HEADER **) safe_malloc (sizeof (HEADER *) * (Context->msgcount - oldcount));
-	  for (j = oldcount; j < Context->msgcount; j++)
-	    save_new[j-oldcount] = Context->hdrs[j];
-        }
-
-	/* if the mailbox was reopened, need to rethread from scratch */
-	mutt_sort_headers (Context, (check == M_REOPENED));
-
-	/* uncollapse threads with new mail */
-	if ((Sort & SORT_MASK) == SORT_THREADS)
-	{
-	  if (check == M_REOPENED)
-	  {
-	    HEADER *h;
-
-	    h = Context->tree;
-	    Context->collapsed = 0;
-
-	    while (h)
-	    {
-	      mutt_uncollapse_thread (Context, h);
-	      h = h->next;
-	    }
-	    mutt_set_virtual (Context);
-	  }
-	  else if (oldcount)
-	  {
-	    for (j = 0; j < Context->msgcount - oldcount; j++)
-	    {
-	      int k;
-	      
-	      for (k = 0; k < Context->msgcount; k++)
-	      {
-		HEADER *h = Context->hdrs[k];
-		if (h == save_new[j] && (!Context->pattern || h->limited))
-		  mutt_uncollapse_thread (Context, h);
-	      }
-	    }
-	    FREE (&save_new);
-	    mutt_set_virtual (Context);
-	  }
-	}
-
-	menu->current = -1;
-	if (oldcount)
-	{
-	  /* restore the current message to the message it was pointing to */
-	  for (j = 0; j < Context->vcount; j++)
-	    if (Context->hdrs[Context->v2r[j]]->index == menu->oldcurrent)
-	    {
-	      menu->current = j;
-	      break;
-	    }
-	}
-
-	if (menu->current < 0)
-	  menu->current = ci_first_message ();
-
+	update_index (menu, Context, check, oldcount, index_hint);
+	
 	/* notify the user of new mail */
 	if (check == M_REOPENED)
 	  mutt_error _("Mailbox was externally modified.  Flags may be wrong.");
@@ -428,14 +463,18 @@ int mutt_index_menu (void)
 	}
 	/* avoid the message being overwritten by buffy */
 	do_buffy_notify = 0;
-
+	
 	menu->redraw = REDRAW_FULL;
 	menu->max = Context->vcount;
-
+	
 	set_option (OPTSEARCHINVALID);
       }
     }
 
+#ifdef USE_IMAP
+    imap_disallow_reopen (Context);
+#endif
+
     if (!attach_msg)
     {
      /* check for new mail in the incoming folders */
@@ -451,7 +490,8 @@ int mutt_index_menu (void)
        do_buffy_notify = 1;
     }
 
-    mutt_curs_set (0);
+    if (op != -1)
+      mutt_curs_set (0);
 
     if (menu->redraw & REDRAW_FULL)
     {
@@ -487,7 +527,10 @@ int mutt_index_menu (void)
       }
 
       menu->redraw = 0;
-      menu->oldcurrent = menu->current;
+      if (menu->current < menu->max)
+	menu->oldcurrent = menu->current;
+      else
+	menu->oldcurrent = -1;
 
       if (option (OPTARROWCURSOR))
 	move (menu->current - menu->top + menu->offset, 2);
@@ -499,16 +542,14 @@ int mutt_index_menu (void)
 
       dprint(4, (debugfile, "mutt_index_menu[%d]: Got op %d\n", __LINE__, op));
 
-      mutt_curs_set (1);
-      
 #if defined (USE_SLANG_CURSES) || defined (HAVE_RESIZETERM)
-      if (Signals & S_SIGWINCH)
+      if (SigWinch)
       {
 	mutt_flushinp ();
 	mutt_resize_screen ();
 	menu->redraw = REDRAW_FULL;
 	menu->menu = MENU_MAIN;
-	Signals &= ~S_SIGWINCH;
+	SigWinch = 0;
 	menu->top = 0; /* so we scroll the right amount */
 	continue;
       }
@@ -516,7 +557,9 @@ int mutt_index_menu (void)
 
       if (op == -1)
 	continue; /* either user abort or timeout */
-
+      
+      mutt_curs_set (1);
+      
       /* special handling for the tag-prefix function */
       if (op == OP_TAG_PREFIX)
       {
@@ -698,7 +741,8 @@ int mutt_index_menu (void)
       case OP_MAIN_LIMIT:
 
 	CHECK_MSGCOUNT;
-	menu->oldcurrent = Context->vcount ? CURHDR->index : -1;
+	menu->oldcurrent = (Context->vcount && menu->current < Context->vcount) ?
+		CURHDR->index : -1;
 	if (mutt_pattern_func (M_LIMIT, _("Limit to messages matching: ")) == 0)
 	{
 	  if (menu->oldcurrent >= 0)
@@ -730,11 +774,26 @@ int mutt_index_menu (void)
 	}
 
 	if (query_quadoption (OPT_QUIT, _("Quit Mutt?")) == M_YES)
-	{ 
-	  if (!Context || mx_close_mailbox (Context) == 0)
+	{
+	  int check;
+	  
+#ifdef USE_IMAP
+          /* logout gracefully from the IMAP server */
+          if (Context && Context->magic == M_IMAP)
+            imap_set_logout (Context);
+#endif
+	  oldcount = Context ? Context->msgcount : 0;
+
+	  if (!Context || (check = mx_close_mailbox (Context, &index_hint)) == 0)
 	    done = 1;
 	  else
+	  {
+	    if (check == M_NEW_MAIL || check == M_REOPENED)
+	      update_index (menu, Context, check, oldcount, index_hint);
+
 	    menu->redraw = REDRAW_FULL; /* new mail arrived? */
+	    set_option (OPTSEARCHINVALID);
+	  }
 	}
 	break;
 
@@ -775,24 +834,30 @@ int mutt_index_menu (void)
 	CHECK_READONLY;
 	{
 	  int oldvcount = Context->vcount;
+	  int oldcount  = Context->msgcount;
 	  int dcount = 0;
+	  int check;
 
 	  /* calculate the number of messages _above_ the cursor,
 	   * so we can keep the cursor on the current message
 	   */ 
-	  for (j = 0; j < menu->current; j++)
+	  for (j = 0; j <= menu->current; j++)
 	  {
 	    if (Context->hdrs[Context->v2r[j]]->deleted)
 	      dcount++;
 	  }
-	  if (mx_sync_mailbox (Context) == 0)
+
+	  if ((check = mx_sync_mailbox (Context, &index_hint)) == 0)
 	  {
 	    if (Context->vcount != oldvcount)
 	      menu->current -= dcount;
 	    set_option (OPTSEARCHINVALID);
 	  }
-	  
-	  /* do a sanity check even if mx_sync_mailbox failed.
+	  else if (check == M_NEW_MAIL || check == M_REOPENED)
+	    update_index (menu, Context, check, oldcount, index_hint);
+
+	  /* 
+	   * do a sanity check even if mx_sync_mailbox failed.
 	   */
 
 	  if (menu->current < 0 || menu->current >= Context->vcount)
@@ -856,17 +921,17 @@ int mutt_index_menu (void)
 
       case OP_MAIN_CHANGE_FOLDER:
       
-	if (attach_msg || option (OPTREADONLY))
+	if (attach_msg)
 	  op = OP_MAIN_CHANGE_FOLDER_READONLY;
 
 	/* fallback to the readonly case */
 
       case OP_MAIN_CHANGE_FOLDER_READONLY:
 
-        if (op == OP_MAIN_CHANGE_FOLDER)
-          cp = _("Open mailbox");
-        else
+        if ((op == OP_MAIN_CHANGE_FOLDER_READONLY) || option (OPTREADONLY))
           cp = _("Open mailbox in read-only mode");
+        else
+          cp = _("Open mailbox");
 
 	buf[0] = '\0';
 	mutt_buffy (buf);
@@ -888,14 +953,17 @@ int mutt_index_menu (void)
 
         if (Context)
         {
-	  FREE (&LastFolder);
-	  LastFolder = safe_strdup (Context->path);
-	}
+	  int check;
 
-	if (Context)
-	{
-	  if (mx_close_mailbox (Context) != 0)
+	  mutt_str_replace (&LastFolder, Context->path);
+	  oldcount = Context ? Context->msgcount : 0;
+
+	  if ((check = mx_close_mailbox (Context, &index_hint)) != 0)
 	  {
+	    if (check == M_NEW_MAIL || check == M_REOPENED)
+	      update_index (menu, Context, check, oldcount, index_hint);
+		
+	    set_option (OPTSEARCHINVALID);
 	    menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
 	    break;
 	  }
@@ -949,13 +1017,8 @@ int mutt_index_menu (void)
 	  break;
 	}
 
-	if (option (OPTNEEDRESORT) && Context && Context->msgcount)
-	{
-	  resort_index (menu);
-	}
-
 	menu->menu = MENU_PAGER;
-	menu->oldcurrent = menu->current;
+ 	menu->oldcurrent = menu->current;
 	continue;
 
       case OP_EXIT:
@@ -973,6 +1036,10 @@ int mutt_index_menu (void)
 	{
 	  if (Context)
 	  {
+#ifdef USE_IMAP
+            if (Context->magic == M_IMAP)
+              imap_set_logout (Context);
+#endif
 	    mx_fastclose_mailbox (Context);
 	    safe_free ((void **) &Context);
 	  }
@@ -980,6 +1047,21 @@ int mutt_index_menu (void)
 	}
 	break;
 
+      case OP_EDIT_TYPE:
+
+	CHECK_MSGCOUNT;
+	CHECK_ATTACH;
+	mutt_edit_content_type (CURHDR, CURHDR->content);
+	/* if we were in the pager, redisplay the message */
+	if (menu->menu == MENU_PAGER)
+	{
+	  op = OP_DISPLAY_MESSAGE;
+	  continue;
+	}
+        else
+	  menu->redraw = REDRAW_CURRENT;
+	break;
+
       case OP_MAIN_NEXT_UNDELETED:
 
 	CHECK_MSGCOUNT;
@@ -1068,24 +1150,24 @@ int mutt_index_menu (void)
       case OP_SAVE:
       case OP_DECODE_COPY:
       case OP_DECODE_SAVE:
-#ifdef _PGPPATH
+#ifdef HAVE_PGP
       case OP_DECRYPT_COPY:
       case OP_DECRYPT_SAVE:
 #endif
 	CHECK_MSGCOUNT;
         if (mutt_save_message (tag ? NULL : CURHDR,
-#ifdef _PGPPATH
+#ifdef HAVE_PGP
 			       (op == OP_DECRYPT_SAVE) ||
 #endif
 			       (op == OP_SAVE) || (op == OP_DECODE_SAVE),
 			       (op == OP_DECODE_SAVE) || (op == OP_DECODE_COPY),
-#ifdef _PGPPATH
+#ifdef HAVE_PGP
 			       (op == OP_DECRYPT_SAVE) || (op == OP_DECRYPT_COPY) ||
 #endif
 			       0,
 			       &menu->redraw) == 0 &&
 	    (op == OP_SAVE || op == OP_DECODE_SAVE
-#ifdef _PGPPATH
+#ifdef HAVE_PGP
 	     || op == OP_DECRYPT_SAVE
 #endif
 	     ))
@@ -1137,7 +1219,7 @@ int mutt_index_menu (void)
 	    }
 	  }
 
-	  if (CUR->collapsed)
+	  if (CUR->collapsed && (Sort & SORT_MASK) == SORT_THREADS)
 	  {
 	    if ((op == OP_MAIN_NEXT_UNREAD || op == OP_MAIN_PREV_UNREAD) &&
 		UNREAD (CUR))
@@ -1181,20 +1263,34 @@ int mutt_index_menu (void)
 
 	CHECK_MSGCOUNT;
 	CHECK_READONLY;
-	mutt_set_flag (Context, CURHDR, M_FLAG, !CURHDR->flagged);
 
-	if (option (OPTRESOLVE))
-	{
-	  if ((menu->current = ci_next_undeleted (menu->current)) == -1)
+        if (tag)
+        {
+	  for (j = 0; j < Context->vcount; j++)
 	  {
-	    menu->current = menu->oldcurrent;
-	    menu->redraw = REDRAW_CURRENT;
+	    if (Context->hdrs[Context->v2r[j]]->tagged)
+	      mutt_set_flag (Context, Context->hdrs[Context->v2r[j]],
+			     M_FLAG, !Context->hdrs[Context->v2r[j]]->flagged);
+	  }
+
+	  menu->redraw |= REDRAW_INDEX;
+	}
+        else
+        {
+	  mutt_set_flag (Context, CURHDR, M_FLAG, !CURHDR->flagged);
+	  if (option (OPTRESOLVE))
+	  {
+	    if ((menu->current = ci_next_undeleted (menu->current)) == -1)
+	    {
+	      menu->current = menu->oldcurrent;
+	      menu->redraw = REDRAW_CURRENT;
+	    }
+	    else
+	      menu->redraw = REDRAW_MOTION_RESYNCH;
 	  }
 	  else
-	    menu->redraw = REDRAW_MOTION_RESYNCH;
+	    menu->redraw = REDRAW_CURRENT;
 	}
-	else
-	  menu->redraw = REDRAW_CURRENT;
 	menu->redraw |= REDRAW_STATUS;
 	break;
 
@@ -1289,6 +1385,23 @@ int mutt_index_menu (void)
 	  menu->redraw = REDRAW_MOTION;
 	break;
 
+      case OP_MAIN_PARENT_MESSAGE:
+
+	CHECK_MSGCOUNT;
+
+	if ((menu->current = mutt_parent_message (Context, CURHDR)) < 0)
+	{
+	  menu->current = menu->oldcurrent;
+	}
+	else if (menu->menu == MENU_PAGER)
+        {
+          op = OP_DISPLAY_MESSAGE;
+          continue;
+        }
+        else
+          menu->redraw = REDRAW_MOTION;
+	break;
+
       case OP_MAIN_SET_FLAG:
       case OP_MAIN_CLEAR_FLAG:
 
@@ -1419,11 +1532,15 @@ int mutt_index_menu (void)
 	if (tag)
 	{
 	  mutt_tag_set_flag (M_DELETE, 1);
+	  if (option (OPTDELETEUNTAG))
+	    mutt_tag_set_flag (M_TAG, 0);
 	  menu->redraw = REDRAW_INDEX;
 	}
 	else
 	{
 	  mutt_set_flag (Context, CURHDR, M_DELETE, 1);
+	  if (option (OPTDELETEUNTAG))
+	    mutt_set_flag (Context, CURHDR, M_TAG, 0);
 	  if (option (OPTRESOLVE))
 	  {
 	    if ((menu->current = ci_next_undeleted (menu->current)) == -1)
@@ -1456,6 +1573,9 @@ int mutt_index_menu (void)
 
 	if (rc != -1)
 	{
+	  if (option (OPTDELETEUNTAG))
+	    mutt_thread_set_flag (CURHDR, M_TAG, 0, 
+				  op == OP_DELETE_THREAD ? 0 : 1);
 	  if (option (OPTRESOLVE))
 	    if ((menu->current = ci_next_undeleted (menu->current)) == -1)
 	      menu->current = menu->oldcurrent;
@@ -1466,7 +1586,7 @@ int mutt_index_menu (void)
       case OP_DISPLAY_ADDRESS:
 
 	CHECK_MSGCOUNT;
-	mutt_display_address (CURHDR->env->from);
+	mutt_display_address (CURHDR->env);
 	break;
 
       case OP_ENTER_COMMAND:
@@ -1474,10 +1594,6 @@ int mutt_index_menu (void)
 	CurrentMenu = MENU_MAIN;
 	mutt_enter_command ();
 	mutt_check_rescore (Context);
-	if (option (OPTNEEDRESORT) && Context && Context->msgcount)
-	{
-	  resort_index (menu);
-	}
 	if (option (OPTFORCEREDRAWINDEX))
 	  menu->redraw = REDRAW_FULL;
 	unset_option (OPTFORCEREDRAWINDEX);
@@ -1489,11 +1605,10 @@ int mutt_index_menu (void)
 	CHECK_MSGCOUNT;
 	CHECK_READONLY;
 	CHECK_ATTACH;
-        
-        set_option(OPTUSEHEADERDATE);
-	ci_send_message (SENDEDITMSG, NULL, NULL, Context, CURHDR);
-        unset_option(OPTUSEHEADERDATE);
+
+        mutt_edit_message (Context, tag ? NULL : CURHDR);
 	menu->redraw = REDRAW_FULL;
+
 	break;
 
       case OP_FORWARD_MESSAGE:
@@ -1506,12 +1621,12 @@ int mutt_index_menu (void)
 
 
 
-#ifdef _PGPPATH
+#ifdef HAVE_PGP
       case OP_FORGET_PASSPHRASE:
 
 	mutt_forget_passphrase ();
 	break;
-#endif /* _PGPPATH */
+#endif /* HAVE_PGP */
 
 
 
@@ -1544,7 +1659,7 @@ int mutt_index_menu (void)
 
 
 
-#ifdef _PGPPATH
+#ifdef HAVE_PGP
       case OP_MAIL_KEY:
 	
 	CHECK_ATTACH;
@@ -1559,7 +1674,7 @@ int mutt_index_menu (void)
         menu->redraw = REDRAW_FULL;
         break;
       
-#endif /* _PGPPATH */
+#endif /* HAVE_PGP */
 
 
 
@@ -1607,6 +1722,25 @@ int mutt_index_menu (void)
 	menu->redraw = REDRAW_FULL;
 	break;
 
+      case OP_RESEND:
+      
+        CHECK_ATTACH;
+        CHECK_MSGCOUNT;
+      
+        if (tag)
+        {
+	  for (j = 0; j < Context->vcount; j++)
+	  {
+	    if (Context->hdrs[Context->v2r[j]]->tagged)
+	      mutt_resend_message (NULL, Context, Context->hdrs[Context->v2r[j]]);
+	  }
+	}
+        else
+	  mutt_resend_message (NULL, Context, CURHDR);
+      
+        menu->redraw = REDRAW_FULL;
+        break;
+      
       case OP_REPLY:
 
 	CHECK_ATTACH;
@@ -1710,7 +1844,9 @@ int mutt_index_menu (void)
     {
       menu->menu = MENU_MAIN;
       menu->redraw = REDRAW_FULL;
+#if 0
       set_option (OPTWEED); /* turn header weeding back on. */
+#endif
     }
 
     if (done) break;
