@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */ 
 
 /*
@@ -68,6 +68,8 @@ struct binding_t OpMain[] = {
   { "bounce-message",		OP_BOUNCE_MESSAGE,		"b" },
   { "change-folder",		OP_MAIN_CHANGE_FOLDER,		"c" },
   { "change-folder-readonly",	OP_MAIN_CHANGE_FOLDER_READONLY,	"\033c" },
+  { "collapse-thread",		OP_MAIN_COLLAPSE_THREAD,	"\033v" },
+  { "collapse-all",		OP_MAIN_COLLAPSE_ALL,		"\033V" },
   { "copy-message",		OP_COPY_MESSAGE,		"C" },
   { "decode-copy",		OP_DECODE_COPY,			"\033C" },
   { "decode-save",		OP_DECODE_SAVE,			"\033s" },
@@ -75,13 +77,15 @@ struct binding_t OpMain[] = {
   { "delete-pattern",		OP_MAIN_DELETE_PATTERN,		"D" },
   { "delete-thread",		OP_DELETE_THREAD,		"\004" },
   { "delete-subthread",		OP_DELETE_SUBTHREAD,		"\033d" },
+  { "edit",			OP_EDIT_MESSAGE,		"e" },
+  { "edit-type",		OP_EDIT_TYPE,			"\005" },
   { "forward-message",		OP_FORWARD_MESSAGE,		"f" },
   { "flag-message",		OP_FLAG_MESSAGE,		"F" },
   { "group-reply",		OP_GROUP_REPLY,			"g" },
 #ifdef USE_POP
   { "fetch-mail",		OP_MAIN_FETCH_MAIL,		"G" },
 #endif
-  { "display-headers",		OP_DISPLAY_HEADERS,		"h" },
+  { "display-toggle-weed",		OP_DISPLAY_HEADERS,		"h" },
   { "next-undeleted",		OP_MAIN_NEXT_UNDELETED,		"j" },
   { "previous-undeleted",	OP_MAIN_PREV_UNDELETED,		"k" },
   { "limit",			OP_MAIN_LIMIT,			"l" },
@@ -91,8 +95,10 @@ struct binding_t OpMain[] = {
   { "toggle-write",		OP_TOGGLE_WRITE,		"%" },
   { "next-thread",		OP_MAIN_NEXT_THREAD,		"\016" },
   { "next-subthread",		OP_MAIN_NEXT_SUBTHREAD,		"\033n" },
+  { "query",			OP_QUERY,			"Q" },
   { "quit",			OP_QUIT,			"q" },
   { "reply",			OP_REPLY,			"r" },
+  { "show-limit",		OP_MAIN_SHOW_LIMIT,		"\033l" },
   { "sort-mailbox",		OP_SORT,			"o" },
   { "sort-reverse",		OP_SORT_REVERSE,		"O" },
   { "print-message",		OP_PRINT,			"p" },
@@ -101,6 +107,7 @@ struct binding_t OpMain[] = {
   { "recall-message",		OP_RECALL_MESSAGE,		"R" },
   { "read-thread",		OP_MAIN_READ_THREAD,		"\022" },
   { "read-subthread",		OP_MAIN_READ_SUBTHREAD,		"\033r" },
+  { "resend-message",		OP_RESEND,			"\033e" },
   { "save-message",		OP_SAVE,			"s" },
   { "tag-pattern",		OP_MAIN_TAG_PATTERN,		"T" },
   { "tag-subthread",		OP_TAG_SUBTHREAD,		NULL },
@@ -122,9 +129,16 @@ struct binding_t OpMain[] = {
   { "previous-new",		OP_MAIN_PREV_NEW,		"\033\t" },
   { "next-unread",		OP_MAIN_NEXT_UNREAD,		NULL },
   { "previous-unread",		OP_MAIN_PREV_UNREAD,		NULL },
+  { "parent-message",		OP_MAIN_PARENT_MESSAGE,		"P" },
 
 
-
+#ifdef HAVE_PGP
+  { "extract-keys",		OP_EXTRACT_KEYS,		"\013" },
+  { "forget-passphrase",	OP_FORGET_PASSPHRASE,		"\006" },
+  { "mail-key",			OP_MAIL_KEY,			"\033k" },
+  { "decrypt-copy",		OP_DECRYPT_COPY,		NULL },
+  { "decrypt-save",		OP_DECRYPT_SAVE,		NULL },
+#endif
 
 
 
@@ -141,11 +155,12 @@ struct binding_t OpPager[] = {
   { "delete-message",	OP_DELETE,			"d" },
   { "delete-thread",	OP_DELETE_THREAD,		"\004" },
   { "delete-subthread",	OP_DELETE_SUBTHREAD,		"\033d" },
+  { "edit",		OP_EDIT_MESSAGE,		"e" },
+  { "edit-type",	OP_EDIT_TYPE,			"\005" },
   { "forward-message",	OP_FORWARD_MESSAGE,		"f" },
   { "flag-message",	OP_FLAG_MESSAGE,		"F" },
   { "group-reply",	OP_GROUP_REPLY,			"g" },
-  { "display-headers",	OP_DISPLAY_HEADERS,		"h" },
-  { "exit",		OP_PAGER_EXIT,			"i" },
+  { "display-toggle-weed",	OP_DISPLAY_HEADERS,		"h" },
   { "next-undeleted",	OP_MAIN_NEXT_UNDELETED,		"j" },
   { "next-entry",	OP_NEXT_ENTRY,			"J" },
   { "previous-undeleted",OP_MAIN_PREV_UNDELETED,	"k" },
@@ -161,10 +176,12 @@ struct binding_t OpPager[] = {
   { "previous-thread",	OP_MAIN_PREV_THREAD,		"\020" },
   { "previous-subthread",OP_MAIN_PREV_SUBTHREAD,	"\033p" },
   { "quit",		OP_QUIT,			"Q" },
+  { "exit",		OP_EXIT,			"q" },
   { "reply",		OP_REPLY,			"r" },
   { "recall-message",	OP_RECALL_MESSAGE,		"R" },
   { "read-thread",	OP_MAIN_READ_THREAD,		"\022" },
   { "read-subthread",	OP_MAIN_READ_SUBTHREAD,		"\033r" },
+  { "resend-message",	OP_RESEND,			"\033e" },
   { "save-message",	OP_SAVE,			"s" },
   { "skip-quoted",	OP_PAGER_SKIP_QUOTED,		"S" },
   { "decode-save",	OP_DECODE_SAVE,			"\033s" },
@@ -197,44 +214,18 @@ struct binding_t OpPager[] = {
   { "half-up",		OP_HALF_UP,			NULL },
   { "half-down",	OP_HALF_DOWN,			NULL },
   { "previous-line",	OP_PREV_LINE,			NULL },
+  { "parent-message",	OP_MAIN_PARENT_MESSAGE,		"P" },
 
 
 
 
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
+#ifdef HAVE_PGP
+  { "extract-keys",	OP_EXTRACT_KEYS,		"\013" },
+  { "forget-passphrase",OP_FORGET_PASSPHRASE,		"\006" },
+  { "mail-key",		OP_MAIL_KEY,			"\033k" },
+  { "decrypt-copy",		OP_DECRYPT_COPY,		NULL },
+  { "decrypt-save",		OP_DECRYPT_SAVE,		NULL },
+#endif
 
 
 
@@ -244,20 +235,27 @@ struct binding_t OpPager[] = {
 
 struct binding_t OpAttach[] = {
   { "bounce-message",	OP_BOUNCE_MESSAGE,		"b" },
-  { "display-headers",	OP_DISPLAY_HEADERS,		"h" },
+  { "display-toggle-weed",	OP_DISPLAY_HEADERS,		"h" },
+  { "edit-type",	OP_EDIT_TYPE,			"\005" },
   { "print-entry",	OP_PRINT,			"p" },
   { "save-entry",	OP_SAVE,			"s" },
   { "pipe-entry",	OP_PIPE,			"|" },
   { "view-mailcap",	OP_ATTACH_VIEW_MAILCAP,		"m" },
   { "reply",		OP_REPLY,			"r" },
+  { "resend-message",	OP_RESEND,			"\033e" },
   { "group-reply",	OP_GROUP_REPLY,			"g" },
   { "list-reply",	OP_LIST_REPLY,			"L" },
   { "forward-message",	OP_FORWARD_MESSAGE,		"f" },
   { "view-text",	OP_ATTACH_VIEW_TEXT,		"T" },
   { "view-attach",	OP_VIEW_ATTACH,			M_ENTER_S },
+  { "delete-entry",	OP_DELETE,			"d" },
+  { "undelete-entry",	OP_UNDELETE,			"u" },
 
 
 
+#ifdef HAVE_PGP
+  { "extract-keys",	OP_EXTRACT_KEYS,		"\013" },
+#endif
 
 
 
@@ -266,11 +264,13 @@ struct binding_t OpAttach[] = {
 
 struct binding_t OpCompose[] = {
   { "attach-file",	OP_COMPOSE_ATTACH_FILE,		"a" },
+  { "attach-message",	OP_COMPOSE_ATTACH_MESSAGE,	"A" },
+  { "change-charset",	OP_COMPOSE_CHANGE_CHARSET,	NULL },
   { "edit-bcc",		OP_COMPOSE_EDIT_BCC,		"b" },
   { "edit-cc",		OP_COMPOSE_EDIT_CC,		"c" },
   { "copy-file",	OP_SAVE,			"C" },
   { "detach-file",	OP_DELETE,			"D" },
-  { "display-headers",	OP_DISPLAY_HEADERS,		"h" },
+  { "toggle-disposition",OP_COMPOSE_TOGGLE_DISPOSITION,	"\004" },
   { "edit-description",	OP_COMPOSE_EDIT_DESCRIPTION,	"d" },
   { "edit-message",	OP_COMPOSE_EDIT_MESSAGE,	"e" },
   { "edit-headers",	OP_COMPOSE_EDIT_HEADERS,	"E" },
@@ -278,10 +278,11 @@ struct binding_t OpCompose[] = {
   { "edit-encoding",	OP_COMPOSE_EDIT_ENCODING,	"\005" },
   { "edit-from",	OP_COMPOSE_EDIT_FROM,		"\033f" },
   { "edit-fcc",		OP_COMPOSE_EDIT_FCC,		"f" },
-  { "filter-entry",	OP_FILTER,			"F" }, 
+  { "filter-entry",	OP_FILTER,			"F" },
+  { "get-attachment",	OP_COMPOSE_GET_ATTACHMENT,	"G" },
+  { "display-toggle-weed",	OP_DISPLAY_HEADERS,		"h" },
   { "ispell",		OP_COMPOSE_ISPELL,		"i" },
   { "print-entry",	OP_PRINT,			"l" },
-  { "redraw-screen",	OP_REDRAW,			"\014" },
   { "edit-mime",	OP_COMPOSE_EDIT_MIME,		"m" },
   { "new-mime",		OP_COMPOSE_NEW_MIME,		"n" },
   { "postpone-message",	OP_COMPOSE_POSTPONE_MESSAGE,	"P" },
@@ -289,13 +290,26 @@ struct binding_t OpCompose[] = {
   { "rename-file",	OP_COMPOSE_RENAME_FILE,		"R" },
   { "edit-subject",	OP_COMPOSE_EDIT_SUBJECT,	"s" },
   { "edit-to",		OP_COMPOSE_EDIT_TO,		"t" },
-  { "edit-type",	OP_COMPOSE_EDIT_TYPE,		"\024" },
+  { "edit-type",	OP_EDIT_TYPE,			"\024" },
+  { "write-fcc",	OP_COMPOSE_WRITE_MESSAGE,	"w" },
+  { "recode-attachment",OP_COMPOSE_RECODE,		NULL },
   { "toggle-unlink",	OP_COMPOSE_TOGGLE_UNLINK,	"u" },
+  { "toggle-recode",    OP_COMPOSE_TOGGLE_RECODE,	NULL },
+  { "update-encoding",	OP_COMPOSE_UPDATE_ENCODING,	"U" },
   { "view-attach",	OP_VIEW_ATTACH,			M_ENTER_S },
   { "send-message",	OP_COMPOSE_SEND_MESSAGE,	"y" },
   { "pipe-entry",	OP_PIPE,			"|" },
 
+#ifdef HAVE_PGP
+  { "attach-key",	OP_COMPOSE_ATTACH_KEY,		"\033k" },
+  { "forget-passphrase",OP_FORGET_PASSPHRASE,		"\006"  },
+  { "pgp-menu",		OP_COMPOSE_PGP_MENU,		"p" 	},
+#endif
 
+#ifdef MIXMASTER
+  { "mix",		OP_COMPOSE_MIX,			"M" },
+#endif
+  
   { NULL,		0,				NULL }
 };
 
@@ -308,11 +322,30 @@ struct binding_t OpPost[] = {
 /* The file browser */
 struct binding_t OpBrowser[] = {
   { "change-dir",	OP_CHANGE_DIRECTORY,	"c" },
+  { "display-filename",	OP_BROWSER_TELL,	"@" },
   { "enter-mask",	OP_ENTER_MASK,		"m" },
   { "sort",		OP_SORT,		"o" },
   { "sort-reverse",	OP_SORT_REVERSE,	"O" },
   { "select-new",	OP_BROWSER_NEW_FILE,	"N" },
-  { "check-new",	OP_CHECK_NEW,		"\t" },
+  { "check-new",	OP_CHECK_NEW,		NULL },
+  { "toggle-mailboxes", OP_TOGGLE_MAILBOXES, 	"\t" },
+  { "view-file",	OP_BROWSER_VIEW_FILE,	" " },
+#ifdef USE_IMAP
+  { "new-mailbox",      OP_NEW_MAILBOX,         "n" },
+  { "delete-mailbox",   OP_DELETE_MAILBOX,      "d" },
+  { "subscribe",	OP_BROWSER_SUBSCRIBE,	"s" },
+  { "unsubscribe",	OP_BROWSER_UNSUBSCRIBE,	"u" },
+  { "toggle-subscribed", OP_BROWSER_TOGGLE_LSUB, "T" },
+#endif
+  { NULL,		0,			NULL }
+};
+
+/* External Query Menu */
+struct binding_t OpQuery[] = {
+  { "create-alias",	OP_CREATE_ALIAS,	"a" },
+  { "mail",		OP_MAIL,		"m" },
+  { "query",		OP_QUERY,		"Q" },
+  { "query-append",	OP_QUERY_APPEND,	"A" },
   { NULL,		0,			NULL }
 };
 
@@ -328,6 +361,7 @@ struct binding_t OpEditor[] = {
   { "quote-char",	OP_EDITOR_QUOTE_CHAR,		"\026" },
   { "kill-word",	OP_EDITOR_KILL_WORD,		"\027" },
   { "complete",		OP_EDITOR_COMPLETE,		"\t" },
+  { "complete-query",	OP_EDITOR_COMPLETE_QUERY,	"\024" },
   { "buffy-cycle",	OP_EDITOR_BUFFY_CYCLE,		" " },
   { "history-up",	OP_EDITOR_HISTORY_UP,		NULL },
   { "history-down",	OP_EDITOR_HISTORY_DOWN,		NULL },
@@ -336,3 +370,23 @@ struct binding_t OpEditor[] = {
 
 
 
+#ifdef HAVE_PGP
+struct binding_t OpPgp[] = {
+  { "verify-key",	OP_VERIFY_KEY,		"c" },
+  { "view-name",	OP_VIEW_ID,		"%" },
+  { NULL,		0,				NULL }
+};
+#endif /* HAVE_PGP */
+
+
+#ifdef MIXMASTER
+struct binding_t OpMix[] = {
+  { "accept",		OP_MIX_USE,	M_ENTER_S },
+  { "append",		OP_MIX_APPEND,	"a"       },
+  { "insert",		OP_MIX_INSERT,	"i"       },
+  { "delete",		OP_MIX_DELETE,  "d"	  },
+  { "chain-prev",	OP_MIX_CHAIN_PREV, "<left>" },
+  { "chain-next",	OP_MIX_CHAIN_NEXT, "<right>" },
+  { NULL, 		0, 		NULL }
+};
+#endif /* MIXMASTER */
