@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -13,55 +13,15 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */
 
 #include "mutt.h"
 
-
-
-#ifdef _PGPPATH
+#ifdef HAVE_PGP
 #include "pgp.h"
 #endif
 
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
 #include <sys/stat.h>
 #include <string.h>
 #include <ctype.h>
@@ -90,7 +50,7 @@ void mutt_edit_headers (const char *editor,
     return;
   }
 
-  mutt_write_rfc822_header (ofp, msg->env, NULL, 1);
+  mutt_write_rfc822_header (ofp, msg->env, NULL, 1, 0);
   fputc ('\n', ofp);	/* tie off the header. */
 
   /* now copy the body of the message. */
@@ -126,9 +86,21 @@ void mutt_edit_headers (const char *editor,
   mutt_free_list (&msg->env->userhdrs);
 
   /* Read the temp file back in */
-  ifp = fopen (path, "r");
-  ofp = safe_fopen (body, "w");
-  n = mutt_read_rfc822_header (ifp, NULL, 1);
+  if ((ifp = fopen (path, "r")) == NULL)
+  {
+    mutt_perror (path);
+    return;
+  }
+  
+  if ((ofp = safe_fopen (body, "w")) == NULL)
+  {
+    /* intentionally leak a possible temporary file here */
+    fclose (ifp);
+    mutt_perror (body);
+    return;
+  }
+  
+  n = mutt_read_rfc822_header (ifp, NULL, 1, 0);
   while ((i = fread (buffer, 1, sizeof (buffer), ifp)) > 0)
     fwrite (buffer, 1, i, ofp);
   fclose (ofp);
@@ -141,14 +113,12 @@ void mutt_edit_headers (const char *editor,
   mutt_free_envelope (&msg->env);
   msg->env = n;
 
-  msg->env->to = mutt_expand_aliases (msg->env->to);
-  msg->env->cc = mutt_expand_aliases (msg->env->cc);
-  msg->env->bcc = mutt_expand_aliases (msg->env->bcc);
-  msg->env->reply_to = mutt_expand_aliases (msg->env->reply_to);
-  msg->env->mail_followup_to = mutt_expand_aliases (msg->env->mail_followup_to);
+  mutt_expand_aliases_env (msg->env);
+  
+  /* search through the user defined headers added to see if either a 
+   * fcc: or attach-file: field was specified.  
+   */
 
-  /* search through the user defined headers added to see if either a * fcc:
-     or attach-file: field was specified.  */
   cur = msg->env->userhdrs;
   while (cur)
   {
@@ -206,7 +176,7 @@ void mutt_edit_headers (const char *editor,
 
 
 
-#ifdef _PGPPATH
+#ifdef HAVE_PGP
     else if (mutt_strncasecmp ("pgp:", cur->data, 4) == 0)
     {
       msg->pgp = mutt_parse_pgp_hdr (cur->data + 4, 0);
