@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1999-2000 Thomas Roessler <roessler@guug.de>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -13,7 +14,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */ 
 
 #define MAIN_C 1
@@ -32,14 +33,30 @@
 #include <sys/stat.h>
 #include <sys/utsname.h>
 
-const char Notice[] = "\
-Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>\n\
+#ifdef HAVE_GETOPT_H
+#include <getopt.h>
+#endif
+
+static const char *ReachingUs = N_("\
+To contact the developers, please mail to <mutt-dev@mutt.org>.\n\
+To report a bug, please use the muttbug utility.\n");
+
+static const char *Notice = N_("\
+Copyright (C) 1996-2000 Michael R. Elkins and others.\n\
 Mutt comes with ABSOLUTELY NO WARRANTY; for details type `mutt -vv'.\n\
 Mutt is free software, and you are welcome to redistribute it\n\
-under certain conditions; type `mutt -vv' for details.\n";
+under certain conditions; type `mutt -vv' for details.\n");
 
-const char Copyright[] = "\
-Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>\n\
+static const char *Copyright = N_("\
+Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>\n\
+Copyright (C) 1996-2000 Brandon Long <blong@fiction.net>\n\
+Copyright (C) 1997-2000 Thomas Roessler <roessler@guug.de>\n\
+Copyright (C) 1998-2000 Werner Koch <wk@isil.d.shuttle.de>\n\
+Copyright (C) 1999-2000 Brendan Cully <brendan@kublai.com>\n\
+Copyright (C) 1999-2000 Tommi Komulainen <Tommi.Komulainen@iki.fi>\n\
+\n\
+Lots of others not mentioned here contributed lots of code,\n\
+fixes, and suggestions.\n\
 \n\
     This program is free software; you can redistribute it and/or modify\n\
     it under the terms of the GNU General Public License as published by\n\
@@ -53,7 +70,25 @@ Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>\n\
 \n\
     You should have received a copy of the GNU General Public License\n\
     along with this program; if not, write to the Free Software\n\
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n";
+    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.\n\
+");
+#ifdef HAVE_PGP
+
+const char* ShaCopyright = N_("\n\
+SHA1 implementation Copyright (C) 1995-1997 Eric A. Young <eay@cryptsoft.com>\n\
+\n\
+    Redistribution and use in source and binary forms, with or without\n\
+    modification, are permitted under certain conditions.\n\
+\n\
+    The SHA1 implementation comes AS IS, and ANY EXPRESS OR IMPLIED\n\
+    WARRANTIES, including, but not limited to, the implied warranties of\n\
+    merchantability and fitness for a particular purpose ARE DISCLAIMED.\n\
+\n\
+    You should have received a copy of the full distribution terms\n\
+    along with this program; if not, write to the program's developers.\n\
+");
+#endif
+
 
 void mutt_exit (int code)
 {
@@ -63,8 +98,8 @@ void mutt_exit (int code)
 
 static void mutt_usage (void)
 {
-  printf ("Mutt %s (%s)\n", VERSION, ReleaseDate);
-  puts (
+  printf ("Mutt %s (%s)\n", MUTT_VERSION, ReleaseDate);
+  puts _(
 "usage: mutt [ -nRzZ ] [ -e <cmd> ] [ -F <file> ] [ -m <type> ] [ -f <file> ]\n\
        mutt [ -nx ] [ -e <cmd> ] [ -a <file> ] [ -F <file> ] [ -H <file> ] [ -i <file> ] [ -s <subj> ] [ -b <addr> ] [ -c <addr> ] <addr> [ ... ]\n\
        mutt [ -n ] [ -e <cmd> ] [ -F <file> ] -p\n\
@@ -98,8 +133,8 @@ static void show_version (void)
 {
   struct utsname uts;
 
-  printf ("Mutt %s (%s)\n", VERSION, ReleaseDate);
-  puts (Notice);
+  printf ("Mutt %s (%s)\n", MUTT_VERSION, ReleaseDate);
+  puts (_(Notice));
 
   uname (&uts);
 
@@ -117,7 +152,7 @@ static void show_version (void)
   printf (" [using slang %d]", SLANG_VERSION);
 #endif
 
-  puts ("\nCompile options:");
+  puts (_("\nCompile options:"));
 
 #ifdef DOMAIN
   printf ("DOMAIN=\"%s\"\n", DOMAIN);
@@ -125,12 +160,15 @@ static void show_version (void)
   puts ("-DOMAIN");
 #endif
 
-  puts (
-#ifdef HIDDEN_HOST
-	"+HIDDEN_HOST  "
+#ifdef DEBUG
+  puts ("+DEBUG");
 #else
-	"-HIDDEN_HOST  "
+  puts ("-DEBUG");
 #endif
+  
+
+  
+  puts (
 
 #ifdef HOMESPOOL
 	"+HOMESPOOL  "
@@ -162,8 +200,25 @@ static void show_version (void)
 	"-USE_FLOCK"
 #endif
 	);
-
   puts (
+#ifdef USE_IMAP
+        "+USE_IMAP  "
+#else
+        "-USE_IMAP  "
+#endif
+
+#ifdef USE_GSS
+	"+USE_GSS  "
+#else
+	"-USE_GSS  "
+#endif
+
+#ifdef USE_SSL
+	"+USE_SSL  "
+#else
+	"-USE_SSL  "
+#endif
+
 #ifdef USE_POP
 	"+USE_POP  "
 #else
@@ -176,44 +231,49 @@ static void show_version (void)
 	"-HAVE_REGCOMP  "
 #endif
 
-#ifdef USE_GNU_RX
-	"+USE_GNU_RX  "
+#ifdef USE_GNU_REGEX
+	"+USE_GNU_REGEX  "
 #else
-	"-USE_GNU_RX  "
+	"-USE_GNU_REGEX  "
 #endif
 
+	"\n"
+	
 #ifdef HAVE_COLOR
 	"+HAVE_COLOR  "
 #else
 	"-HAVE_COLOR  "
 #endif
 
-#ifdef _PGPPATH
-#ifdef HAVE_PGP5
-	"+HAVE_PGP5  "
-#endif
-#ifdef HAVE_PGP2
-	"+HAVE_PGP2  "
-#endif
+#ifdef HAVE_PGP
+	"+HAVE_PGP  "
+#else
+	"-HAVE_PGP  "
 #endif
 
 #ifdef BUFFY_SIZE
-	"+BUFFY_SIZE  "
+	"+BUFFY_SIZE "
 #else
-	"-BUFFY_SIZE  "
+	"-BUFFY_SIZE "
 #endif
-
 #ifdef EXACT_ADDRESS
 	"+"
 #else
 	"-"
 #endif
-	"EXACT_ADDRESS"
+	"EXACT_ADDRESS  "
+#ifdef ENABLE_NLS
+	"+"
+#else
+	"-"
+#endif
+	"ENABLE_NLS"
 	);
 
   printf ("SENDMAIL=\"%s\"\n", SENDMAIL);
   printf ("MAILPATH=\"%s\"\n", MAILPATH);
   printf ("SHAREDIR=\"%s\"\n", SHAREDIR);
+  printf ("SYSCONFDIR=\"%s\"\n", SYSCONFDIR);
 
 #ifdef ISPELL
   printf ("ISPELL=\"%s\"\n", ISPELL);
@@ -221,17 +281,7 @@ static void show_version (void)
   puts ("-ISPELL");
 #endif
 
-#ifdef _PGPPATH
-  printf ("_PGPPATH=\"%s\"\n", _PGPPATH);
-# ifdef _PGPV2PATH
-  printf ("_PGPV2PATH=\"%s\"\n", _PGPV2PATH);
-# endif
-# ifdef _PGPV3PATH
-  printf ("_PGPV3PATH=\"%s\"\n", _PGPV3PATH);
-# endif
-#endif
-
-  puts ("\nMail bug reports along with this output to <mutt-users@cs.hmc.edu>.");
+  puts(_(ReachingUs));
 
   exit (0);
 }
@@ -243,12 +293,20 @@ static void start_curses (void)
 #ifdef USE_SLANG_CURSES
   SLtt_Ignore_Beep = 1; /* don't do that #*$@^! annoying visual beep! */
   SLsmg_Display_Eight_Bit = 128; /* characters above this are printable */
+#else
+  /* should come before initscr() so that ncurses 4.2 doesn't try to install
+     its own SIGWINCH handler */
+  mutt_signal_init ();
 #endif
   if (initscr () == NULL)
   {
-    puts ("Error initializing terminal.");
+    puts _("Error initializing terminal.");
     exit (1);
   }
+#ifdef USE_SLANG_CURSES
+  /* slang requires the signal handlers to be set after initializing */
+  mutt_signal_init ();
+#endif
   ci_start_color ();
   keypad (stdscr, TRUE);
   cbreak ();
@@ -256,7 +314,6 @@ static void start_curses (void)
 #if HAVE_TYPEAHEAD
   typeahead (-1);       /* simulate smooth scrolling */
 #endif
-  mutt_signal_init ();
 #if HAVE_META
   meta (stdscr, TRUE);
 #endif
@@ -268,26 +325,6 @@ static void start_curses (void)
 #define M_RO      (1<<3)	/* -R */
 #define M_SELECT  (1<<4)	/* -y */
 
-#ifdef USE_SLANG_CURSES
-
-void check_slang(void)
-{
-  char *term = getenv("TERM");
-
-  /* slang < 1.2x */
-  
-  if(SLANG_VERSION < 12000)
-  {
-    if(term && strlen(term) > 100)
-    {
-      fputs("Sorry, your TERM variable's value is too long.\n", stderr);
-      exit(1);
-    }
-  }
-}
-
-#endif
-
 int main (int argc, char **argv)
 {
   char folder[_POSIX_PATH_MAX] = "";
@@ -302,44 +339,35 @@ int main (int argc, char **argv)
   int flags = 0;
   int version = 0;
   int i;
+  int explicit_folder = 0;
   extern char *optarg;
   extern int optind;
 
-
-#ifdef USE_SETGID
-  /* Determine the user's default gid and the gid to use for locking the spool
-   * mailbox on those systems which require setgid "mail" to write to the
-   * directory.  This function also resets the gid to "normal" since the
-   * effective gid will be "mail" when we start (Mutt attempts to run
-   * non-setgid whenever possible to reduce the possibility of security holes).
-   */
-
-  /* Get the default gid for the user */
-  UserGid = getgid ();
-
-  /* it is assumed that we are setgid to the correct gid to begin with */
-  MailGid = getegid ();
-
-  /* reset the effective gid to the normal gid */
-  if (SETEGID (UserGid) != 0)
+  /* sanity check against stupid administrators */
+  
+  if(getegid() != getgid())
   {
-    perror ("setegid");
-    exit (0);
+    fprintf(stderr, "%s: I don't want to run with privileges!\n",
+	    argv[0]);
+    exit(1);
   }
+
+#ifdef ENABLE_NLS
+  /* FIXME what about init.c:1439 ? */
+  setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, MUTTLOCALEDIR);
+  textdomain (PACKAGE);
 #endif
 
-#ifdef USE_SLANG_CURSES
-  
-  check_slang();
-  
-#endif
-  
-  SRAND (time (NULL));
   setlocale (LC_CTYPE, "");
+
+  mutt_error = mutt_nocurses_error;
+  SRAND (time (NULL));
   umask (077);
 
   memset (Options, 0, sizeof (Options));
-
+  memset (QuadOptions, 0, sizeof (QuadOptions));
+  
   while ((i = getopt (argc, argv, "a:b:F:f:c:d:e:H:s:i:hm:npRvxyzZ")) != EOF)
     switch (i)
     {
@@ -348,11 +376,12 @@ int main (int argc, char **argv)
 	break;
 
       case 'F':
-	strfcpy (Muttrc, optarg, sizeof (Muttrc));
+	mutt_str_replace (&Muttrc, optarg);
 	break;
 
       case 'f':
 	strfcpy (folder, optarg, sizeof (folder));
+        explicit_folder = 1;
 	break;
 
       case 'b':
@@ -370,9 +399,9 @@ int main (int argc, char **argv)
       case 'd':
 #ifdef DEBUG
 	debuglevel = atoi (optarg);
-	printf ("Debugging at level %d.\n", debuglevel);
+	printf (_("Debugging at level %d.\n"), debuglevel);
 #else
-	printf ("DEBUG was not defined during compilation.  Ignored.\n");
+	printf _("DEBUG was not defined during compilation.  Ignored.\n");
 #endif
 	break;
 
@@ -441,8 +470,12 @@ int main (int argc, char **argv)
       show_version ();
       break;
     default:
-      printf ("Mutt %s (%s)\n", VERSION, ReleaseDate);
-      puts (Copyright);
+      printf ("Mutt %s (%s)\n", MUTT_VERSION, ReleaseDate);
+      puts (_(Copyright));
+#ifdef HAVE_PGP
+      puts(_(ShaCopyright));
+#endif
+      puts (_(ReachingUs));
       exit (0);
   }
 
@@ -453,10 +486,8 @@ int main (int argc, char **argv)
     sendflags = SENDBATCH;
   }
 
-  /*
-   * This must come before mutt_init() because curses needs to be started
-   * before calling the init_pair() function to set the color scheme.
-   */
+  /* This must come before mutt_init() because curses needs to be started
+     before calling the init_pair() function to set the color scheme.  */
   if (!option (OPTNOCURSES))
     start_curses ();
 
@@ -471,6 +502,31 @@ int main (int argc, char **argv)
   {
     SETCOLOR (MT_COLOR_NORMAL);
     clear ();
+    mutt_error = mutt_curses_error;
+  }
+
+  /* Create the Maildir directory if it doesn't exist. */
+  if (!option (OPTNOCURSES) && Maildir)
+  {
+    struct stat sb;
+    char fpath[_POSIX_PATH_MAX];
+    char msg[STRING];
+
+    strfcpy (fpath, Maildir, sizeof (fpath));
+    mutt_expand_path (fpath, sizeof (fpath));
+#ifdef USE_IMAP
+    /* we're not connected yet - skip mail folder creation */
+    if (!mx_is_imap (fpath))
+#endif
+    if (stat (fpath, &sb) == -1 && errno == ENOENT)
+    {
+      snprintf (msg, sizeof (msg), _("%s does not exist. Create it?"), Maildir);
+      if (mutt_yesorno (msg, 1) == 1)
+      {
+	if (mkdir (fpath, 0700) == -1 && errno != EEXIST)
+	  mutt_error ( _("Can't create %s: %s."), Maildir, strerror (errno));
+      }
+    }
   }
 
   if (sendflags & SENDPOSTPONED)
@@ -503,11 +559,11 @@ int main (int argc, char **argv)
       for (i = optind; i < argc; i++)
 	msg->env->to = rfc822_parse_adrlist (msg->env->to, argv[i]);
 
-      if (!msg->env->to && !msg->env->cc)
+      if (option (OPTAUTOEDIT) && !msg->env->to && !msg->env->cc)
       {
 	if (!option (OPTNOCURSES))
 	  mutt_endwin (NULL);
-	fputs ("No recipients specified.\n", stderr);
+	fputs (_("No recipients specified.\n"), stderr);
 	exit (1);
       }
 
@@ -519,20 +575,27 @@ int main (int argc, char **argv)
 
     if (infile)
     {
-      if (strcmp ("-", infile) == 0)
+      if (mutt_strcmp ("-", infile) == 0)
 	fin = stdin;
-      else if ((fin = fopen (infile, "r")) == NULL)
+      else
       {
-	if (!option (OPTNOCURSES))
-	  mutt_endwin (NULL);
-	perror (infile);
-	exit (1);
+	char path[_POSIX_PATH_MAX];
+
+	strfcpy (path, infile, sizeof (path));
+	mutt_expand_path (path, sizeof (path));
+	if ((fin = fopen (path, "r")) == NULL)
+	{
+	  if (!option (OPTNOCURSES))
+	    mutt_endwin (NULL);
+	  perror (path);
+	  exit (1);
+	}
       }
       mutt_mktemp (buf);
       tempfile = safe_strdup (buf);
 
       if (draftFile)
-	msg->env = mutt_read_rfc822_header (fin, NULL);
+	msg->env = mutt_read_rfc822_header (fin, NULL, 1, 0);
 
       if (tempfile)
       {
@@ -544,7 +607,7 @@ int main (int argc, char **argv)
 	    mutt_endwin (NULL);
 	  perror (tempfile);
 	  fclose (fin);
-	  free (tempfile);
+	  FREE (&tempfile);
 	  exit (1);
 	}
 
@@ -564,16 +627,16 @@ int main (int argc, char **argv)
       {
 	if (a)
 	{
-	  a->next = mutt_make_attach (t->data);
+	  a->next = mutt_make_file_attach (t->data);
 	  a = a->next;
 	}
 	else
-	  msg->content = a = mutt_make_attach (t->data);
+	  msg->content = a = mutt_make_file_attach (t->data);
 	if (!a)
 	{
 	  if (!option (OPTNOCURSES))
 	    mutt_endwin (NULL);
-	  fprintf (stderr, "%s: unable to attach file.\n", t->data);
+	  fprintf (stderr, _("%s: unable to attach file.\n"), t->data);
 	  mutt_free_list (&attach);
 	  exit (1);
 	}
@@ -593,7 +656,7 @@ int main (int argc, char **argv)
     {
       if (!mutt_buffy_check (0))
       {
-	mutt_endwin ("No mailbox with new mail.");
+	mutt_endwin _("No mailbox with new mail.");
 	exit (1);
       }
       folder[0] = 0;
@@ -601,6 +664,10 @@ int main (int argc, char **argv)
     }
     else if (flags & M_SELECT)
     {
+      if (!Incoming) {
+	mutt_endwin _("No incoming mailboxes defined.");
+	exit (1);
+      }
       folder[0] = 0;
       mutt_select_file (folder, sizeof (folder), 1);
       if (!folder[0])
@@ -611,9 +678,11 @@ int main (int argc, char **argv)
     }
 
     if (!folder[0])
-      strfcpy (folder, Spoolfile, sizeof (folder));
+      strfcpy (folder, NONULL(Spoolfile), sizeof (folder));
     mutt_expand_path (folder, sizeof (folder));
 
+    mutt_str_replace (&LastFolder, folder);
+
     if (flags & M_IGNORE)
     {
       struct stat st;
@@ -627,20 +696,21 @@ int main (int argc, char **argv)
 
       if (st.st_size == 0)
       {
-	mutt_endwin ("Mailbox is empty.");
+	mutt_endwin _("Mailbox is empty.");
 	exit (1);
       }
     }
 
     mutt_folder_hook (folder);
 
-    if ((Context = mx_open_mailbox (folder, ((flags & M_RO) || option (OPTREADONLY)) ? M_READONLY : 0, NULL)) != NULL)
+    if((Context = mx_open_mailbox (folder, ((flags & M_RO) || option (OPTREADONLY)) ? M_READONLY : 0, NULL))
+       || !explicit_folder)
     {
       mutt_index_menu ();
-      mutt_endwin (NULL);
+      if (Context)
+	safe_free ((void **)&Context);
     }
-    else
-      mutt_endwin (Errorbuf);
+    mutt_endwin (Errorbuf);
   }
 
   exit (0);
