@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -13,11 +13,15 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */ 
 
 /* This file contains code to parse ``mbox'' and ``mmdf'' style mailboxes */
 
+/* OS/2's "kendra" mail folder format is also supported.  It's a slightly
+ * modified version of MMDF.
+ */
+
 #include "mutt.h"
 #include "mailbox.h"
 #include "mx.h"
@@ -36,64 +40,13 @@
 /* struct used by mutt_sync_mailbox() to store new offsets */
 struct m_update_t
 {
+  short valid;
   long hdr;
   long body;
+  long lines;
+  long length;
 };
 
-#ifdef USE_DOTLOCK
-/*
- * Determine whether or not to use a dotlock to lock the indicated file.
- * On some systems, the spool directory is not world-writable.  If it is
- * group-writable, we might need to be setgid() to write the lock.  If not
- * group-writable, then we assume that fcntl() locking is enough and skip
- * the dotlocking.
- *
- * return values:
- *	2	need to be setgid to dotlock
- *	1	can use a dotlock
- *	0	don't use a dotlock
- *	-1	error
- */
-static int can_dotlock (const char *path)
-{
-  char tmp[_POSIX_PATH_MAX];
-  char *p;
-#ifdef USE_SETGID
-  struct stat sb;
-#endif
-
-  strfcpy (tmp, path, sizeof (tmp));
-  if ((p = strrchr (tmp, '/')))
-    *p = 0;
-  else
-    strfcpy (tmp, ".", sizeof (tmp)); /* use current directory */
-
-  if (access (tmp, W_OK) == 0) return 1;
-
-#ifdef USE_SETGID
-  if (stat (tmp, &sb) == 0)
-  {
-    if ((sb.st_mode & S_IWGRP) == S_IWGRP)
-    {
-      /* can dotlock, but need to be setgid */
-      if (sb.st_gid == MailGid)
-	return (2);
-      else
-      {
-	mutt_error ("Need to be running setgid %d to lock mailbox!", sb.st_gid);
-	return (-1);
-      }
-    }
-  }
-#endif
-
-  if (mutt_yesorno ("Can't dotlock mailbox, continue anyway?", 0) == 1)
-    return 0;
-
-  return (-1);
-}
-#endif
-
 /* parameters:
  * ctx - context to lock
  * excl - exclusive lock?
@@ -101,36 +54,16 @@ static int can_dotlock (const char *path)
  */
 int mbox_lock_mailbox (CONTEXT *ctx, int excl, int retry)
 {
-  int r = 0;
+  int r;
 
-#ifdef USE_DOTLOCK
-  r = can_dotlock (ctx->path);
-
-  if (r == -1)
-    return (-1);
-#ifdef USE_SETGID
-  else if (r == 2)
-  {
-    /* need to be setgid to lock the mailbox */
-    if (SETEGID (MailGid) != 0)
-    {
-      mutt_perror ("setegid");
-      return (-1);
-    }
-
-    ctx->setgid = 1;
-  }
-#endif /* USE_SETGID */
-#endif /* USE_DOTLOCK */
-
-  if ((r = mx_lock_file (ctx->path, fileno (ctx->fp), excl, r, retry)) == 0)
+  if ((r = mx_lock_file (ctx->path, fileno (ctx->fp), excl, 1, retry)) == 0)
     ctx->locked = 1;
-
-#ifdef USE_SETGID
-  if (ctx->setgid)
-    SETEGID (UserGid);
-#endif
-
+  else if (retry && !excl)
+  {
+    ctx->readonly = 1;
+    return 0;
+  }
+  
   return (r);
 }
 
@@ -140,21 +73,8 @@ void mbox_unlock_mailbox (CONTEXT *ctx)
   {
     fflush (ctx->fp);
 
-#ifdef USE_SETGID
-    if (ctx->setgid)
-      SETEGID (MailGid);
-#endif /* USE_SETGID */
-
-    mx_unlock_file (ctx->path, fileno (ctx->fp));
+    mx_unlock_file (ctx->path, fileno (ctx->fp), 1);
     ctx->locked = 0;
-
-#ifdef USE_SETGID
-    if (ctx->setgid)
-    {
-      SETEGID (UserGid);
-      ctx->setgid = 0;
-    }
-#endif
   }
 }
 
@@ -162,6 +82,7 @@ int mmdf_parse_mailbox (CONTEXT *ctx)
 {
   char buf[HUGE_STRING];
   char return_path[LONG_STRING];
+  int count = 0;
   int lines;
   time_t t, tz;
   long loc, tmploc;
@@ -190,17 +111,24 @@ int mmdf_parse_mailbox (CONTEXT *ctx)
 
   /* precompute the local timezone to speed up calculation of the
      received time */
-  tz = mutt_local_tz ();
+  tz = mutt_local_tz (0);
 
   buf[sizeof (buf) - 1] = 0;
+  
   FOREVER
   {
     if (fgets (buf, sizeof (buf) - 1, ctx->fp) == NULL)
       break;
 
-    if (strcmp (buf, MMDF_SEP) == 0)
+    if (mutt_strcmp (buf, ctx->magic == M_MMDF ? MMDF_SEP : KENDRA_SEP) == 0)
     {
       loc = ftell (ctx->fp);
+      
+      count++;
+      if (!ctx->quiet && ReadInc && ((count % ReadInc == 0) || count == 1))
+	mutt_message (_("Reading %s... %d (%d%%)"), ctx->path, count,
+		      loc / (ctx->size / 100 + 1));
+
 
       if (ctx->msgcount == ctx->hdrmax)
 	mx_alloc_memory (ctx);
@@ -218,11 +146,18 @@ int mmdf_parse_mailbox (CONTEXT *ctx)
       t = is_from (buf, return_path, sizeof (return_path));
 
       if (!t)
-	fseek (ctx->fp, loc, 0);
+      {
+	if (fseek (ctx->fp, loc, SEEK_SET) != 0)
+	{
+	  dprint (1, (debugfile, "mmdf_parse_mailbox: fseek() failed\n"));
+	  mutt_error _("Mailbox is corrupt!");
+	  return (-1);
+	}
+      } 
       else
-	hdr->received = t + tz;
+	hdr->received = t - tz;
 
-      hdr->env = mutt_read_rfc822_header (ctx->fp, hdr);
+      hdr->env = mutt_read_rfc822_header (ctx->fp, hdr, 0, 0);
 
       loc = ftell (ctx->fp);
 
@@ -230,13 +165,14 @@ int mmdf_parse_mailbox (CONTEXT *ctx)
       {
 	tmploc = loc + hdr->content->length;
 
-	if (tmploc < ctx->size)
+	if (0 < tmploc && tmploc < ctx->size)
 	{
-	  fseek (ctx->fp, tmploc, 0);
-	  if (fgets (buf, sizeof (buf) - 1, ctx->fp) == NULL ||
-	      strcmp (MMDF_SEP, buf) != 0)
+	  if (fseek (ctx->fp, tmploc, SEEK_SET) != 0 ||
+	      fgets (buf, sizeof (buf) - 1, ctx->fp) == NULL ||
+	      mutt_strcmp (ctx->magic == M_MMDF ? MMDF_SEP : KENDRA_SEP, buf) != 0)
 	  {
-	    fseek (ctx->fp, loc, 0);
+	    if (fseek (ctx->fp, loc, SEEK_SET) != 0)
+	      dprint (1, (debugfile, "mmdf_parse_mailbox: fseek() failed\n"));
 	    hdr->content->length = -1;
 	  }
 	}
@@ -254,7 +190,7 @@ int mmdf_parse_mailbox (CONTEXT *ctx)
 	  if (fgets (buf, sizeof (buf) - 1, ctx->fp) == NULL)
 	    break;
 	  lines++;
-	} while (strcmp (buf, MMDF_SEP) != 0);
+	} while (mutt_strcmp (buf, ctx->magic == M_MMDF ? MMDF_SEP : KENDRA_SEP) != 0);
 
 	hdr->lines = lines;
 	hdr->content->length = loc - hdr->content->offset;
@@ -267,11 +203,13 @@ int mmdf_parse_mailbox (CONTEXT *ctx)
 	hdr->env->from = rfc822_cpy_adr (hdr->env->return_path);
 
       mx_update_context (ctx);
+      if(ctx->magic == M_KENDRA && feof(ctx->fp))
+	break;
     }
     else
     {
       dprint (1, (debugfile, "mmdf_parse_mailbox: corrupt mailbox!\n"));
-      mutt_error ("Mailbox is corrupt!");
+      mutt_error _("Mailbox is corrupt!");
       return (-1);
     }
   }
@@ -321,7 +259,7 @@ int mbox_parse_mailbox (CONTEXT *ctx)
 
   /* precompute the local timezone to speed up calculation of the
      date received */
-  tz = mutt_local_tz ();
+  tz = mutt_local_tz (0);
 
   loc = ftell (ctx->fp);
   while (fgets (buf, sizeof (buf), ctx->fp) != NULL)
@@ -346,18 +284,18 @@ int mbox_parse_mailbox (CONTEXT *ctx)
       count++;
 
       if (!ctx->quiet && ReadInc && ((count % ReadInc == 0) || count == 1))
-	mutt_message ("Reading %s... %d (%d%%)", ctx->path, count,
+	mutt_message (_("Reading %s... %d (%d%%)"), ctx->path, count,
 		      ftell (ctx->fp) / (ctx->size / 100 + 1));
 
       if (ctx->msgcount == ctx->hdrmax)
 	mx_alloc_memory (ctx);
       
       curhdr = ctx->hdrs[ctx->msgcount] = mutt_new_header ();
-      curhdr->received = t + tz;
+      curhdr->received = t - tz;
       curhdr->offset = loc;
       curhdr->index = ctx->msgcount;
 	
-      curhdr->env = mutt_read_rfc822_header (ctx->fp, curhdr);
+      curhdr->env = mutt_read_rfc822_header (ctx->fp, curhdr, 0, 0);
 
       /* if we know how long this message is, either just skip over the body,
        * or if we don't know how many lines there are, count them now (this will
@@ -370,19 +308,22 @@ int mbox_parse_mailbox (CONTEXT *ctx)
 	loc = ftell (ctx->fp);
 	tmploc = loc + curhdr->content->length + 1;
 
-	if (tmploc < ctx->size)
+	if (0 < tmploc && tmploc < ctx->size)
 	{
 	  /*
 	   * check to see if the content-length looks valid.  we expect to
 	   * to see a valid message separator at this point in the stream
 	   */
-	  fseek (ctx->fp, tmploc, 0);
-	  if (fgets (buf, sizeof (buf), ctx->fp) == NULL ||
-	      strncmp ("From ", buf, 5) != 0)
+	  if (fseek (ctx->fp, tmploc, SEEK_SET) != 0 ||
+	      fgets (buf, sizeof (buf), ctx->fp) == NULL ||
+	      mutt_strncmp ("From ", buf, 5) != 0)
 	  {
 	    dprint (1, (debugfile, "mbox_parse_mailbox: bad content-length in message %d (cl=%ld)\n", curhdr->index, curhdr->content->length));
 	    dprint (1, (debugfile, "\tLINE: %s", buf));
-	    fseek (ctx->fp, loc, 0); /* nope, return the previous position */
+	    if (fseek (ctx->fp, loc, SEEK_SET) != 0) /* nope, return the previous position */
+	    {
+	      dprint (1, (debugfile, "mbox_parse_mailbox: fseek() failed\n"));
+	    }
 	    curhdr->content->length = -1;
 	  }
 	}
@@ -404,7 +345,8 @@ int mbox_parse_mailbox (CONTEXT *ctx)
 	    int cl = curhdr->content->length;
 
 	    /* count the number of lines in this message */
-	    fseek (ctx->fp, loc, 0);
+	    if (fseek (ctx->fp, loc, SEEK_SET) != 0)
+	      dprint (1, (debugfile, "mbox_parse_mailbox: fseek() failed\n"));
 	    while (cl-- > 0)
 	    {
 	      if (fgetc (ctx->fp) == '\n')
@@ -413,7 +355,8 @@ int mbox_parse_mailbox (CONTEXT *ctx)
 	  }
 
 	  /* return to the offset of the next message separator */
-	  fseek (ctx->fp, tmploc, 0);
+	  if (fseek (ctx->fp, tmploc, SEEK_SET) != 0)
+	    dprint (1, (debugfile, "mbox_parse_mailbox: fseek() failed\n"));
 	}
       }
 
@@ -476,7 +419,7 @@ int mbox_open_mailbox (CONTEXT *ctx)
 
   if (ctx->magic == M_MBOX)
     rc = mbox_parse_mailbox (ctx);
-  else if (ctx->magic == M_MMDF)
+  else if (ctx->magic == M_MMDF || ctx->magic == M_KENDRA)
     rc = mmdf_parse_mailbox (ctx);
   else
     rc = -1;
@@ -643,7 +586,7 @@ int mbox_check_mailbox (CONTEXT *ctx, int *index_hint)
 	  /* we couldn't lock the mailbox, but nothing serious happened:
 	   * probably the new mail arrived: no reason to wait till we can
 	   * parse it: we'll get it on the next pass
-	   *  */
+	   */
 	  return (M_LOCKED);
 	}
 	unlock = 1;
@@ -655,13 +598,16 @@ int mbox_check_mailbox (CONTEXT *ctx, int *index_hint)
        * see the message separator at *exactly* what used to be the end of the
        * folder.
        */
-      fseek (ctx->fp, ctx->size, 0);
+      if (fseek (ctx->fp, ctx->size, SEEK_SET) != 0)
+	dprint (1, (debugfile, "mbox_check_mailbox: fseek() failed\n"));
       if (fgets (buffer, sizeof (buffer), ctx->fp) != NULL)
       {
-	if ((ctx->magic == M_MBOX && strncmp ("From ", buffer, 5) == 0) ||
-	    (ctx->magic == M_MMDF && strcmp (MMDF_SEP, buffer) == 0))
+	if ((ctx->magic == M_MBOX && mutt_strncmp ("From ", buffer, 5) == 0) ||
+	    (ctx->magic == M_MMDF && mutt_strcmp (MMDF_SEP, buffer) == 0) ||
+	    (ctx->magic == M_KENDRA && mutt_strcmp(KENDRA_SEP, buffer) == 0))
 	{
-	  fseek (ctx->fp, ctx->size, 0);
+	  if (fseek (ctx->fp, ctx->size, SEEK_SET) != 0)
+	    dprint (1, (debugfile, "mbox_check_mailbox: fseek() failed\n"));
 	  if (ctx->magic == M_MBOX)
 	    mbox_parse_mailbox (ctx);
 	  else
@@ -711,7 +657,7 @@ int mbox_check_mailbox (CONTEXT *ctx, int *index_hint)
   mbox_unlock_mailbox (ctx);
   mx_fastclose_mailbox (ctx);
   mutt_unblock_signals ();
-  mutt_error ("Mailbox was corrupted!");
+  mutt_error _("Mailbox was corrupted!");
   return (-1);
 }
 
@@ -719,18 +665,20 @@ int mbox_check_mailbox (CONTEXT *ctx, int *index_hint)
  *	0	success
  *	-1	failure
  */
-int mbox_sync_mailbox (CONTEXT *ctx)
+int mbox_sync_mailbox (CONTEXT *ctx, int *index_hint)
 {
   char tempfile[_POSIX_PATH_MAX];
-  char buf[16];
+  char buf[32];
   int i, j, save_sort = SORT_ORDER;
-  int need_sort = 0; /* flag to restort mailbox if new mail arrives */
-  int first;	/* first message to be written */
+  int rc = -1;
+  int need_sort = 0; /* flag to resort mailbox if new mail arrives */
+  int first = -1;	/* first message to be written */
   long offset;	/* location in mailbox to write changed messages */
   struct stat statbuf;
   struct utimbuf utimebuf;
   struct m_update_t *newOffset = NULL;
-  FILE *fp;
+  struct m_update_t *oldOffset = NULL;
+  FILE *fp = NULL;
 
   /* sort message by their position in the mailbox on disk */
   if (Sort != SORT_ORDER)
@@ -746,7 +694,7 @@ int mbox_sync_mailbox (CONTEXT *ctx)
   if ((ctx->fp = freopen (ctx->path, "r+", ctx->fp)) == NULL)
   {
     mx_fastclose_mailbox (ctx);
-    mutt_error ("Fatal error!  Could not reopen mailbox.");
+    mutt_error _("Fatal error!  Could not reopen mailbox!");
     return (-1);
   }
 
@@ -755,15 +703,16 @@ int mbox_sync_mailbox (CONTEXT *ctx)
   if (mbox_lock_mailbox (ctx, 1, 1) == -1)
   {
     mutt_unblock_signals ();
-    mutt_error ("Unable to lock mailbox!");
+    mutt_error _("Unable to lock mailbox!");
     goto bail;
   }
 
   /* Check to make sure that the file hasn't changed on disk */
-  if ((i = mbox_check_mailbox (ctx, NULL)) == M_NEW_MAIL ||  i == M_REOPENED)
+  if ((i = mbox_check_mailbox (ctx, index_hint)) == M_NEW_MAIL ||  i == M_REOPENED)
   {
     /* new mail arrived, or mailbox reopened */
     need_sort = i;
+    rc = i;
     goto bail;
   }
   else if (i < 0)
@@ -778,14 +727,21 @@ int mbox_sync_mailbox (CONTEXT *ctx)
   if ((i = open (tempfile, O_WRONLY | O_EXCL | O_CREAT, 0600)) == -1 ||
       (fp = fdopen (i, "w")) == NULL)
   {
-    mutt_error ("Could not create temporary file!");
+    if (-1 != i)
+    {
+      close (i);
+      unlink (tempfile);
+    }
+    mutt_error _("Could not create temporary file!");
+    sleep (5);
     goto bail;
   }
 
   /* find the first deleted/changed message.  we save a lot of time by only
    * rewriting the mailbox from the point where it has actually changed.
    */
-  for (i = 0 ; i < ctx->msgcount && !ctx->hdrs[i]->deleted && !ctx->hdrs[i]->changed ; i++)
+  for (i = 0 ; i < ctx->msgcount && !ctx->hdrs[i]->deleted && 
+               !ctx->hdrs[i]->changed && !ctx->hdrs[i]->attach_del; i++)
     ;
   if (i == ctx->msgcount)
   { 
@@ -793,37 +749,70 @@ int mbox_sync_mailbox (CONTEXT *ctx)
      * messages were found to be changed or deleted.  This should
      * never happen, is we presume it is a bug in mutt.
      */
-    mutt_error ("sync: mbox modified, but no modified messages! (report this bug)");
+    mutt_error _("sync: mbox modified, but no modified messages! (report this bug)");
     sleep(5); /* the mutt_error /will/ get cleared! */
     dprint(1, (debugfile, "mbox_sync_mailbox(): no modified messages.\n"));
+    unlink (tempfile);
     goto bail;
   }
 
+    /* save the index of the first changed/deleted message */
+  first = i; 
+  /* where to start overwriting */
+  offset = ctx->hdrs[i]->offset; 
 
-  first = i; /* save the index of the first changed/deleted message */
-  offset = ctx->hdrs[i]->offset; /* where to start overwriting */
   /* the offset stored in the header does not include the MMDF_SEP, so make
    * sure we seek to the correct location
    */
   if (ctx->magic == M_MMDF)
-    offset -= strlen (MMDF_SEP);
-
+    offset -= (sizeof MMDF_SEP - 1);
+  else if (ctx->magic == M_KENDRA)
+    offset -= (sizeof KENDRA_SEP - 1);
+  
   /* allocate space for the new offsets */
   newOffset = safe_calloc (ctx->msgcount - first, sizeof (struct m_update_t));
+  oldOffset = safe_calloc (ctx->msgcount - first, sizeof (struct m_update_t));
 
   for (i = first, j = 0; i < ctx->msgcount; i++)
   {
+    /*
+     * back up some information which is needed to restore offsets when
+     * something fails.
+     */
+    
+    oldOffset[i-first].valid  = 1;
+    oldOffset[i-first].hdr    = ctx->hdrs[i]->offset;
+    oldOffset[i-first].body   = ctx->hdrs[i]->content->offset;
+    oldOffset[i-first].lines  = ctx->hdrs[i]->lines;
+    oldOffset[i-first].length = ctx->hdrs[i]->content->length;
+    
     if (! ctx->hdrs[i]->deleted)
     {
       j++;
-      if (!ctx->quiet && WriteInc && ((j % WriteInc) == 0 || j == 1))
-	mutt_message ("Writing messages... %d (%d%%)", j,
+      if (!ctx->quiet && WriteInc && ((i % WriteInc) == 0 || j == 1))
+	mutt_message (_("Writing messages... %d (%d%%)"), i,
 		      ftell (ctx->fp) / (ctx->size / 100 + 1));
 
       if (ctx->magic == M_MMDF)
       {
 	if (fputs (MMDF_SEP, fp) == EOF)
+	{
+	  mutt_perror (tempfile);
+	  sleep (5);
+	  unlink (tempfile);
 	  goto bail;
+	}
+	  
+      }
+      else if (ctx->magic == M_KENDRA)
+      {
+	if (fputs (KENDRA_SEP, fp) == EOF)
+	{
+	  mutt_perror (tempfile);
+	  sleep (5);
+	  unlink (tempfile);
+	  goto bail;
+	}
       }
 
       /* save the new offset for this message.  we add `offset' because the
@@ -832,8 +821,13 @@ int mbox_sync_mailbox (CONTEXT *ctx)
        */
       newOffset[i - first].hdr = ftell (fp) + offset;
 
-      if (mutt_copy_message (fp, ctx, ctx->hdrs[i], 0, CH_FROM | CH_UPDATE | CH_UPDATE_LEN) == -1)
+      if (mutt_copy_message (fp, ctx, ctx->hdrs[i], M_CM_UPDATE, CH_FROM | CH_UPDATE | CH_UPDATE_LEN) == -1)
+      {
+	mutt_perror (tempfile);
+	sleep (5);
+	unlink (tempfile);
 	goto bail;
+      }
 
       /* Since messages could have been deleted, the offsets stored in memory
        * will be wrong, so update what we can, which is the offset of this
@@ -844,21 +838,54 @@ int mbox_sync_mailbox (CONTEXT *ctx)
       newOffset[i - first].body = ftell (fp) - ctx->hdrs[i]->content->length + offset;
       mutt_free_body (&ctx->hdrs[i]->content->parts);
 
-      if (fputs (ctx->magic == M_MMDF ? MMDF_SEP : "\n", fp) == EOF)
-	goto bail;
+      switch(ctx->magic)
+      {
+	case M_MMDF: 
+	  if(fputs(MMDF_SEP, fp) == EOF) 
+	  {
+	    mutt_perror (tempfile);
+	    sleep (5);
+	    unlink (tempfile);
+	    goto bail; 
+	  }
+	  break;
+	case M_KENDRA:
+	  if(fputs(KENDRA_SEP, fp) == EOF)
+	  {
+	    mutt_perror (tempfile);
+	    sleep (5);
+	    unlink (tempfile);
+	    goto bail;
+	  }
+	  break;
+	default:
+	  if(fputs("\n", fp) == EOF) 
+	  {
+	    mutt_perror (tempfile);
+	    sleep (5);
+	    unlink (tempfile);
+	    goto bail;
+	  }
+      }
     }
   }
-
+  
   if (fclose (fp) != 0)
   {
-    dprint(1, (debugfile, "mutt_sync_mailbox(): fclose() returned non-zero.\n"));
+    fp = NULL;
+    dprint(1, (debugfile, "mbox_sync_mailbox: fclose() returned non-zero.\n"));
     unlink (tempfile);
+    mutt_perror (tempfile);
+    sleep (5);
     goto bail;
   }
+  fp = NULL;
 
   /* Save the state of this folder. */
   if (stat (ctx->path, &statbuf) == -1)
   {
+    mutt_perror (ctx->path);
+    sleep (5);
     unlink (tempfile);
     goto bail;
   }
@@ -869,32 +896,38 @@ int mbox_sync_mailbox (CONTEXT *ctx)
     mx_fastclose_mailbox (ctx);
     dprint (1, (debugfile, "mbox_sync_mailbox: unable to reopen temp copy of mailbox!\n"));
     mutt_perror (tempfile);
+    sleep (5);
     return (-1);
   }
 
-  fseek (ctx->fp, offset, 0);  /* seek the append location */
-  
-  /* do a sanity check to make sure the mailbox looks ok */
-  if (fgets (buf, sizeof (buf), ctx->fp) == NULL ||
-      (ctx->magic == M_MBOX && strncmp ("From ", buf, 5) != 0) ||
-      (ctx->magic == M_MMDF && strcmp (MMDF_SEP, buf) != 0))
+  if (fseek (ctx->fp, offset, SEEK_SET) != 0 ||  /* seek the append location */
+      /* do a sanity check to make sure the mailbox looks ok */
+      fgets (buf, sizeof (buf), ctx->fp) == NULL ||
+      (ctx->magic == M_MBOX && mutt_strncmp ("From ", buf, 5) != 0) ||
+      (ctx->magic == M_MMDF && mutt_strcmp (MMDF_SEP, buf) != 0) ||
+      (ctx->magic == M_KENDRA && mutt_strcmp (KENDRA_SEP, buf) != 0))
   {
-    dprint (1, (debugfile, "mbox_sync_mailbox(): message not in expected position."));
+    dprint (1, (debugfile, "mbox_sync_mailbox: message not in expected position."));
     dprint (1, (debugfile, "\tLINE: %s\n", buf));
     i = -1;
   }
   else
   {
-    fseek (ctx->fp, offset, 0); /* return to proper offset */
-
-    /* copy the temp mailbox back into place starting at the first
-     * change/deleted message
-     */
-    i = mutt_copy_stream (fp, ctx->fp);
-
-    if (ferror (ctx->fp))
+    if (fseek (ctx->fp, offset, SEEK_SET) != 0) /* return to proper offset */
+    {
       i = -1;
+      dprint (1, (debugfile, "mbox_sync_mailbox: fseek() failed\n"));
+    }
+    else
+    {
+      /* copy the temp mailbox back into place starting at the first
+       * change/deleted message
+       */
+      i = mutt_copy_stream (fp, ctx->fp);
 
+      if (ferror (ctx->fp))
+        i = -1;
+    }
     if (i == 0)
     {
       ctx->size = ftell (ctx->fp); /* update the size of the mailbox */
@@ -903,6 +936,7 @@ int mbox_sync_mailbox (CONTEXT *ctx)
   }
 
   fclose (fp);
+  fp = NULL;
   mbox_unlock_mailbox (ctx);
 
   if (fclose (ctx->fp) != 0 || i == -1)
@@ -914,12 +948,13 @@ int mbox_sync_mailbox (CONTEXT *ctx)
     char savefile[_POSIX_PATH_MAX];
     
     snprintf (savefile, sizeof (savefile), "%s/mutt.%s-%s-%d",
-	      Tempdir, Username, Hostname, getpid ());
+	      NONULL (Tempdir), NONULL(Username), NONULL(Hostname), getpid ());
     rename (tempfile, savefile);
     mutt_unblock_signals ();
     mx_fastclose_mailbox (ctx);
     mutt_pretty_mailbox (savefile);
-    mutt_error ("Write failed!  Saved partial mailbox to %s", savefile);
+    mutt_error (_("Write failed!  Saved partial mailbox to %s"), savefile);
+    sleep (5);
     return (-1);
   }
 
@@ -934,7 +969,7 @@ int mbox_sync_mailbox (CONTEXT *ctx)
     unlink (tempfile);
     mutt_unblock_signals ();
     mx_fastclose_mailbox (ctx);
-    mutt_error ("Fatal error!  Could not reopen mailbox!");
+    mutt_error _("Fatal error!  Could not reopen mailbox!");
     Sort = save_sort;
     return (-1);
   }
@@ -951,6 +986,7 @@ int mbox_sync_mailbox (CONTEXT *ctx)
     }
   }
   safe_free ((void **) &newOffset);
+  safe_free ((void **) &oldOffset);
   unlink (tempfile); /* remove partial copy of the mailbox */
   mutt_unblock_signals ();
   Sort = save_sort; /* Restore the default value. */
@@ -959,15 +995,31 @@ int mbox_sync_mailbox (CONTEXT *ctx)
 
 bail:  /* Come here in case of disaster */
 
+  safe_fclose (&fp);
+
+  /* restore offsets, as far as they are valid */
+  if (first >= 0 && oldOffset)
+  {
+    for (i = first; i < ctx->msgcount && oldOffset[i-first].valid; i++)
+    {
+      ctx->hdrs[i]->offset = oldOffset[i-first].hdr;
+      ctx->hdrs[i]->content->hdr_offset = oldOffset[i-first].hdr;
+      ctx->hdrs[i]->content->offset = oldOffset[i-first].body;
+      ctx->hdrs[i]->lines = oldOffset[i-first].lines;
+      ctx->hdrs[i]->content->length = oldOffset[i-first].length;
+    }
+  }
+  
   /* this is ok to call even if we haven't locked anything */
   mbox_unlock_mailbox (ctx);
 
   mutt_unblock_signals ();
   safe_free ((void **) &newOffset);
+  safe_free ((void **) &oldOffset);
 
   if ((ctx->fp = freopen (ctx->path, "r", ctx->fp)) == NULL)
   {
-    mutt_error ("Could not reopen mailbox!");
+    mutt_error _("Could not reopen mailbox!");
     mx_fastclose_mailbox (ctx);
     return (-1);
   }
@@ -980,28 +1032,189 @@ bail:  /* Come here in case of disaster */
     mutt_sort_headers (ctx, (need_sort == M_REOPENED));
   }
 
-  return (-1);
+  return rc;
 }
 
 /* close a mailbox opened in write-mode */
 int mbox_close_mailbox (CONTEXT *ctx)
 {
-#ifdef USE_SETGID
-  if (ctx->setgid)
-    SETEGID (MailGid);
-#endif
-
-  mx_unlock_file (ctx->path, fileno (ctx->fp));
-
-#ifdef USE_SETGID
-  if (ctx->setgid)
-  {
-    SETEGID (UserGid);
-    ctx->setgid = 0;
-  }
-#endif
-
+  mx_unlock_file (ctx->path, fileno (ctx->fp), 1);
   mutt_unblock_signals ();
   mx_fastclose_mailbox (ctx);
   return 0;
 }
+
+int mutt_reopen_mailbox (CONTEXT *ctx, int *index_hint)
+{
+  int (*cmp_headers) (const HEADER *, const HEADER *) = NULL;
+  HEADER **old_hdrs;
+  int old_msgcount;
+  int msg_mod = 0;
+  int index_hint_set;
+  int i, j;
+  int rc = -1;
+
+  /* silent operations */
+  ctx->quiet = 1;
+  
+  mutt_message _("Reopening mailbox...");
+  
+  /* our heuristics require the old mailbox to be unsorted */
+  if (Sort != SORT_ORDER)
+  {
+    short old_sort;
+
+    old_sort = Sort;
+    Sort = SORT_ORDER;
+    mutt_sort_headers (ctx, 1);
+    Sort = old_sort;
+  }
+
+  old_hdrs = NULL;
+  old_msgcount = 0;
+  
+  /* simulate a close */
+  hash_destroy (&ctx->id_hash, NULL);
+  hash_destroy (&ctx->subj_hash, NULL);
+  safe_free ((void **) &ctx->v2r);
+  if (ctx->readonly)
+  {
+    for (i = 0; i < ctx->msgcount; i++)
+      mutt_free_header (&(ctx->hdrs[i])); /* nothing to do! */
+    safe_free ((void **) &ctx->hdrs);
+  }
+  else
+  {
+      /* save the old headers */
+    old_msgcount = ctx->msgcount;
+    old_hdrs = ctx->hdrs;
+    ctx->hdrs = NULL;
+  }
+
+  ctx->hdrmax = 0;	/* force allocation of new headers */
+  ctx->msgcount = 0;
+  ctx->vcount = 0;
+  ctx->tagged = 0;
+  ctx->deleted = 0;
+  ctx->new = 0;
+  ctx->unread = 0;
+  ctx->flagged = 0;
+  ctx->changed = 0;
+  ctx->id_hash = hash_create (1031);
+  ctx->subj_hash = hash_create (1031);
+
+  switch (ctx->magic)
+  {
+    case M_MBOX:
+    case M_MMDF:
+    case M_KENDRA:
+      if (fseek (ctx->fp, 0, SEEK_SET) != 0)
+      {
+        dprint (1, (debugfile, "mutt_reopen_mailbox: fseek() failed\n"));
+        rc = -1;
+      } 
+      else 
+      {
+        cmp_headers = mbox_strict_cmp_headers;
+        rc = ((ctx->magic == M_MBOX) ? mbox_parse_mailbox
+				     : mmdf_parse_mailbox) (ctx);
+      }
+      break;
+
+    default:
+      rc = -1;
+      break;
+  }
+  
+  if (rc == -1)
+  {
+    /* free the old headers */
+    for (j = 0; j < old_msgcount; j++)
+      mutt_free_header (&(old_hdrs[j]));
+    safe_free ((void **) &old_hdrs);
+
+    ctx->quiet = 0;
+    return (-1);
+  }
+
+  /* now try to recover the old flags */
+
+  index_hint_set = (index_hint == NULL);
+
+  if (!ctx->readonly)
+  {
+    for (i = 0; i < ctx->msgcount; i++)
+    {
+      int found = 0;
+
+      /* some messages have been deleted, and new  messages have been
+       * appended at the end; the heuristic is that old messages have then
+       * "advanced" towards the beginning of the folder, so we begin the
+       * search at index "i"
+       */
+      for (j = i; j < old_msgcount; j++)
+      {
+	if (old_hdrs[j] == NULL)
+	  continue;
+	if (cmp_headers (ctx->hdrs[i], old_hdrs[j]))
+	{
+	  found = 1;
+	  break;
+	}
+      }
+      if (!found)
+      {
+	for (j = 0; j < i && j < old_msgcount; j++)
+	{
+	  if (old_hdrs[j] == NULL)
+	    continue;
+	  if (cmp_headers (ctx->hdrs[i], old_hdrs[j]))
+	  {
+	    found = 1;
+	    break;
+	  }
+	}
+      }
+
+      if (found)
+      {
+	/* this is best done here */
+	if (!index_hint_set && *index_hint == j)
+	  *index_hint = i;
+
+	if (old_hdrs[j]->changed)
+	{
+	  /* Only update the flags if the old header was changed;
+	   * otherwise, the header may have been modified externally,
+	   * and we don't want to lose _those_ changes
+	   */
+	  mutt_set_flag (ctx, ctx->hdrs[i], M_FLAG, old_hdrs[j]->flagged);
+	  mutt_set_flag (ctx, ctx->hdrs[i], M_REPLIED, old_hdrs[j]->replied);
+	  mutt_set_flag (ctx, ctx->hdrs[i], M_OLD, old_hdrs[j]->old);
+	  mutt_set_flag (ctx, ctx->hdrs[i], M_READ, old_hdrs[j]->read);
+	}
+	mutt_set_flag (ctx, ctx->hdrs[i], M_DELETE, old_hdrs[j]->deleted);
+	mutt_set_flag (ctx, ctx->hdrs[i], M_TAG, old_hdrs[j]->tagged);
+
+	/* we don't need this header any more */
+	mutt_free_header (&(old_hdrs[j]));
+      }
+    }
+
+    /* free the remaining old headers */
+    for (j = 0; j < old_msgcount; j++)
+    {
+      if (old_hdrs[j])
+      {
+	mutt_free_header (&(old_hdrs[j]));
+	msg_mod = 1;
+      }
+    }
+    safe_free ((void **) &old_hdrs);
+  }
+
+  ctx->quiet = 0;
+
+  return ((ctx->changed || msg_mod) ? M_REOPENED : M_NEW_MAIL);
+}
+
