@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
  *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -13,59 +13,63 @@
  *
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */
 
 #include "mutt.h"
 #include "mutt_curses.h"
 #include "mutt_menu.h"
 
+#ifdef USE_IMAP
+#include "imap.h"
+#endif
+
 #include <string.h>
 #include <stdlib.h>
 
-#define M_MODEFMT "-- Mutt: %s"
-
-static void print_enriched_string (unsigned char *s, int do_color)
+static void print_enriched_string (int attr, unsigned char *s, int do_color)
 {
   while (*s)
   {
-    if (*s < ctrl ('I'))
+    if (*s < M_TREE_MAX)
     {
       if (do_color)
 	SETCOLOR (MT_COLOR_TREE);
-      while (*s && *s < ctrl ('I'))
+      while (*s && *s < M_TREE_MAX)
       {
 	switch (*s)
 	{
-	  case 1:
+	  case M_TREE_LLCORNER:
 	    addch (option (OPTASCIICHARS) ? '`' : ACS_LLCORNER);
 	    break;
-	  case 2:
+	  case M_TREE_ULCORNER:
 	    addch (option (OPTASCIICHARS) ? ',' : ACS_ULCORNER);
 	    break;
-	  case 3:
+	  case M_TREE_LTEE:
 	    addch (option (OPTASCIICHARS) ? '|' : ACS_LTEE);
 	    break;
-	  case 4:
+	  case M_TREE_HLINE:
 	    addch (option (OPTASCIICHARS) ? '-' : ACS_HLINE);
 	    break;
-	  case 5:
+	  case M_TREE_VLINE:
 	    addch (option (OPTASCIICHARS) ? '|' : ACS_VLINE);
 	    break;
-	  case 6:
+	  case M_TREE_SPACE:
 	    addch (' ');
 	    break;
-	  case 7:
+	  case M_TREE_RARROW:
 	    addch ('>');
 	    break;
-	  case 8:
+	  case M_TREE_STAR:
 	    addch ('*'); /* fake thread indicator */
 	    break;
+	  case M_TREE_HIDDEN:
+	    addch ('&');
+	    break;
 	}
 	s++;
       }
-      if (do_color)
-	SETCOLOR (MT_COLOR_NORMAL);
+      if (do_color) attrset(attr);
     }
     else
     {
@@ -75,10 +79,21 @@ static void print_enriched_string (unsigned char *s, int do_color)
   }
 }
 
+static void menu_make_entry (char *s, int l, MUTTMENU *menu, int i) 
+{
+  if (menu->dialog) 
+  {
+    strncpy (s, menu->dialog[i], l);
+    menu->current = -1; /* hide menubar */
+  }
+  else
+    menu->make_entry (s, l, menu, i);
+}
+
 void menu_pad_string (char *s, size_t l)
 {
 #if !defined(HAVE_BKGDSET) && !defined (USE_SLANG_CURSES)
-  int n = strlen (s);
+  int n = mutt_strlen (s);
 #endif
   int shift = option (OPTARROWCURSOR) ? 3 : 0;
   
@@ -124,6 +139,17 @@ void menu_redraw_full (MUTTMENU *menu)
   menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
 }
 
+void menu_redraw_status (MUTTMENU *menu)
+{
+  char buf[STRING];
+
+  snprintf (buf, sizeof (buf), M_MODEFMT, menu->title);
+  SETCOLOR (MT_COLOR_STATUS);
+  mvprintw (option (OPTSTATUSONTOP) ? 0 : LINES - 2, 0, "%-*.*s", COLS, COLS, buf);
+  SETCOLOR (MT_COLOR_NORMAL);
+  menu->redraw &= ~REDRAW_STATUS;
+}
+
 void menu_redraw_index (MUTTMENU *menu)
 {
   char buf[STRING];
@@ -133,23 +159,26 @@ void menu_redraw_index (MUTTMENU *menu)
   {
     if (i < menu->max)
     {
-      menu->make_entry (buf, sizeof (buf), menu, i);
+      menu_make_entry (buf, sizeof (buf), menu, i);
       menu_pad_string (buf, sizeof (buf));
 
       if (option (OPTARROWCURSOR))
       {
+        attrset (menu->color (i));
 	CLEARLINE (i - menu->top + menu->offset);
 
 	if (i == menu->current)
 	{
 	  SETCOLOR (MT_COLOR_INDICATOR);
 	  addstr ("->");
-	  SETCOLOR (MT_COLOR_NORMAL);
+          attrset (menu->color (i));
 	  addch (' ');
 	}
 	else
 	  move (i - menu->top + menu->offset, 3);
-	print_enriched_string ((unsigned char *) buf, 1);
+
+        print_enriched_string (menu->color(i), (unsigned char *) buf, 1);
+        SETCOLOR (MT_COLOR_NORMAL);          
       }
       else
       {
@@ -158,13 +187,13 @@ void menu_redraw_index (MUTTMENU *menu)
 	  SETCOLOR (MT_COLOR_INDICATOR);
 	  BKGDSET (MT_COLOR_INDICATOR);
 	}
+        else
+          attrset (menu->color (i));
+            
 	CLEARLINE (i - menu->top + menu->offset);
-	print_enriched_string ((unsigned char *) buf, i != menu->current);
-	if (i == menu->current)
-	{
-	  SETCOLOR (MT_COLOR_NORMAL);
-	  BKGDSET (MT_COLOR_NORMAL);
-	}
+	print_enriched_string (menu->color(i), (unsigned char *) buf, i != menu->current);
+        SETCOLOR (MT_COLOR_NORMAL);
+        BKGDSET (MT_COLOR_NORMAL);
       }
     }
     else
@@ -176,6 +205,12 @@ void menu_redraw_index (MUTTMENU *menu)
 void menu_redraw_motion (MUTTMENU *menu)
 {
   char buf[STRING];
+
+  if (menu->dialog) 
+  {
+    menu->redraw &= ~REDRAW_MOTION;
+    return;
+  }
   
   move (menu->oldcurrent + menu->offset - menu->top, 0);
   SETCOLOR (MT_COLOR_NORMAL);
@@ -184,15 +219,17 @@ void menu_redraw_motion (MUTTMENU *menu)
   if (option (OPTARROWCURSOR))
   {
     /* clear the pointer */
+    attrset (menu->color (menu->oldcurrent));
     addstr ("  ");
 
     if (menu->redraw & REDRAW_MOTION_RESYNCH)
     {
       clrtoeol ();
-      menu->make_entry (buf, sizeof (buf), menu, menu->oldcurrent);
+      menu_make_entry (buf, sizeof (buf), menu, menu->oldcurrent);
       menu_pad_string (buf, sizeof (buf));
       move (menu->oldcurrent + menu->offset - menu->top, 3);
-      print_enriched_string ((unsigned char *) buf, 1);
+      print_enriched_string (menu->color(menu->oldcurrent), (unsigned char *) buf, 1);
+      SETCOLOR (MT_COLOR_NORMAL);
     }
 
     /* now draw it in the new location */
@@ -204,18 +241,19 @@ void menu_redraw_motion (MUTTMENU *menu)
   else
   {
     /* erase the current indicator */
+    attrset (menu->color (menu->oldcurrent));
     clrtoeol ();
-    menu->make_entry (buf, sizeof (buf), menu, menu->oldcurrent);
+    menu_make_entry (buf, sizeof (buf), menu, menu->oldcurrent);
     menu_pad_string (buf, sizeof (buf));
-    print_enriched_string ((unsigned char *) buf, 1);
+    print_enriched_string (menu->color(menu->oldcurrent), (unsigned char *) buf, 1);
 
     /* now draw the new one to reflect the change */
-    menu->make_entry (buf, sizeof (buf), menu, menu->current);
+    menu_make_entry (buf, sizeof (buf), menu, menu->current);
     menu_pad_string (buf, sizeof (buf));
     SETCOLOR (MT_COLOR_INDICATOR);
     BKGDSET (MT_COLOR_INDICATOR);
     CLEARLINE (menu->current - menu->top + menu->offset);
-    print_enriched_string ((unsigned char *) buf, 0);
+    print_enriched_string (menu->color(menu->current), (unsigned char *) buf, 0);
     SETCOLOR (MT_COLOR_NORMAL);
     BKGDSET (MT_COLOR_NORMAL);
   }
@@ -227,40 +265,68 @@ void menu_redraw_current (MUTTMENU *menu)
   char buf[STRING];
   
   move (menu->current + menu->offset - menu->top, 0);
-  menu->make_entry (buf, sizeof (buf), menu, menu->current);
+  menu_make_entry (buf, sizeof (buf), menu, menu->current);
   menu_pad_string (buf, sizeof (buf));
 
   if (option (OPTARROWCURSOR))
   {
+    int attr = menu->color (menu->current);
+    attrset (attr);
     clrtoeol ();
     SETCOLOR (MT_COLOR_INDICATOR);
     addstr ("->");
-    SETCOLOR (MT_COLOR_NORMAL);
+    attrset (attr);
     addch (' ');
     menu_pad_string (buf, sizeof (buf));
-    print_enriched_string ((unsigned char *) buf, 1);
+    print_enriched_string (menu->color(menu->current), (unsigned char *) buf, 1);
+    SETCOLOR (MT_COLOR_NORMAL);
   }
   else
   {
     SETCOLOR (MT_COLOR_INDICATOR);
     BKGDSET (MT_COLOR_INDICATOR);
     clrtoeol ();
-    print_enriched_string ((unsigned char *) buf, 0);
+    print_enriched_string (menu->color(menu->current), (unsigned char *) buf, 0);
     SETCOLOR (MT_COLOR_NORMAL);
     BKGDSET (MT_COLOR_NORMAL);
   }
   menu->redraw &= REDRAW_STATUS;
 }
 
+void menu_redraw_prompt (MUTTMENU *menu)
+{
+  if (menu->dialog) 
+  {
+    if (option (OPTMSGERR)) 
+    {
+      sleep (1);
+      unset_option (OPTMSGERR);
+    }
+
+    if (*Errorbuf)
+      mutt_clear_error ();
+
+    SETCOLOR (MT_COLOR_NORMAL);
+    mvaddstr (LINES - 1, 0, menu->prompt);
+    clrtoeol ();
+  }
+}
+
 void menu_check_recenter (MUTTMENU *menu)
 {
-  if (menu->current >= menu->top + menu->pagelen)
+  if (menu->max <= menu->pagelen && menu->top != 0)
+  {
+    menu->top = 0;
+    set_option (OPTNEEDREDRAW);
+    menu->redraw |= REDRAW_INDEX;
+  }
+  else if (menu->current >= menu->top + menu->pagelen)
   {
     if (option (OPTMENUSCROLL))
       menu->top = menu->current - menu->pagelen + 1;
     else
       menu->top += menu->pagelen * ((menu->current - menu->top) / menu->pagelen);
-    menu->redraw = REDRAW_INDEX;
+    menu->redraw |= REDRAW_INDEX;
   }
   else if (menu->current < menu->top)
   {
@@ -272,7 +338,7 @@ void menu_check_recenter (MUTTMENU *menu)
       if (menu->top < 0)
 	menu->top = 0;
     }
-    menu->redraw = REDRAW_INDEX;
+    menu->redraw |= REDRAW_INDEX;
   }
 }
 
@@ -283,9 +349,9 @@ void menu_jump (MUTTMENU *menu)
 
   if (menu->max)
   {
-    mutt_ungetch (LastKey);
+    mutt_ungetch (LastKey, 0);
     buf[0] = 0;
-    if (mutt_get_field ("Jump to: ", buf, sizeof (buf), 0) == 0 && buf[0])
+    if (mutt_get_field (_("Jump to: "), buf, sizeof (buf), 0) == 0 && buf[0])
     {
       n = atoi (buf) - 1;
       if (n >= 0 && n < menu->max)
@@ -294,18 +360,18 @@ void menu_jump (MUTTMENU *menu)
 	menu->redraw = REDRAW_MOTION;
       }
       else
-	mutt_error ("Invalid index number.");
+	mutt_error _("Invalid index number.");
     }
   }
   else
-    mutt_error ("No entries.");
+    mutt_error _("No entries.");
 }
 
 void menu_next_line (MUTTMENU *menu)
 {
   if (menu->max)
   {
-    if (menu->top < menu->max - 1)
+    if (menu->top + 1 < menu->max)
     {
       menu->top++;
       if (menu->current < menu->top)
@@ -313,10 +379,10 @@ void menu_next_line (MUTTMENU *menu)
       menu->redraw = REDRAW_INDEX;
     }
     else
-      mutt_error ("You cannot scroll down farther.");
+      mutt_error _("You cannot scroll down farther.");
   }
   else
-    mutt_error ("No entries.");
+    mutt_error _("No entries.");
 }
 
 void menu_prev_line (MUTTMENU *menu)
@@ -329,7 +395,7 @@ void menu_prev_line (MUTTMENU *menu)
     menu->redraw = REDRAW_INDEX;
   }
   else
-    mutt_error ("You cannot scroll up farther.");
+    mutt_error _("You cannot scroll up farther.");
 }
 
 void menu_next_page (MUTTMENU *menu)
@@ -343,16 +409,16 @@ void menu_next_page (MUTTMENU *menu)
 	menu->current = menu->top;
       menu->redraw = REDRAW_INDEX;
     }
-    else if (menu->current != menu->max - 1)
+    else if (menu->current != menu->max - 1 && !menu->dialog)
     {
       menu->current = menu->max - 1;
       menu->redraw = REDRAW_MOTION;
     }
     else
-      mutt_error ("You are on the last page.");
+      mutt_error _("You are on the last page.");
   }
   else
-    mutt_error ("No entries.");
+    mutt_error _("No entries.");
 }
 
 void menu_prev_page (MUTTMENU *menu)
@@ -365,13 +431,13 @@ void menu_prev_page (MUTTMENU *menu)
       menu->current = menu->top + menu->pagelen - 1;
     menu->redraw = REDRAW_INDEX;
   }
-  else if (menu->current)
+  else if (menu->current && !menu->dialog)
   {
     menu->current = 0;
     menu->redraw = REDRAW_MOTION;
   }
   else
-    mutt_error ("You are on the first page.");
+    mutt_error _("You are on the first page.");
 }
 
 void menu_top_page (MUTTMENU *menu)
@@ -393,7 +459,7 @@ void menu_bottom_page (MUTTMENU *menu)
     menu->redraw = REDRAW_MOTION;
   }
   else
-    mutt_error ("No entries.");
+    mutt_error _("No entries.");
 }
 
 void menu_middle_page (MUTTMENU *menu)
@@ -409,7 +475,7 @@ void menu_middle_page (MUTTMENU *menu)
     menu->redraw = REDRAW_MOTION;
   }
   else
-    mutt_error ("No entries.");
+    mutt_error _("No entries.");
 }
 
 void menu_first_entry (MUTTMENU *menu)
@@ -420,7 +486,7 @@ void menu_first_entry (MUTTMENU *menu)
     menu->redraw = REDRAW_MOTION;
   }
   else
-    mutt_error ("No entries.");
+    mutt_error _("No entries.");
 }
 
 void menu_last_entry (MUTTMENU *menu)
@@ -431,7 +497,7 @@ void menu_last_entry (MUTTMENU *menu)
     menu->redraw = REDRAW_MOTION;
   }
   else
-    mutt_error ("No entries.");
+    mutt_error _("No entries.");
 }
 
 void menu_half_up (MUTTMENU *menu)
@@ -444,13 +510,13 @@ void menu_half_up (MUTTMENU *menu)
       menu->current = menu->top + menu->pagelen - 1;
     menu->redraw = REDRAW_INDEX;
   }
-  else if (menu->current)
+  else if (menu->current && !menu->dialog)
   {
     menu->current = 0;
     menu->redraw = REDRAW_MOTION;
   }
   else
-    mutt_error ("First entry is shown.");
+    mutt_error _("First entry is shown.");
 }
 
 void menu_half_down (MUTTMENU *menu)
@@ -464,16 +530,16 @@ void menu_half_down (MUTTMENU *menu)
 	menu->current = menu->top;
       menu->redraw = REDRAW_INDEX;
     }
-    else if (menu->current != menu->max - 1)
+    else if (menu->current != menu->max - 1 && !menu->dialog)
     {
       menu->current = menu->max - 1;
       menu->redraw = REDRAW_INDEX;
     }
     else
-      mutt_error ("Last entry is shown.");
+      mutt_error _("Last entry is shown.");
   }
   else
-    mutt_error ("No entries.");
+    mutt_error _("No entries.");
 }
 
 void menu_current_top (MUTTMENU *menu)
@@ -484,7 +550,7 @@ void menu_current_top (MUTTMENU *menu)
     menu->redraw = REDRAW_INDEX;
   }
   else
-    mutt_error ("No entries.");
+    mutt_error _("No entries.");
 }
 
 void menu_current_middle (MUTTMENU *menu)
@@ -497,7 +563,7 @@ void menu_current_middle (MUTTMENU *menu)
     menu->redraw = REDRAW_INDEX;
   }
   else
-    mutt_error ("No entries.");
+    mutt_error _("No entries.");
 }
 
 void menu_current_bottom (MUTTMENU *menu)
@@ -510,7 +576,7 @@ void menu_current_bottom (MUTTMENU *menu)
     menu->redraw = REDRAW_INDEX;
   }
   else
-    mutt_error ("No entries.");
+    mutt_error _("No entries.");
 }
 
 void menu_next_entry (MUTTMENU *menu)
@@ -521,7 +587,7 @@ void menu_next_entry (MUTTMENU *menu)
     menu->redraw = REDRAW_MOTION;
   }
   else
-    mutt_error ("You are on the last entry.");
+    mutt_error _("You are on the last entry.");
 }
 
 void menu_prev_entry (MUTTMENU *menu)
@@ -532,22 +598,50 @@ void menu_prev_entry (MUTTMENU *menu)
     menu->redraw = REDRAW_MOTION;
   }
   else
-    mutt_error ("You are on the first entry.");
+    mutt_error _("You are on the first entry.");
+}
+
+static int default_color (int i)
+{
+   return ColorDefs[MT_COLOR_NORMAL];
+}
+
+static int menu_search_generic (MUTTMENU *m, regex_t *re, int n)
+{
+  char buf[LONG_STRING];
+
+  menu_make_entry (buf, sizeof (buf), m, n);
+  return (regexec (re, buf, 0, NULL, 0));
 }
 
 MUTTMENU *mutt_new_menu (void)
 {
   MUTTMENU *p = (MUTTMENU *) safe_calloc (1, sizeof (MUTTMENU));
 
+  p->current = 0;
+  p->top = 0;
   p->offset = 1;
   p->redraw = REDRAW_FULL;
   p->pagelen = PAGELEN;
+  p->color = default_color;
+  p->search = menu_search_generic;
   return (p);
 }
 
 void mutt_menuDestroy (MUTTMENU **p)
 {
+  int i;
+
   safe_free ((void **) &(*p)->searchBuf);
+
+  if ((*p)->dialog) 
+  {
+    for (i=0; i < (*p)->max; i++)
+      safe_free ((void **) &(*p)->dialog[i]);
+
+    safe_free ((void **) (*p)->dialog);
+  }
+
   safe_free ((void **) p);
 }
 
@@ -557,31 +651,33 @@ void mutt_menuDestroy (MUTTMENU **p)
 static int menu_search (MUTTMENU *menu, int op)
 {
   int r;
-  int searchDir = (menu->searchDir == M_SEARCH_UP) ? -1 : 1;
+  int searchDir;
   regex_t re;
   char buf[SHORT_STRING];
 
   if (op != OP_SEARCH_NEXT && op != OP_SEARCH_OPPOSITE)
   {
     strfcpy (buf, menu->searchBuf ? menu->searchBuf : "", sizeof (buf));
-    if (mutt_get_field ("Search for: ", buf, sizeof (buf), M_CLEAR) != 0 || !buf[0])
+    if (mutt_get_field ((op == OP_SEARCH) ? _("Search for: ") : 
+                                            _("Reverse search for: "),
+			 buf, sizeof (buf), M_CLEAR) != 0 || !buf[0])
       return (-1);
-    safe_free ((void **) &menu->searchBuf);
-    menu->searchBuf = safe_strdup (buf);
+    mutt_str_replace (&menu->searchBuf, buf);
     menu->searchDir = (op == OP_SEARCH) ? M_SEARCH_DOWN : M_SEARCH_UP;
   }
   else 
   {
     if (!menu->searchBuf)
     {
-      mutt_error ("No search pattern.");
+      mutt_error _("No search pattern.");
       return (-1);
     }
-
-    if (op == OP_SEARCH_OPPOSITE)
-      searchDir = -searchDir;
   }
 
+  searchDir = (menu->searchDir == M_SEARCH_UP) ? -1 : 1;
+  if (op == OP_SEARCH_OPPOSITE)
+    searchDir = -searchDir;
+
   if ((r = REGCOMP (&re, menu->searchBuf, REG_NOSUB | mutt_which_case (menu->searchBuf))) != 0)
   {
     regerror (r, &re, buf, sizeof (buf));
@@ -603,19 +699,66 @@ static int menu_search (MUTTMENU *menu, int op)
   }
 
   regfree (&re);
-  mutt_error ("Not found.");
+  mutt_error _("Not found.");
   return (-1);
 }
 
+static int menu_dialog_translate_op (int i)
+{
+  switch (i)
+  {
+    case OP_NEXT_ENTRY:   
+      return OP_NEXT_LINE;
+    case OP_PREV_ENTRY:	  
+      return OP_PREV_LINE;
+    case OP_CURRENT_TOP:   case OP_FIRST_ENTRY:  
+      return OP_TOP_PAGE;
+    case OP_CURRENT_BOTTOM:    case OP_LAST_ENTRY:	  
+      return OP_BOTTOM_PAGE;
+    case OP_CURRENT_MIDDLE: 
+      return OP_MIDDLE_PAGE; 
+  }
+  
+  return i;
+}
+
+static int menu_dialog_dokey (MUTTMENU *menu, int *ip)
+{
+  event_t ch;
+  char *p;
+
+  ch = mutt_getch ();
+
+  if (ch.ch == -1)
+  {
+    *ip = -1;
+    return 0;
+  }
+
+  if (ch.ch && (p = strchr (menu->keys, ch.ch)))
+  {
+    *ip = OP_MAX + (p - menu->keys + 1);
+    return 0;
+  }
+  else
+  {
+    mutt_ungetch (ch.op ? 0 : ch.ch, ch.op ? ch.op : 0);
+    return -1;
+  }
+}
+
 int mutt_menuLoop (MUTTMENU *menu)
 {
   int i = OP_NULL;
-  char buf[STRING];
 
   FOREVER
   {
     mutt_curs_set (0);
 
+#ifdef USE_IMAP
+    imap_keepalive ();
+#endif
+
     /* See if all or part of the screen needs to be updated.  */
     if (menu->redraw & REDRAW_FULL)
     {
@@ -624,17 +767,11 @@ int mutt_menuLoop (MUTTMENU *menu)
       return (OP_REDRAW);
     }
 
-    menu_check_recenter (menu);
+    if (!menu->dialog)
+      menu_check_recenter (menu);
 
     if (menu->redraw & REDRAW_STATUS)
-    {
-      snprintf (buf, sizeof (buf), M_MODEFMT, menu->title);
-      SETCOLOR (MT_COLOR_STATUS);
-      mvprintw (option (OPTSTATUSONTOP) ? 0 : LINES - 2, 0, "%-*.*s", COLS, COLS, buf);
-      SETCOLOR (MT_COLOR_NORMAL);
-      menu->redraw &= ~REDRAW_STATUS;
-    }
-
+      menu_redraw_status (menu);
     if (menu->redraw & REDRAW_INDEX)
       menu_redraw_index (menu);
     else if (menu->redraw & (REDRAW_MOTION | REDRAW_MOTION_RESYNCH))
@@ -642,20 +779,38 @@ int mutt_menuLoop (MUTTMENU *menu)
     else if (menu->redraw == REDRAW_CURRENT)
       menu_redraw_current (menu);
 
+    if (menu->dialog)
+      menu_redraw_prompt (menu);
+    
     menu->oldcurrent = menu->current;
 
+
     /* move the cursor out of the way */
     move (menu->current - menu->top + menu->offset,
 	  (option (OPTARROWCURSOR) ? 2 : COLS-1));
 
     mutt_refresh ();
+    
+    /* try to catch dialog keys before ops */
+    if (menu->dialog && menu_dialog_dokey (menu, &i) == 0)
+      return i;
+		    
     i = km_dokey (menu->menu);
     if (i == OP_TAG_PREFIX)
     {
-      mvaddstr (LINES - 1, 0, "Tag-");
-      i = km_dokey (menu->menu);
-      menu->tagprefix = 1;
-      CLEARLINE (LINES - 1);
+      if (menu->tagged)
+      {
+	mvaddstr (LINES - 1, 0, "Tag-");
+	clrtoeol ();
+	i = km_dokey (menu->menu);
+	menu->tagprefix = 1;
+	CLEARLINE (LINES - 1);
+      }
+      else 
+      {
+	mutt_error _("No tagged entries.");
+	i = -1;
+      }
     }
     else if (menu->tagged && option (OPTAUTOTAG))
       menu->tagprefix = 1;
@@ -665,18 +820,23 @@ int mutt_menuLoop (MUTTMENU *menu)
     mutt_curs_set (1);
 
 #if defined (USE_SLANG_CURSES) || defined (HAVE_RESIZETERM)
-    if (Signals & S_SIGWINCH)
+    if (SigWinch)
     {
       mutt_resize_screen ();
       menu->redraw = REDRAW_FULL;
-      Signals &= ~S_SIGWINCH;
+      SigWinch = 0;
     }
 #endif
 
     if (i == -1)
       continue;
 
-    mutt_clear_error ();
+    if (!menu->dialog)
+      mutt_clear_error ();
+
+    /* Convert menubar movement to scrolling */
+    if (menu->dialog) 
+      i = menu_dialog_translate_op (i);
 
     switch (i)
     {
@@ -732,7 +892,7 @@ int mutt_menuLoop (MUTTMENU *menu)
       case OP_SEARCH_REVERSE:
       case OP_SEARCH_NEXT:
       case OP_SEARCH_OPPOSITE:
-	if (menu->search)
+	if (menu->search && !menu->dialog) /* Searching dialogs won't work */
 	{
 	  menu->oldcurrent = menu->current;
 	  if ((menu->current = menu_search (menu, i)) != -1)
@@ -741,14 +901,18 @@ int mutt_menuLoop (MUTTMENU *menu)
 	    menu->current = menu->oldcurrent;
 	}
 	else
-	  mutt_error ("Search is not implemented for this menu.");
+	  mutt_error _("Search is not implemented for this menu.");
 	break;
 
       case OP_JUMP:
-	menu_jump (menu);
+	if (menu->dialog)
+	  mutt_error _("Jumping is not implemented for dialogs.");
+	else
+	  menu_jump (menu);
 	break;
 
       case OP_ENTER_COMMAND:
+	CurrentMenu = menu->menu;
 	mutt_enter_command ();
 	if (option (OPTFORCEREDRAWINDEX))
 	{
@@ -759,15 +923,13 @@ int mutt_menuLoop (MUTTMENU *menu)
 	break;
 
       case OP_TAG:
-	if (menu->tag)
+	if (menu->tag && !menu->dialog)
 	{
 	  if (menu->max)
 	  {
-	    if (menu->tag (menu, menu->current))
-	      menu->tagged++;
-	    else
-	      menu->tagged--;
-	    if (option (OPTRESOLVE) && menu->current < menu->max - 1)
+	    int i = menu->tag (menu, menu->current);
+	    menu->tagged += i;
+	    if (i && option (OPTRESOLVE) && menu->current < menu->max - 1)
 	    {
 	      menu->current++;
 	      menu->redraw = REDRAW_MOTION_RESYNCH;
@@ -776,10 +938,10 @@ int mutt_menuLoop (MUTTMENU *menu)
 	      menu->redraw = REDRAW_CURRENT;
 	  }
 	  else
-	    mutt_error ("No entries.");
+	    mutt_error _("No entries.");
 	}
 	else
-	  mutt_error ("Tagging is not supported.");
+	  mutt_error _("Tagging is not supported.");
 	break;
 
       case OP_SHELL_ESCAPE:
