@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
  *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
  *
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */
 
 /*
@@ -30,6 +30,9 @@
 #define REDRAW_STATUS		(1<<4)
 #define REDRAW_FULL		(1<<5)
 #define REDRAW_BODY		(1<<6)
+#define REDRAW_SIGWINCH		(1<<7)
+
+#define M_MODEFMT "-- Mutt: %s"
 
 typedef struct menu_t
 {
@@ -43,6 +46,15 @@ typedef struct menu_t
   int offset;	/* which screen row to start the index */
   int pagelen;	/* number of entries per screen */
   int tagprefix;
+
+  /* Setting dialog != NULL overrides normal menu behaviour. 
+   * In dialog mode menubar is hidden and prompt keys are checked before
+   * normal menu movement keys. This can cause problems with scrolling, if 
+   * prompt keys override movement keys.
+   */
+  char **dialog;	/* dialog lines themselves */
+  char *prompt;		/* prompt for user, similar to mutt_multi_choice */
+  char *keys;		/* keys used in the prompt */
   
   /* callback to generate an index line for the requested element */
   void (*make_entry) (char *, size_t, struct menu_t *, int);
@@ -52,6 +64,11 @@ typedef struct menu_t
 
   int (*tag) (struct menu_t *, int i);
 
+  /* color pair to be used for the requested element 
+   * (default function returns ColorDefs[MT_COLOR_NORMAL])
+   */
+  int (*color) (int i);
+   
   /* the following are used only by mutt_menuLoop() */
   int top;		/* entry that is the top of the current page */
   int oldcurrent;	/* for driver use only. */
@@ -63,6 +80,7 @@ typedef struct menu_t
 void menu_jump (MUTTMENU *);
 void menu_redraw_full (MUTTMENU *);
 void menu_redraw_index (MUTTMENU *);
+void menu_redraw_status (MUTTMENU *);
 void menu_redraw_motion (MUTTMENU *);
 void menu_redraw_current (MUTTMENU *);
 void menu_first_entry (MUTTMENU *);
@@ -88,3 +106,4 @@ int mutt_menuLoop (MUTTMENU *);
 
 /* used in both the index and pager index to make an entry. */
 void index_make_entry (char *, size_t, struct menu_t *, int);
+int index_color (int);
