@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1999-2000 Thomas Roessler <roessler@guug.de>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -13,7 +14,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */ 
 
 #include "mutt.h"
@@ -25,6 +26,9 @@
 #include "keymap.h"
 
 
+#ifdef HAVE_PGP
+#include "pgp.h"
+#endif
 
 #ifdef USE_IMAP
 #include "imap.h"
@@ -34,6 +38,10 @@
 #include "buffy.h"
 #endif
 
+#ifdef USE_DOTLOCK
+#include "dotlock.h"
+#endif
+
 #include <dirent.h>
 #include <fcntl.h>
 #include <sys/file.h>
@@ -47,161 +55,86 @@
 #include <utime.h>
 #endif
 
-/* HP-UX and ConvexOS don't have this macro */
-#ifndef S_ISLNK
-#define S_ISLNK(x) (((x) & S_IFMT) == S_IFLNK ? 1 : 0)
-#endif
 
-#define mutt_is_spool(s)  (strcmp (Spoolfile, s) == 0)
-
-#define MAXLOCKATTEMPT 5
+#define mutt_is_spool(s)  (mutt_strcmp (Spoolfile, s) == 0)
 
 #ifdef USE_DOTLOCK
 /* parameters: 
  * path - file to lock
  * retry - should retry if unable to lock?
  */
-static int dotlock_file (const char *path, int retry)
+
+#ifdef DL_STANDALONE
+
+static int invoke_dotlock (const char *path, int dummy, int flags, int retry)
 {
-  const char *pathptr = path;
-  char lockfile[_POSIX_PATH_MAX];
-  char nfslockfile[_POSIX_PATH_MAX];
-  char realpath[_POSIX_PATH_MAX];
-  struct stat sb;
-  size_t prev_size = 0;
-  int count = 0;
-  int attempt = 0;
-  int fd;
-
-  /* if the file is a symlink, find the real file to which it refers */
-  FOREVER
-  {
-    dprint(2,(debugfile,"dotlock_file(): locking %s\n", pathptr));
-
-    if (lstat (pathptr, &sb) != 0)
-    {
-      mutt_perror (pathptr);
-      return (-1);
-    }
-
-    if (S_ISLNK (sb.st_mode))
-    {
-      char linkfile[_POSIX_PATH_MAX];
-      char linkpath[_POSIX_PATH_MAX];
-
-      if ((count = readlink (pathptr, linkfile, sizeof (linkfile))) == -1)
-      {
-	mutt_perror (path);
-	return (-1);
-      }
-      linkfile[count] = 0; /* readlink() does not NUL terminate the string! */
-      mutt_expand_link (linkpath, pathptr, linkfile);
-      strfcpy (realpath, linkpath, sizeof (realpath));
-      pathptr = realpath;
-    }
-    else
-      break;
-  }
-
-  snprintf (nfslockfile, sizeof (nfslockfile), "%s.%s.%d", pathptr, Hostname, (int) getpid ());
-  snprintf (lockfile, sizeof (lockfile), "%s.lock", pathptr);
-  unlink (nfslockfile);
-
-  while ((fd = open (nfslockfile, O_WRONLY | O_EXCL | O_CREAT, 0)) < 0)
-    if (errno != EAGAIN)
-    {
-      mutt_perror ("cannot open NFS lock file!");
-      return (-1);
-    }
-
-  close (fd);
-
-  count = 0;
-  FOREVER
-  {
-    link (nfslockfile, lockfile);
-    if (stat (nfslockfile, &sb) != 0)
-    {
-      mutt_perror ("stat");
-      return (-1);
-    }
-
-    if (sb.st_nlink == 2)
-      break;
-    
-    if (stat (path, &sb) != 0)
-      sb.st_size = 0;
-
-    if (count == 0)
-      prev_size = sb.st_size;
-
-    /* only try to remove the lock if the file is not changing */
-    if (prev_size == sb.st_size && ++count >= (retry ? MAXLOCKATTEMPT : 0))
-    {
-      if (retry && mutt_yesorno ("Lock count exceeded, remove lock?", 1) == 1)
-      {
-	unlink (lockfile);
-	count = 0;
-	attempt = 0;
-	continue;
-      }
-      else
-	return (-1);
-    }
-
-    prev_size = sb.st_size;
-
-    mutt_message ("Waiting for lock attempt #%d...", ++attempt);
-    sleep (1);
-  }
-
-  unlink (nfslockfile);
-
-  return 0;
+  char cmd[LONG_STRING + _POSIX_PATH_MAX];
+  char f[SHORT_STRING + _POSIX_PATH_MAX];
+  char r[SHORT_STRING];
+  
+  if (flags & DL_FL_RETRY)
+    snprintf (r, sizeof (r), "-r %d ", retry ? MAXLOCKATTEMPT : 0);
+  
+  mutt_quote_filename (f, sizeof (f), path);
+  
+  snprintf (cmd, sizeof (cmd),
+	    "%s %s%s%s%s%s%s%s",
+	    NONULL (MuttDotlock),
+	    flags & DL_FL_TRY ? "-t " : "",
+	    flags & DL_FL_UNLOCK ? "-u " : "",
+	    flags & DL_FL_USEPRIV ? "-p " : "",
+	    flags & DL_FL_FORCE ? "-f " : "",
+	    flags & DL_FL_UNLINK ? "-d " : "",
+	    flags & DL_FL_RETRY ? r : "",
+	    f);
+  
+  return mutt_system (cmd);
 }
 
-static int undotlock_file (const char *path)
+#else 
+
+#define invoke_dotlock dotlock_invoke
+
+#endif
+
+static int dotlock_file (const char *path, int fd, int retry)
 {
-  const char *pathptr = path;
-  char lockfile[_POSIX_PATH_MAX];
-  char realpath[_POSIX_PATH_MAX];
-  struct stat sb;
-  int n;
+  int r;
+  int flags = DL_FL_USEPRIV | DL_FL_RETRY;
 
-  FOREVER
+  if (retry) retry = 1;
+
+retry_lock:
+  if ((r = invoke_dotlock(path, fd, flags, retry)) == DL_EX_EXIST)
   {
-    dprint (2,(debugfile,"undotlock: unlocking %s\n",path));
-
-    if (lstat (pathptr, &sb) != 0)
+    if (!option (OPTNOCURSES))
     {
-      mutt_perror (pathptr);
-      return (-1);
-    }
-
-    if (S_ISLNK (sb.st_mode))
-    {
-      char linkfile[_POSIX_PATH_MAX];
-      char linkpath[_POSIX_PATH_MAX];
-
-      if ((n = readlink (pathptr, linkfile, sizeof (linkfile))) == -1)
+      char msg[LONG_STRING];
+      
+      snprintf(msg, sizeof(msg), _("Lock count exceeded, remove lock for %s?"),
+	       path);
+      if(retry && mutt_yesorno(msg, 1) == 1)
       {
-	mutt_perror (pathptr);
-	return (-1);
+	flags |= DL_FL_FORCE;
+	retry--;
+	mutt_clear_error ();
+	goto retry_lock;
       }
-      linkfile[n] = 0; /* readlink() does not NUL terminate the string! */
-      mutt_expand_link (linkpath, pathptr, linkfile);
-      strfcpy (realpath, linkpath, sizeof (realpath));
-      pathptr = realpath;
-      continue;
-    }
+    } 
     else
-      break;
+    {
+      mutt_error ( _("Can't dotlock %s.\n"), path);
+    }
   }
-
-  snprintf (lockfile, sizeof (lockfile), "%s.lock", pathptr);
-  unlink (lockfile);
-  return 0;
+  return (r == DL_EX_OK ? 0 : -1);
 }
+
+static int undotlock_file (const char *path, int fd)
+{
+  return (invoke_dotlock(path, fd, DL_FL_USEPRIV | DL_FL_UNLOCK, 0) == DL_EX_OK ? 
+	  0 : -1);
+}
+
 #endif /* USE_DOTLOCK */
 
 /* Args:
@@ -248,13 +181,13 @@ int mx_lock_file (const char *path, int fd, int excl, int dot, int timeout)
     if (prev_sb.st_size == sb.st_size && ++count >= (timeout?MAXLOCKATTEMPT:0))
     {
       if (timeout)
-	mutt_error ("Timeout exceeded while attempting fcntl lock!");
+	mutt_error _("Timeout exceeded while attempting fcntl lock!");
       return (-1);
     }
 
     prev_sb = sb;
 
-    mutt_message ("Waiting for fcntl lock... %d", ++attempt);
+    mutt_message (_("Waiting for fcntl lock... %d"), ++attempt);
     sleep (1);
   }
 #endif /* USE_FCNTL */
@@ -282,21 +215,21 @@ int mx_lock_file (const char *path, int fd, int excl, int dot, int timeout)
     if (prev_sb.st_size == sb.st_size && ++count >= (timeout?MAXLOCKATTEMPT:0))
     {
       if (timeout)
-	mutt_error ("Timeout exceeded while attempting flock lock!");
+	mutt_error _("Timeout exceeded while attempting flock lock!");
       r = -1;
       break;
     }
 
     prev_sb = sb;
 
-    mutt_message ("Waiting for flock attempt... %d", ++attempt);
+    mutt_message (_("Waiting for flock attempt... %d"), ++attempt);
     sleep (1);
   }
 #endif /* USE_FLOCK */
 
 #ifdef USE_DOTLOCK
   if (r == 0 && dot)
-    r = dotlock_file (path,timeout);
+    r = dotlock_file (path, fd, timeout);
 #endif /* USE_DOTLOCK */
 
   if (r == -1)
@@ -318,7 +251,7 @@ int mx_lock_file (const char *path, int fd, int excl, int dot, int timeout)
   return 0;
 }
 
-int mx_unlock_file (const char *path, int fd)
+int mx_unlock_file (const char *path, int fd, int dot)
 {
 #ifdef USE_FCNTL
   struct flock unlockit = { F_UNLCK, 0, 0, 0 };
@@ -334,27 +267,38 @@ int mx_unlock_file (const char *path, int fd)
 #endif
 
 #ifdef USE_DOTLOCK
-  undotlock_file (path);
+  if (dot)
+    undotlock_file (path, fd);
 #endif
   
   return 0;
 }
 
-/* open a file and lock it */
-FILE *mx_open_file_lock (const char *path, const char *mode)
+void mx_unlink_empty (const char *path)
 {
-  FILE *f;
+  int fd;
+#ifndef USE_DOTLOCK
+  struct stat sb;
+#endif
 
-  if ((f = safe_fopen (path, mode)) != NULL)
+  if ((fd = open (path, O_RDWR)) == -1)
+    return;
+
+  if (mx_lock_file (path, fd, 1, 0, 1) == -1)
   {
-    if (mx_lock_file (path, fileno (f), *mode != 'r', 1, 1) != 0)
-    {
-      fclose (f);
-      f = NULL;
-    }
+    close (fd);
+    return;
   }
 
-  return (f);
+#ifdef USE_DOTLOCK
+  invoke_dotlock (path, fd, DL_FL_UNLINK, 1);
+#else
+  if (fstat (fd, &sb) == 0 && sb.st_size == 0)
+    unlink (path);
+#endif
+
+  mx_unlock_file (path, fd, 0);
+  close (fd);
 }
 
 /* try to figure out what type of mailbox ``path'' is
@@ -367,7 +311,7 @@ FILE *mx_open_file_lock (const char *path, const char *mode)
 
 #ifdef USE_IMAP
 
-static int mx_is_imap(const char *p)
+int mx_is_imap(const char *p)
 {
   return p && (*p == '{');
 }
@@ -390,7 +334,6 @@ int mx_get_magic (const char *path)
   {
     dprint (1, (debugfile, "mx_get_magic(): unable to stat %s: %s (errno %d).\n",
 		path, strerror (errno), errno));
-    mutt_perror (path);
     return (-1);
   }
 
@@ -411,6 +354,20 @@ int mx_get_magic (const char *path)
     snprintf (tmp, sizeof (tmp), "%s/.xmhcache", path);
     if (access (tmp, F_OK) == 0)
       return (M_MH);
+    
+    snprintf (tmp, sizeof (tmp), "%s/.mew_cache", path);
+    if (access (tmp, F_OK) == 0)
+      return (M_MH);
+
+    /* 
+     * ok, this isn't an mh folder, but mh mode can be used to read
+     * Usenet news from the spool. ;-) 
+     */
+
+    snprintf (tmp, sizeof (tmp), "%s/.overview", path);
+    if (access (tmp, F_OK) == 0)
+      return (M_MH);
+
   }
   else if (st.st_size == 0)
   {
@@ -427,11 +384,11 @@ int mx_get_magic (const char *path)
 #endif
 
     fgets (tmp, sizeof (tmp), f);
-    if (strncmp ("From ", tmp, 5) == 0)
+    if (mutt_strncmp ("From ", tmp, 5) == 0)
       magic = M_MBOX;
-    else if (strcmp (MMDF_SEP, tmp) == 0)
+    else if (mutt_strcmp (MMDF_SEP, tmp) == 0)
       magic = M_MMDF;
-    fclose (f);
+    safe_fclose (&f);
 #ifndef BUFFY_SIZE
     /* need to restore the times here, the file was not really accessed,
      * only the type was accessed.  This is important, because detection
@@ -458,14 +415,16 @@ int mx_get_magic (const char *path)
  */
 int mx_set_magic (const char *s)
 {
-  if (strcasecmp (s, "mbox") == 0)
+  if (mutt_strcasecmp (s, "mbox") == 0)
     DefaultMagic = M_MBOX;
-  else if (strcasecmp (s, "mmdf") == 0)
+  else if (mutt_strcasecmp (s, "mmdf") == 0)
     DefaultMagic = M_MMDF;
-  else if (strcasecmp (s, "mh") == 0)
+  else if (mutt_strcasecmp (s, "mh") == 0)
     DefaultMagic = M_MH;
-  else if (strcasecmp (s, "maildir") == 0)
+  else if (mutt_strcasecmp (s, "maildir") == 0)
     DefaultMagic = M_MAILDIR;
+  else if (mutt_strcasecmp (s, "kendra") == 0)
+    DefaultMagic = M_KENDRA;
   else
     return (-1);
 
@@ -492,7 +451,7 @@ static int mx_open_mailbox_append (CONTEXT *ctx)
     switch (ctx->magic)
     {
       case 0:
-	mutt_error ("%s is not a mailbox.", ctx->path);
+	mutt_error (_("%s is not a mailbox."), ctx->path);
 	/* fall through */
       case -1:
 	return (-1);
@@ -508,7 +467,7 @@ static int mx_open_mailbox_append (CONTEXT *ctx)
 
       if (mkdir (ctx->path, S_IRWXU))
       {
-	mutt_perror (tmp);
+	mutt_perror (ctx->path);
 	return (-1);
       }
 
@@ -573,6 +532,11 @@ static int mx_open_mailbox_append (CONTEXT *ctx)
       {
 	if (!ctx->fp)
 	  mutt_perror (ctx->path);
+	else
+	{
+	  mutt_error (_("Couldn't lock %s\n"), ctx->path);
+	  safe_fclose (&ctx->fp);
+	}
 	return (-1);
       }
       fseek (ctx->fp, 0, 2);
@@ -611,6 +575,7 @@ CONTEXT *mx_open_mailbox (const char *path, int flags, CONTEXT *pctx)
   ctx->path = safe_strdup (path);
 
   ctx->msgnotreadyet = -1;
+  ctx->collapsed = 0;
   
   if (flags & M_QUIET)
     ctx->quiet = 1;
@@ -629,19 +594,22 @@ CONTEXT *mx_open_mailbox (const char *path, int flags, CONTEXT *pctx)
     return ctx;
   }
 
-  switch (ctx->magic = mx_get_magic (path))
+  ctx->magic = mx_get_magic (path);
+  
+  if(ctx->magic == 0)
+    mutt_error (_("%s is not a mailbox."), path);
+
+  if(ctx->magic == -1)
+    mutt_perror(path);
+  
+  if(ctx->magic <= 0)
   {
-    case 0:
-      mutt_error ("%s is not a mailbox.", path);
-      /* fall through */
-
-    case -1:
-      mx_fastclose_mailbox (ctx);
-      if (!pctx)
-	free (ctx);
-      return (NULL);
+    mx_fastclose_mailbox (ctx);
+    if (!pctx)
+      FREE (&ctx);
+    return (NULL);
   }
-
+  
   /* if the user has a `push' command in their .muttrc, or in a folder-hook,
    * it will cause the progress messages not to be displayed because
    * mutt_refresh() will think we are in the middle of a macro.  so set a
@@ -650,11 +618,11 @@ CONTEXT *mx_open_mailbox (const char *path, int flags, CONTEXT *pctx)
   set_option (OPTFORCEREFRESH);
 
   /* create hash tables */
-  ctx->id_hash = hash_create (257);
-  ctx->subj_hash = hash_create (257);
+  ctx->id_hash = hash_create (1031);
+  ctx->subj_hash = hash_create (1031);
 
   if (!ctx->quiet)
-    mutt_message ("Reading %s...", ctx->path);
+    mutt_message (_("Reading %s..."), ctx->path);
 
   switch (ctx->magic)
   {
@@ -710,6 +678,9 @@ CONTEXT *mx_open_mailbox (const char *path, int flags, CONTEXT *pctx)
 void mx_fastclose_mailbox (CONTEXT *ctx)
 {
   int i;
+
+  if(!ctx) 
+    return;
   
 #ifdef USE_IMAP
   if (ctx->magic == M_IMAP)
@@ -727,13 +698,12 @@ void mx_fastclose_mailbox (CONTEXT *ctx)
   safe_free ((void **) &ctx->pattern);
   if (ctx->limit_pattern) 
     mutt_pattern_free (&ctx->limit_pattern);
-  if (ctx->fp)
-    fclose (ctx->fp);
+  safe_fclose (&ctx->fp);
   memset (ctx, 0, sizeof (CONTEXT));
 }
 
 /* save changes to disk */
-static int sync_mailbox (CONTEXT *ctx)
+static int sync_mailbox (CONTEXT *ctx, int *index_hint)
 {
 #ifdef BUFFY_SIZE
   BUFFY *tmp = NULL;
@@ -741,12 +711,13 @@ static int sync_mailbox (CONTEXT *ctx)
   int rc = -1;
 
   if (!ctx->quiet)
-    mutt_message ("Writing %s...", ctx->path);
+    mutt_message (_("Writing %s..."), ctx->path);
+
   switch (ctx->magic)
   {
     case M_MBOX:
     case M_MMDF:
-      rc = mbox_sync_mailbox (ctx);
+      rc = mbox_sync_mailbox (ctx, index_hint);
 #ifdef BUFFY_SIZE
       tmp = mutt_find_mailbox (ctx->path);
 #endif
@@ -754,16 +725,22 @@ static int sync_mailbox (CONTEXT *ctx)
       
     case M_MH:
     case M_MAILDIR:
-      rc = mh_sync_mailbox (ctx);
+      rc = mh_sync_mailbox (ctx, index_hint);
       break;
       
 #ifdef USE_IMAP
     case M_IMAP:
-      rc = imap_sync_mailbox (ctx);
+      /* extra argument means EXPUNGE */
+      rc = imap_sync_mailbox (ctx, 1, index_hint);
       break;
 #endif /* USE_IMAP */
   }
 
+#if 0
+  if (!ctx->quiet && !ctx->shutup && rc == -1)
+    mutt_error ( _("Could not synchronize mailbox %s!"), ctx->path);
+#endif
+  
 #ifdef BUFFY_SIZE
   if (tmp && tmp->new == 0)
     mutt_update_mailbox (tmp);
@@ -772,14 +749,19 @@ static int sync_mailbox (CONTEXT *ctx)
 }
 
 /* save changes and close mailbox */
-int mx_close_mailbox (CONTEXT *ctx)
+int mx_close_mailbox (CONTEXT *ctx, int *index_hint)
 {
   int i, move_messages = 0, purge = 1, read_msgs = 0;
+  int check;
   int isSpool = 0;
   CONTEXT f;
   char mbox[_POSIX_PATH_MAX];
   char buf[SHORT_STRING];
 
+  if (!ctx) return 0;
+
+  ctx->closing = 1;
+
   if (ctx->readonly || ctx->dontwrite)
   {
     /* mailbox is readonly or we don't want to write */
@@ -814,14 +796,14 @@ int mx_close_mailbox (CONTEXT *ctx)
     }
     else
     {
-      strfcpy (mbox, Inbox, sizeof (mbox));
+      strfcpy (mbox, NONULL(Inbox), sizeof (mbox));
       isSpool = mutt_is_spool (ctx->path) && !mutt_is_spool (mbox);
     }
     mutt_expand_path (mbox, sizeof (mbox));
 
     if (isSpool)
     {
-      snprintf (buf, sizeof (buf), "Move read messages to %s?", mbox);
+      snprintf (buf, sizeof (buf), _("Move read messages to %s?"), mbox);
       if ((move_messages = query_quadoption (OPT_MOVE, buf)) == -1)
 	return (-1);
     }
@@ -829,12 +811,17 @@ int mx_close_mailbox (CONTEXT *ctx)
 
   if (ctx->deleted)
   {
-    snprintf (buf, sizeof (buf), "Purge %d deleted message%s?",
-	      ctx->deleted, ctx->deleted == 1 ? "" : "s");
+    snprintf (buf, sizeof (buf), ctx->deleted == 1
+	     ? _("Purge %d deleted message?") : _("Purge %d deleted messages?"),
+	      ctx->deleted);
     if ((purge = query_quadoption (OPT_DELETE, buf)) < 0)
       return (-1);
   }
 
+#ifdef USE_IMAP
+  /* IMAP servers manage the OLD flag themselves */
+  if (ctx->magic != M_IMAP)
+#endif
   if (option (OPTMARKOLD))
   {
     for (i = 0; i < ctx->msgcount; i++)
@@ -846,91 +833,213 @@ int mx_close_mailbox (CONTEXT *ctx)
 
   if (move_messages)
   {
-    if (mx_open_mailbox (mbox, M_APPEND, &f) == NULL)
-      return (-1);
+    mutt_message (_("Moving read messages to %s..."), mbox);
 
-    mutt_message ("Moving read messages to %s...", mbox);
-
-    for (i = 0; i < ctx->msgcount; i++)
+#ifdef USE_IMAP
+    /* try to use server-side copy first */
+    i = 1;
+    
+    if (ctx->magic == M_IMAP && mx_is_imap (mbox))
     {
-      if (ctx->hdrs[i]->read && !ctx->hdrs[i]->deleted)
-      {
-	mutt_append_message (&f, ctx, ctx->hdrs[i], 0, CH_UPDATE_LEN);
-	ctx->hdrs[i]->deleted = 1;
-	ctx->deleted++;
-      }
+      /* tag messages for moving, and clear old tags, if any */
+      for (i = 0; i < ctx->msgcount; i++)
+	if (ctx->hdrs[i]->read && !ctx->hdrs[i]->deleted)
+	  ctx->hdrs[i]->tagged = 1;
+	else
+	  ctx->hdrs[i]->tagged = 0;
+      
+      i = imap_copy_messages (ctx, NULL, mbox, 1);
     }
+    
+    if (i == 0) /* success */
+      mutt_clear_error ();
+    else if (i == -1) /* horrible error, bail */
+      return -1;
+    else /* use regular append-copy mode */
+#endif
+    {
+      if (mx_open_mailbox (mbox, M_APPEND, &f) == NULL)
+	return -1;
 
-    mx_close_mailbox (&f);
+      for (i = 0; i < ctx->msgcount; i++)
+      {
+	if (ctx->hdrs[i]->read && !ctx->hdrs[i]->deleted)
+        {
+	  if (mutt_append_message (&f, ctx, ctx->hdrs[i], 0, CH_UPDATE_LEN) == 0)
+	  {
+	    ctx->hdrs[i]->deleted = 1;
+	    ctx->deleted++;
+	  }
+	  else
+	  {
+	    mx_close_mailbox (&f, NULL);
+	    return -1;
+	  }
+	}
+      }
+    
+      mx_close_mailbox (&f, NULL);
+    }
+    
   }
   else if (!ctx->changed && ctx->deleted == 0)
   {
-    mutt_message ("Mailbox is unchanged.");
+    mutt_message _("Mailbox is unchanged.");
     mx_fastclose_mailbox (ctx);
     return 0;
   }
   
-  if (!purge)
+#ifdef USE_IMAP
+  /* allow IMAP to preserve the deleted flag across sessions */
+  if (ctx->magic == M_IMAP)
   {
-    for (i = 0; i < ctx->msgcount; i++)
-      ctx->hdrs[i]->deleted = 0;
-    ctx->deleted = 0;
+    if ((check = imap_sync_mailbox (ctx, purge, index_hint)) != 0)
+      return check;
   }
-
-  if (ctx->changed || ctx->deleted)
+  else
+#endif
   {
-    if (sync_mailbox (ctx) == -1)
-      return (-1);
+    if (!purge)
+    {
+      for (i = 0; i < ctx->msgcount; i++)
+        ctx->hdrs[i]->deleted = 0;
+      ctx->deleted = 0;
+    }
+
+    if (ctx->changed || ctx->deleted)
+    {
+      if ((check = sync_mailbox (ctx, index_hint)) != 0)
+	return check;
+    }
   }
 
   if (move_messages)
-    mutt_message ("%d kept, %d moved, %d deleted.",
-		  ctx->msgcount - ctx->deleted, read_msgs, ctx->deleted);
+     mutt_message (_("%d kept, %d moved, %d deleted."),
+       ctx->msgcount - ctx->deleted, read_msgs, ctx->deleted);
   else
-    mutt_message ("%d kept, %d deleted.",
-		  ctx->msgcount - ctx->deleted, ctx->deleted);
+    mutt_message (_("%d kept, %d deleted."),
+      ctx->msgcount - ctx->deleted, ctx->deleted);
 
   if (ctx->msgcount == ctx->deleted &&
       (ctx->magic == M_MMDF || ctx->magic == M_MBOX) &&
-      strcmp (ctx->path, Spoolfile) != 0 && !option (OPTSAVEEMPTY))
-    unlink (ctx->path);
+      !mutt_is_spool(ctx->path) && !option (OPTSAVEEMPTY))
+    mx_unlink_empty (ctx->path);
 
   mx_fastclose_mailbox (ctx);
 
   return 0;
 }
 
+
+/* update a Context structure's internal tables. */
+
+void mx_update_tables(CONTEXT *ctx, int committing)
+{
+  int i, j;
+  
+  /* update memory to reflect the new state of the mailbox */
+  ctx->vcount = 0;
+  ctx->vsize = 0;
+  ctx->tagged = 0;
+  ctx->deleted = 0;
+  ctx->new = 0;
+  ctx->unread = 0;
+  ctx->changed = 0;
+  ctx->flagged = 0;
+#define this_body ctx->hdrs[j]->content
+  for (i = 0, j = 0; i < ctx->msgcount; i++)
+  {
+    if ((committing && !ctx->hdrs[i]->deleted) || 
+	(!committing && ctx->hdrs[i]->active))
+    {
+      if (i != j)
+      {
+	ctx->hdrs[j] = ctx->hdrs[i];
+	ctx->hdrs[i] = NULL;
+      }
+      ctx->hdrs[j]->msgno = j;
+      if (ctx->hdrs[j]->virtual != -1)
+      {
+	ctx->v2r[ctx->vcount] = j;
+	ctx->hdrs[j]->virtual = ctx->vcount++;
+	ctx->vsize += this_body->length + this_body->offset -
+	  this_body->hdr_offset;
+      }
+
+      if(committing)
+	ctx->hdrs[j]->changed = 0;
+      else
+      {
+	if (ctx->hdrs[j]->deleted)
+	  ctx->deleted++;
+	if (ctx->hdrs[j]->changed)
+	  ctx->changed++;
+      }
+
+      if (ctx->hdrs[j]->tagged)
+	ctx->tagged++;
+      if (ctx->hdrs[j]->flagged)
+	ctx->flagged++;
+      if (!ctx->hdrs[j]->read)
+      { 
+	ctx->unread++;
+	if (!ctx->hdrs[j]->old)
+	  ctx->new++;
+      } 
+
+      j++;
+    }
+    else
+    {
+      if (ctx->magic == M_MH || ctx->magic == M_MAILDIR)
+	ctx->size -= (ctx->hdrs[i]->content->length +
+		      ctx->hdrs[i]->content->offset -
+		      ctx->hdrs[i]->content->hdr_offset);
+      /* remove message from the hash tables */
+      if (ctx->hdrs[i]->env->real_subj)
+	hash_delete (ctx->subj_hash, ctx->hdrs[i]->env->real_subj, ctx->hdrs[i], NULL);
+      if (ctx->hdrs[i]->env->message_id)
+	hash_delete (ctx->id_hash, ctx->hdrs[i]->env->message_id, ctx->hdrs[i], NULL);
+      mutt_free_header (&ctx->hdrs[i]);
+    }
+  }
+#undef this_body
+  ctx->msgcount = j;
+}
+
+
 /* save changes to mailbox
  *
  * return values:
  *	0		success
  *	-1		error
  */
-int mx_sync_mailbox (CONTEXT *ctx)
+int mx_sync_mailbox (CONTEXT *ctx, int *index_hint)
 {
-  int rc, i, j;
+  int rc, i;
+  int purge = 1;
 
   if (ctx->dontwrite)
   {
     char buf[STRING], tmp[STRING];
     if (km_expand_key (buf, sizeof(buf),
                        km_find_func (MENU_MAIN, OP_TOGGLE_WRITE)))
-      snprintf (tmp, sizeof(tmp), " Press '%s' to toggle write", buf);
+      snprintf (tmp, sizeof(tmp), _(" Press '%s' to toggle write"), buf);
     else
-      strfcpy (tmp, "Use 'toggle-write' to re-enable write!", sizeof(tmp));
+      strfcpy (tmp, _("Use 'toggle-write' to re-enable write!"), sizeof(tmp));
 
-    mutt_error ("Mailbox is marked unwritable. %s", tmp);
+    mutt_error (_("Mailbox is marked unwritable. %s"), tmp);
     return -1;
   }
   else if (ctx->readonly)
   {
-    mutt_error ("Mailbox is read-only.");
+    mutt_error _("Mailbox is read-only.");
     return -1;
   }
 
   if (!ctx->changed && !ctx->deleted)
   {
-    mutt_message ("Mailbox is unchanged.");
+    mutt_message _("Mailbox is unchanged.");
     return (0);
   }
 
@@ -938,162 +1047,67 @@ int mx_sync_mailbox (CONTEXT *ctx)
   {
     char buf[SHORT_STRING];
 
-    snprintf (buf, sizeof (buf), "Purge %d deleted message%s?",
-	      ctx->deleted, ctx->deleted == 1 ? "" : "s");
-    if ((rc = query_quadoption (OPT_DELETE, buf)) < 0)
+    snprintf (buf, sizeof (buf), ctx->deleted == 1
+	     ? _("Purge %d deleted message?") : _("Purge %d deleted messages?"),
+	      ctx->deleted);
+    if ((purge = query_quadoption (OPT_DELETE, buf)) < 0)
       return (-1);
-    else if (rc == M_NO)
+    else if (purge == M_NO)
     {
       if (!ctx->changed)
 	return 0; /* nothing to do! */
-      for (i = 0 ; i < ctx->msgcount ; i++)
-	ctx->hdrs[i]->deleted = 0;
-      ctx->deleted = 0;
+#ifdef USE_IMAP
+      /* let IMAP servers hold on to D flags */
+      if (ctx->magic != M_IMAP)
+#endif
+      {
+        for (i = 0 ; i < ctx->msgcount ; i++)
+          ctx->hdrs[i]->deleted = 0;
+        ctx->deleted = 0;
+      }
     }
   }
 
-  if ((rc = sync_mailbox (ctx)) == 0)
+#ifdef USE_IMAP
+  if (ctx->magic == M_IMAP)
+    rc = imap_sync_mailbox (ctx, purge, index_hint);
+  else
+#endif
+    rc = sync_mailbox (ctx, index_hint);
+  if (rc == 0)
   {
-    mutt_message ("%d kept, %d deleted.", ctx->msgcount - ctx->deleted,
-		  ctx->deleted);
+#ifdef USE_IMAP
+    if (ctx->magic == M_IMAP && !purge)
+      mutt_message (_("%d kept."), ctx->msgcount);
+    else
+#endif
+    mutt_message (_("%d kept, %d deleted."), ctx->msgcount - ctx->deleted,
+      ctx->deleted);
+
     sleep (1); /* allow the user time to read the message */
 
     if (ctx->msgcount == ctx->deleted &&
 	(ctx->magic == M_MBOX || ctx->magic == M_MMDF) &&
-	strcmp (ctx->path, Spoolfile) != 0 && !option (OPTSAVEEMPTY))
+	!mutt_is_spool(ctx->path) && !option (OPTSAVEEMPTY))
     {
       unlink (ctx->path);
       mx_fastclose_mailbox (ctx);
       return 0;
     }
 
-    /* update memory to reflect the new state of the mailbox */
-    ctx->vcount = 0;
-    ctx->vsize = 0;
-    ctx->tagged = 0;
-    ctx->deleted = 0;
-    ctx->new = 0;
-    ctx->unread = 0;
-    ctx->changed = 0;
-    ctx->flagged = 0;
-#define this_body ctx->hdrs[j]->content
-    for (i = 0, j = 0; i < ctx->msgcount; i++)
+    /* if we haven't deleted any messages, we don't need to resort */
+    if (purge)
     {
-      if (!ctx->hdrs[i]->deleted)
-      {
-	if (i != j)
-	{
-	  ctx->hdrs[j] = ctx->hdrs[i];
-	  ctx->hdrs[i] = NULL;
-	}
-	ctx->hdrs[j]->msgno = j;
-	if (ctx->hdrs[j]->virtual != -1)
-	{
-	  ctx->v2r[ctx->vcount] = j;
-	  ctx->hdrs[j]->virtual = ctx->vcount++;
-	  ctx->vsize += this_body->length + this_body->offset -
-	                this_body->hdr_offset;
-	}
-	ctx->hdrs[j]->changed = 0;
-	if (ctx->hdrs[j]->tagged)
-	  ctx->tagged++;
-	if (ctx->hdrs[j]->flagged)
-	  ctx->flagged++;
-	if (!ctx->hdrs[j]->read)
-	{ 
-	  ctx->unread++;
-	  if (!ctx->hdrs[j]->old)
-	    ctx->new++;
-	} 
-	j++;
-      }
-      else
-      {
-	if (ctx->magic == M_MH || ctx->magic == M_MAILDIR)
-	 ctx->size -= (ctx->hdrs[i]->content->length +
-	               ctx->hdrs[i]->content->offset -
-		       ctx->hdrs[i]->content->hdr_offset);
-	/* remove message from the hash tables */
-	if (ctx->hdrs[i]->env->real_subj)
-	  hash_delete (ctx->subj_hash, ctx->hdrs[i]->env->real_subj, ctx->hdrs[i], NULL);
-	if (ctx->hdrs[i]->env->message_id)
-	  hash_delete (ctx->id_hash, ctx->hdrs[i]->env->message_id, ctx->hdrs[i], NULL);
-	mutt_free_header (&ctx->hdrs[i]);
-      }
+      mx_update_tables(ctx, 1);
+      mutt_sort_headers (ctx, 1); /* rethread from scratch */
     }
-#undef this_body
-    ctx->msgcount = j;
-
-    mutt_sort_headers (ctx, 1); /* rethread from scratch */
   }
 
   return (rc);
 }
 
-int mh_open_new_message (MESSAGE *msg, CONTEXT *dest, HEADER *hdr)
-{
-  int hi = 1;
-  int fd, n;
-  char *cp;
-  char path[_POSIX_PATH_MAX];
-  DIR *dirp;
-  struct dirent *de;
 
-  do
-  {
-    if ((dirp = opendir (dest->path)) == NULL)
-    {
-      mutt_perror (dest->path);
-      return (-1);
-    }
-
-    /* figure out what the next message number is */
-    while ((de = readdir (dirp)) != NULL)
-    {
-      cp = de->d_name;
-      while (*cp)
-      {
-	if (!isdigit ((unsigned char) *cp))
-	  break;
-	cp++;
-      }
-      if (!*cp)
-      {
-	n = atoi (de->d_name);
-	if (n > hi)
-	  hi = n;
-      }
-    }
-    closedir (dirp);
-    hi++;
-    snprintf (path, sizeof (path), "%s/%d", dest->path, hi);
-    if ((fd = open (path, O_WRONLY | O_EXCL | O_CREAT, 0600)) == -1)
-    {
-      if (errno != EEXIST)
-      {
-	mutt_perror (path);
-	return (-1);
-      }
-    }
-  }
-  while (fd < 0);
-
-  if ((msg->fp = fdopen (fd, "w")) == NULL)
-    return (-1);
-
-  return 0;
-}
-
-int maildir_open_new_message (MESSAGE *msg, CONTEXT *dest, HEADER *hdr)
-{
-  char tmp[_POSIX_PATH_MAX];
-  char path[_POSIX_PATH_MAX];
-
-  maildir_create_filename (dest->path, hdr, path, tmp);
-  if ((msg->fp = safe_fopen (tmp, "w")) == NULL)
-    return (-1);
-  return 0;
-}
+/* {maildir,mh}_open_new_message are in mh.c. */
 
 int mbox_open_new_message (MESSAGE *msg, CONTEXT *dest, HEADER *hdr)
 {
@@ -1110,7 +1124,6 @@ int imap_open_new_message (MESSAGE *msg, CONTEXT *dest, HEADER *hdr)
   if ((msg->fp = safe_fopen (tmp, "w")) == NULL)
     return (-1);
   msg->path = safe_strdup(tmp);
-  msg->ctx = dest;
   return 0;
 }
 #endif
@@ -1159,8 +1172,8 @@ MESSAGE *mx_open_new_message (CONTEXT *dest, HEADER *hdr, int flags)
     if (dest->magic == M_MMDF)
       fputs (MMDF_SEP, msg->fp);
 
-    if ((msg->magic != M_MAILDIR) && (msg->magic != M_IMAP) && 
-	(flags & M_ADD_FROM))
+    if ((msg->magic == M_MBOX || msg->magic ==  M_MMDF) &&
+	flags & M_ADD_FROM)
     {
       if (hdr)
       {
@@ -1178,7 +1191,7 @@ MESSAGE *mx_open_new_message (CONTEXT *dest, HEADER *hdr, int flags)
       else
 	t = time (NULL);
 
-      fprintf (msg->fp, "From %s %s", p ? p->mailbox : Username, ctime (&t));
+      fprintf (msg->fp, "From %s %s", p ? p->mailbox : NONULL(Username), ctime (&t));
     }
   }
   else
@@ -1187,192 +1200,40 @@ MESSAGE *mx_open_new_message (CONTEXT *dest, HEADER *hdr, int flags)
   return msg;
 }
 
-int mutt_reopen_mailbox (CONTEXT *ctx, int *index_hint)
-{
-  int (*cmp_headers) (const HEADER *, const HEADER *) = NULL;
-  HEADER **old_hdrs;
-  int old_msgcount;
-  int msg_mod = 0;
-  int index_hint_set;
-  int i, j;
-  int rc = -1;
-
-  /* silent operations */
-  ctx->quiet = 1;
-  
-  mutt_message ("Reopening mailbox...");
-  
-  /* our heuristics require the old mailbox to be unsorted */
-  if (Sort != SORT_ORDER)
-  {
-    short old_sort;
-
-    old_sort = Sort;
-    Sort = SORT_ORDER;
-    mutt_sort_headers (ctx, 1);
-    Sort = old_sort;
-  }
-
-  /* save the old headers */
-  old_msgcount = ctx->msgcount;
-  old_hdrs = ctx->hdrs;
-
-  /* simulate a close */
-  hash_destroy (&ctx->id_hash, NULL);
-  hash_destroy (&ctx->subj_hash, NULL);
-  safe_free ((void **) &ctx->v2r);
-  if (ctx->readonly)
-  {
-    for (i = 0; i < ctx->msgcount; i++)
-      mutt_free_header (&(ctx->hdrs[i])); /* nothing to do! */
-      safe_free ((void **) &ctx->hdrs);
-  }
-  else
-    ctx->hdrs = NULL;
-
-  ctx->hdrmax = 0;	/* force allocation of new headers */
-  ctx->msgcount = 0;
-  ctx->vcount = 0;
-  ctx->tagged = 0;
-  ctx->deleted = 0;
-  ctx->new = 0;
-  ctx->unread = 0;
-  ctx->flagged = 0;
-  ctx->changed = 0;
-  ctx->id_hash = hash_create (257);
-  ctx->subj_hash = hash_create (257);
-
-  switch (ctx->magic)
-  {
-    case M_MBOX:
-      fseek (ctx->fp, 0, 0);
-      cmp_headers = mbox_strict_cmp_headers;
-      rc = mbox_parse_mailbox (ctx);
-      break;
-
-    case M_MMDF:
-      fseek (ctx->fp, 0, 0);
-      cmp_headers = mbox_strict_cmp_headers;
-      rc = mmdf_parse_mailbox (ctx);
-      break;
-
-    case M_MH:
-      /* cmp_headers = mh_strict_cmp_headers; */
-      rc = mh_read_dir (ctx, NULL);
-      break;
-
-    case M_MAILDIR:
-      /* cmp_headers = maildir_strict_cmp_headers; */
-      rc = maildir_read_dir (ctx);
-      break;
-
-    default:
-      rc = -1;
-      break;
-  }
-  
-  if (rc == -1)
-  {
-    /* free the old headers */
-    for (j = 0; j < old_msgcount; j++)
-      mutt_free_header (&(old_hdrs[j]));
-    safe_free ((void **) &old_hdrs);
-
-    ctx->quiet = 0;
-    return (-1);
-  }
-
-  /* now try to recover the old flags */
-
-  index_hint_set = (index_hint == NULL);
-
-  if (!ctx->readonly)
-  {
-    for (i = 0; i < ctx->msgcount; i++)
-    {
-      int found = 0;
-
-      /* some messages have been deleted, and new  messages have been
-       * appended at the end; the heuristic is that old messages have then
-       * "advanced" towards the beginning of the folder, so we begin the
-       * search at index "i"
-       */
-      for (j = i; j < old_msgcount; j++)
-      {
-	if (old_hdrs[j] == NULL)
-	  continue;
-	if (cmp_headers (ctx->hdrs[i], old_hdrs[j]))
-	{
-	  found = 1;
-	  break;
-	}
-      }
-      if (!found)
-      {
-	for (j = 0; j < i; j++)
-	{
-	  if (old_hdrs[j] == NULL)
-	    continue;
-	  if (cmp_headers (ctx->hdrs[i], old_hdrs[j]))
-	  {
-	    found = 1;
-	    break;
-	  }
-	}
-      }
-
-      if (found)
-      {
-	/* this is best done here */
-	if (!index_hint_set && *index_hint == j)
-	  *index_hint = i;
-
-	if (old_hdrs[j]->changed)
-	{
-	  /* Only update the flags if the old header was changed;
-	   * otherwise, the header may have been modified externally,
-	   * and we don't want to lose _those_ changes
-	   */
-	  mutt_set_flag (ctx, ctx->hdrs[i], M_FLAG, old_hdrs[j]->flagged);
-	  mutt_set_flag (ctx, ctx->hdrs[i], M_REPLIED, old_hdrs[j]->replied);
-	  mutt_set_flag (ctx, ctx->hdrs[i], M_OLD, old_hdrs[j]->old);
-	  mutt_set_flag (ctx, ctx->hdrs[i], M_READ, old_hdrs[j]->read);
-	}
-	mutt_set_flag (ctx, ctx->hdrs[i], M_DELETE, old_hdrs[j]->deleted);
-	mutt_set_flag (ctx, ctx->hdrs[i], M_TAG, old_hdrs[j]->tagged);
-
-	/* we don't need this header any more */
-	mutt_free_header (&(old_hdrs[j]));
-      }
-    }
-
-    /* free the remaining old headers */
-    for (j = 0; j < old_msgcount; j++)
-    {
-      if (old_hdrs[j])
-      {
-	mutt_free_header (&(old_hdrs[j]));
-	msg_mod = 1;
-      }
-    }
-    safe_free ((void **) &old_hdrs);
-  }
-
-  ctx->quiet = 0;
-
-  return ((ctx->changed || msg_mod) ? M_REOPENED : M_NEW_MAIL);
-}
-
 /* check for new mail */
-int mx_check_mailbox (CONTEXT *ctx, int *index_hint)
+int mx_check_mailbox (CONTEXT *ctx, int *index_hint, int lock)
 {
+  int rc;
+
   if (ctx)
   {
+    if (ctx->locked) lock = 0;
+
     switch (ctx->magic)
     {
       case M_MBOX:
       case M_MMDF:
-	return (mbox_check_mailbox (ctx, index_hint));
+
+	if (lock)
+	{
+	  mutt_block_signals ();
+	  if (mbox_lock_mailbox (ctx, 0, 0) == -1)
+	  {
+	    mutt_unblock_signals ();
+	    return M_LOCKED;
+	  }
+	}
+	
+	rc = mbox_check_mailbox (ctx, index_hint);
+
+	if (lock)
+	{
+	  mutt_unblock_signals ();
+	  mbox_unlock_mailbox (ctx);
+	}
+	
+	return rc;
+
 
       case M_MH:
       case M_MAILDIR:
@@ -1404,84 +1265,131 @@ MESSAGE *mx_open_message (CONTEXT *ctx, int msgno)
 
     case M_MH:
     case M_MAILDIR:
+    {
+      HEADER *cur = ctx->hdrs[msgno];
+      char path[_POSIX_PATH_MAX];
+      
+      snprintf (path, sizeof (path), "%s/%s", ctx->path, cur->path);
+      if ((msg->fp = fopen (path, "r")) == NULL)
       {
-	HEADER *cur = ctx->hdrs[msgno];
-	char path[_POSIX_PATH_MAX];
-
-	snprintf (path, sizeof (path), "%s/%s", ctx->path, cur->path);
-	if ((msg->fp = fopen (path, "r")) == NULL)
-	{
-	  mutt_perror (path);
-	  dprint (1, (debugfile, "mx_open_message: fopen: %s: %s (errno %d).\n",
-		      path, strerror (errno), errno));
-	  free (msg);
-	  msg = NULL;
-	}
+	mutt_perror (path);
+	dprint (1, (debugfile, "mx_open_message: fopen: %s: %s (errno %d).\n",
+		    path, strerror (errno), errno));
+	FREE (&msg);
       }
-      break;
-
+    }
+    break;
+    
 #ifdef USE_IMAP
     case M_IMAP:
+    {
       if (imap_fetch_message (msg, ctx, msgno) != 0)
-      {
-	free (msg);
-	msg = NULL;
-      }
+	FREE (&msg);
       break;
+    }
 #endif /* USE_IMAP */
 
     default:
-
       dprint (1, (debugfile, "mx_open_message(): function not implemented for mailbox type %d.\n", ctx->magic));
-      safe_free ((void **) &msg);
+      FREE (&msg);
       break;
   }
   return (msg);
 }
 
+/* commit a message to a folder */
+
+int mx_commit_message (MESSAGE *msg, CONTEXT *ctx)
+{
+  int r = 0;
+
+  if (!(msg->write && ctx->append))
+  {
+    dprint (1, (debugfile, "mx_commit_message(): msg->write = %d, ctx->append = %d\n",
+		msg->write, ctx->append));
+    return -1;
+  }
+
+  switch (msg->magic)
+  {
+    case M_MMDF:
+    {
+      if (fputs (MMDF_SEP, msg->fp) == EOF)
+	r = -1;
+      break;
+    }
+    
+    case M_MBOX:
+    {
+      if (fputc ('\n', msg->fp) == EOF)
+	r = -1;
+      break;
+    }
+
+    case M_KENDRA:
+    {
+      if (fputs (KENDRA_SEP, msg->fp) == EOF)
+	r = -1;
+      break;
+    }
+
+#ifdef USE_IMAP
+    case M_IMAP:
+    {
+      if ((r = safe_fclose (&msg->fp)) == 0)
+	r = imap_append_message (ctx, msg);
+      break;
+    }
+#endif
+    
+    case M_MAILDIR:
+    {
+      r = maildir_commit_message (ctx, msg, NULL);
+      break;
+    }
+    
+    case M_MH:
+    {
+      r = mh_commit_message (ctx, msg, NULL);
+      break;
+    }
+  }
+  
+  if (r == 0 && (ctx->magic == M_MBOX || ctx->magic == M_MMDF || ctx->magic == M_KENDRA)
+      && fflush (msg->fp) == EOF)
+  {
+    mutt_perror _("Can't write message");
+    r = -1;
+  }
+ 
+  return r;
+}
+
 /* close a pointer to a message */
 int mx_close_message (MESSAGE **msg)
 {
   int r = 0;
 
-  if ((*msg)->write)
-  {
-    /* add the message terminator */
-    switch ((*msg)->magic)
-    {
-      case M_MMDF:
-	fputs (MMDF_SEP, (*msg)->fp);
-	break;
-
-      case M_MBOX:
-	fputc ('\n', (*msg)->fp);
-	break;
-    }
-  }
-
-  switch ((*msg)->magic) 
-  {
-    case M_MH:
-    case M_MAILDIR:
-      r = fclose ((*msg)->fp);
-      break;
-
+  if ((*msg)->magic == M_MH || (*msg)->magic == M_MAILDIR
 #ifdef USE_IMAP
-    case M_IMAP:
-      r = fclose ((*msg)->fp);
-      if ((*msg)->write && (*msg)->ctx->append)
-      {
-	r = imap_append_message ((*msg)->ctx, *msg);
-	unlink ((*msg)->path);
-      }
+      || (*msg)->magic == M_IMAP
 #endif
+      )
+  {
+    r = safe_fclose (&(*msg)->fp);
+  }
+  else
+    (*msg)->fp = NULL;
 
-    default:
-      break;
+  if ((*msg)->path)
+  {
+    dprint (1, (debugfile, "mx_close_message (): unlinking %s\n",
+		(*msg)->path));
+    unlink ((*msg)->path);
+    FREE (&(*msg)->path);
   }
 
-  free (*msg);
-  *msg = NULL;
+  FREE (msg);
   return (r);
 }
 
@@ -1516,21 +1424,11 @@ void mx_update_context (CONTEXT *ctx)
 
 
 
+#ifdef HAVE_PGP
+  /* NOTE: this _must_ be done before the check for mailcap! */
+  h->pgp = pgp_query (h->content);
+#endif /* HAVE_PGP */
 
-
-
-    if (mutt_needs_mailcap (h->content))
-      h->mailcap = 1;
-  if (h->flagged)
-    ctx->flagged++;
-  if (h->deleted)
-    ctx->deleted++;
-  if (!h->read)
-  {
-    ctx->unread++;
-    if (!h->old)
-      ctx->new++;
-  }
   if (!ctx->pattern)
   {
     ctx->v2r[ctx->vcount] = ctx->msgcount;
@@ -1549,7 +1447,8 @@ void mx_update_context (CONTEXT *ctx)
     if (h2)
     {
       h2->superseded = 1;
-      mutt_score_message (h2);
+      if (option (OPTSCORE)) 
+	mutt_score_message (ctx, h2, 1);
     }
   }
 
@@ -1559,5 +1458,19 @@ void mx_update_context (CONTEXT *ctx)
   if (h->env->real_subj)
     hash_insert (ctx->subj_hash, h->env->real_subj, h, 1);
 
-  mutt_score_message (h);
+  if (option (OPTSCORE)) 
+    mutt_score_message (ctx, h, 0);
+  
+  if (h->changed)
+    ctx->changed = 1;
+  if (h->flagged)
+    ctx->flagged++;
+  if (h->deleted)
+    ctx->deleted++;
+  if (!h->read)
+  {
+    ctx->unread++;
+    if (!h->old)
+      ctx->new++;
+  }
 }
