@@ -5,6 +5,6 @@ BODY *mutt_parse_multipart (FILE *, const char *, long, int);
 BODY *mutt_parse_messageRFC822 (FILE *, BODY *);
 BODY *mutt_read_mime_header (FILE *, int);
 
-ENVELOPE *mutt_read_rfc822_header (FILE *, HEADER *, short);
+ENVELOPE *mutt_read_rfc822_header (FILE *, HEADER *, short, short);
 
 time_t is_from (const char *, char *, size_t);
