@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>, and others
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */ 
 
 #include "mutt.h"
@@ -27,9 +27,13 @@
 #include <ctype.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#include <stdarg.h>
 
 
 
+#ifdef HAVE_PGP
+#include "pgp.h"
+#endif
 
 
 
@@ -57,8 +61,15 @@ Flags[] =
   { 'E', M_EXPIRED,		0,		NULL },
   { 'f', M_FROM,		0,		eat_regexp },
   { 'F', M_FLAG,		0,		NULL },
+#ifdef HAVE_PGP
+  { 'g', M_PGP_SIGN, 		0, 		NULL },
+  { 'G', M_PGP_ENCRYPT, 	0, 		NULL },
+#endif
   { 'h', M_HEADER,		M_FULL_MSG,	eat_regexp },
   { 'i', M_ID,			0,		eat_regexp },
+#ifdef HAVE_PGP
+  { 'k', M_PGP_KEY, 		0, 		NULL },
+#endif
   { 'L', M_ADDRESS,		0,		eat_regexp },
   { 'l', M_LIST,		0,		NULL },
   { 'm', M_MESSAGE,		0,		eat_range },
@@ -75,6 +86,7 @@ Flags[] =
   { 'T', M_TAG,			0,		NULL },
   { 't', M_TO,			0,		eat_regexp },
   { 'U', M_UNREAD,		0,		NULL },
+  { 'v', M_COLLAPSED,		0,		NULL },
   { 'x', M_REFERENCE,		0,		eat_regexp },
   { 'z', M_SIZE,		0,		eat_range },
   { 0 }
@@ -131,6 +143,7 @@ msg_search (CONTEXT *ctx, regex_t *rx, char *buf, size_t blen, int op, int msgno
       /* decode the header / body */
       memset (&s, 0, sizeof (s));
       s.fpin = msg->fp;
+      s.flags = M_CHARCONV;
       mutt_mktemp (tempfile);
       if ((s.fpout = safe_fopen (tempfile, "w+")) == NULL)
       {
@@ -147,6 +160,18 @@ msg_search (CONTEXT *ctx, regex_t *rx, char *buf, size_t blen, int op, int msgno
 
 
 
+#ifdef HAVE_PGP
+	if (h->pgp & PGPENCRYPT && !pgp_valid_passphrase())
+	{
+	  mx_close_message (&msg);
+	  if (fp)
+	  {
+	    fclose (fp);
+	    unlink (tempfile);
+	  }
+	  return (0);
+	}
+#endif
 
 
 
@@ -187,7 +212,7 @@ msg_search (CONTEXT *ctx, regex_t *rx, char *buf, size_t blen, int op, int msgno
 	match = 1;
 	break;
       }
-      lng -= strlen (buf);
+      lng -= mutt_strlen (buf);
     }
     
     mx_close_message (&msg);
@@ -211,7 +236,7 @@ int eat_regexp (pattern_t *pat, BUFFER *s, BUFFER *err)
   if (mutt_extract_token (&buf, s, M_TOKEN_PATTERN | M_TOKEN_COMMENT) != 0 ||
       !buf.data)
   {
-    snprintf (err->data, err->dsize, "Error in expression: %s", s->dptr);
+    snprintf (err->data, err->dsize, _("Error in expression: %s"), s->dptr);
     return (-1);
   }
   pat->rx = safe_malloc (sizeof (regex_t));
@@ -230,14 +255,17 @@ int eat_regexp (pattern_t *pat, BUFFER *s, BUFFER *err)
 int eat_range (pattern_t *pat, BUFFER *s, BUFFER *err)
 {
   char *tmp;
-
+  int do_exclusive = 0;
+  
+  if (*s->dptr == '<')
+    do_exclusive = 1;
   if ((*s->dptr != '-') && (*s->dptr != '<'))
   {
     /* range minimum */
     if (*s->dptr == '>')
     {
       pat->max = M_MAXRANGE;
-      pat->min = strtol (s->dptr + 1, &tmp, 0);
+      pat->min = strtol (s->dptr + 1, &tmp, 0) + 1; /* exclusive range */
     }
     else
       pat->min = strtol (s->dptr, &tmp, 0);
@@ -285,10 +313,13 @@ int eat_range (pattern_t *pat, BUFFER *s, BUFFER *err)
       pat->max *= 1048576;
       tmp++;
     }
+    if (do_exclusive)
+      (pat->max)--;
   }
   else
     pat->max = M_MAXRANGE;
 
+  SKIPWS (tmp);
   s->dptr = tmp;
   return 0;
 }
@@ -302,7 +333,7 @@ static const char *getDate (const char *s, struct tm *t, BUFFER *err)
   t->tm_mday = strtol (s, &p, 0);
   if (t->tm_mday < 1 || t->tm_mday > 31)
   {
-    snprintf (err->data, err->dsize, "Invalid day of month: %s", s);
+    snprintf (err->data, err->dsize, _("Invalid day of month: %s"), s);
     return NULL;
   }
   if (*p != '/')
@@ -316,7 +347,7 @@ static const char *getDate (const char *s, struct tm *t, BUFFER *err)
   t->tm_mon = strtol (p, &p, 0) - 1;
   if (t->tm_mon < 0 || t->tm_mon > 11)
   {
-    snprintf (err->data, err->dsize, "Invalid month: %s", p);
+    snprintf (err->data, err->dsize, _("Invalid month: %s"), p);
     return NULL;
   }
   if (*p != '/')
@@ -370,7 +401,7 @@ static int eat_date (pattern_t *pat, BUFFER *s, BUFFER *err)
   if (mutt_extract_token (&buffer, s, M_TOKEN_COMMENT | M_TOKEN_PATTERN) != 0
       || !buffer.data)
   {
-    strfcpy (err->data, "error in expression", err->dsize);
+    strfcpy (err->data, _("error in expression"), err->dsize);
     return (-1);
   }
 
@@ -525,6 +556,7 @@ pattern_t *mutt_pattern_comp (/* const */ char *s, int flags, BUFFER *err)
   pattern_t *tmp;
   pattern_t *last = NULL;
   int not = 0;
+  int alladdr = 0;
   int or = 0;
   int implicit = 1;	/* used to detect logical AND operator */
   struct pattern_flags *entry;
@@ -534,13 +566,17 @@ pattern_t *mutt_pattern_comp (/* const */ char *s, int flags, BUFFER *err)
 
   memset (&ps, 0, sizeof (ps));
   ps.dptr = s;
-  ps.dsize = strlen (s);
+  ps.dsize = mutt_strlen (s);
 
   while (*ps.dptr)
   {
     SKIPWS (ps.dptr);
     switch (*ps.dptr)
     {
+      case '^':
+	ps.dptr++;
+	alladdr = !alladdr;
+	break;
       case '!':
 	ps.dptr++;
 	not = !not;
@@ -550,7 +586,7 @@ pattern_t *mutt_pattern_comp (/* const */ char *s, int flags, BUFFER *err)
 	{
 	  if (!curlist)
 	  {
-	    snprintf (err->data, err->dsize, "error in pattern at: %s", ps.dptr);
+	    snprintf (err->data, err->dsize, _("error in pattern at: %s"), ps.dptr);
 	    return NULL;
 	  }
 	  if (curlist->next)
@@ -569,6 +605,7 @@ pattern_t *mutt_pattern_comp (/* const */ char *s, int flags, BUFFER *err)
 	ps.dptr++;
 	implicit = 0;
 	not = 0;
+	alladdr = 0;
 	break;
       case '~':
 	if (implicit && or)
@@ -584,7 +621,9 @@ pattern_t *mutt_pattern_comp (/* const */ char *s, int flags, BUFFER *err)
 
 	tmp = new_pattern ();
 	tmp->not = not;
+	tmp->alladdr = alladdr;
 	not = 0;
+	alladdr=0;
 
 	if (last)
 	  last->next = tmp;
@@ -595,13 +634,13 @@ pattern_t *mutt_pattern_comp (/* const */ char *s, int flags, BUFFER *err)
 	ps.dptr++; /* move past the ~ */
 	if ((entry = lookup_tag (*ps.dptr)) == NULL)
 	{
-	  snprintf (err->data, err->dsize, "%c: invalid command", *ps.dptr);
+	  snprintf (err->data, err->dsize, _("%c: invalid command"), *ps.dptr);
 	  mutt_pattern_free (&curlist);
 	  return NULL;
 	}
 	if (entry->class && (flags & entry->class) == 0)
 	{
-	  snprintf (err->data, err->dsize, "%c: not supported in this mode", *ps.dptr);
+	  snprintf (err->data, err->dsize, _("%c: not supported in this mode"), *ps.dptr);
 	  mutt_pattern_free (&curlist);
 	  return NULL;
 	}
@@ -614,7 +653,7 @@ pattern_t *mutt_pattern_comp (/* const */ char *s, int flags, BUFFER *err)
 	{
 	  if (!*ps.dptr)
 	  {
-	    snprintf (err->data, err->dsize, "missing parameter");
+	    snprintf (err->data, err->dsize, _("missing parameter"));
 	    mutt_pattern_free (&curlist);
 	    return NULL;
 	  }
@@ -630,7 +669,7 @@ pattern_t *mutt_pattern_comp (/* const */ char *s, int flags, BUFFER *err)
 	p = find_matching_paren (ps.dptr + 1);
 	if (*p != ')')
 	{
-	  snprintf (err->data, err->dsize, "mismatched parenthesis: %s", ps.dptr);
+	  snprintf (err->data, err->dsize, _("mismatched parenthesis: %s"), ps.dptr);
 	  mutt_pattern_free (&curlist);
 	  return NULL;
 	}
@@ -648,19 +687,21 @@ pattern_t *mutt_pattern_comp (/* const */ char *s, int flags, BUFFER *err)
 	else
 	  curlist = tmp;
 	last = tmp;
-	tmp->not = not;
+	tmp->not ^= not;
+	tmp->alladdr |= alladdr;
 	not = 0;
+	alladdr = 0;
 	ps.dptr = p + 1; /* restore location */
 	break;
       default:
-	snprintf (err->data, err->dsize, "error in pattern at: %s", ps.dptr);
+	snprintf (err->data, err->dsize, _("error in pattern at: %s"), ps.dptr);
 	mutt_pattern_free (&curlist);
 	return NULL;
     }
   }
   if (!curlist)
   {
-    strfcpy (err->data, "empty pattern", err->dsize);
+    strfcpy (err->data, _("empty pattern"), err->dsize);
     return NULL;
   }
   if (curlist->next)
@@ -691,15 +732,29 @@ perform_or (struct pattern_t *pat, pattern_exec_flag flags, CONTEXT *ctx, HEADER
   return 0;
 }
 
-static int match_adrlist (regex_t *rx, int match_personal, ADDRESS *a)
+static int match_adrlist (regex_t *rx, int match_personal, int alladdr,
+			  int n, ...)
 {
-  for (; a; a = a->next)
+  va_list ap;
+  ADDRESS *a;
+
+  va_start (ap, n);
+  for ( ; n ; n --)
   {
-    if ((a->mailbox && regexec (rx, a->mailbox, 0, NULL, 0) == 0) ||
-	(match_personal && a->personal && regexec (rx, a->personal, 0, NULL, 0) == 0))
-      return 1;
+    for (a = va_arg (ap, ADDRESS *) ; a ; a = a->next)
+    {
+      if (alladdr^
+	  ((a->mailbox && regexec (rx, a->mailbox, 0, NULL, 0) == 0) ||
+	   (match_personal && a->personal &&
+	    regexec (rx, a->personal, 0, NULL, 0) == 0)))
+      {
+	va_end (ap);
+	return (! alladdr); /* Found match, or non-match if alladdr */
+      }
+    }
   }
-  return 0;
+  va_end (ap);
+  return alladdr; /* No matches, or all matches if alladdr */
 }
 
 static int match_reference (regex_t *rx, LIST *refs)
@@ -710,12 +765,37 @@ static int match_reference (regex_t *rx, LIST *refs)
   return 0;
 }
 
-static int match_user (ADDRESS *p)
+int mutt_is_list_recipient (int alladdr, ADDRESS *a1, ADDRESS *a2)
 {
-  for (; p; p = p->next)
-    if (mutt_addr_is_user (p))
-      return 1;
-  return 0;
+  for (; a1 ; a1 = a1->next)
+    if (alladdr ^ mutt_is_subscribed_list (a1))
+      return (! alladdr);
+  for (; a2 ; a2 = a2->next)
+    if (alladdr ^ mutt_is_subscribed_list (a2))
+      return (! alladdr);
+  return alladdr;
+}
+
+int mutt_is_list_cc (int alladdr, ADDRESS *a1, ADDRESS *a2)
+{
+  for (; a1 ; a1 = a1->next)
+    if (alladdr ^ mutt_is_mail_list (a1))
+      return (! alladdr);
+  for (; a2 ; a2 = a2->next)
+    if (alladdr ^ mutt_is_mail_list (a2))
+      return (! alladdr);
+  return alladdr;
+}
+
+static int match_user (int alladdr, ADDRESS *a1, ADDRESS *a2)
+{
+  for (; a1 ; a1 = a1->next)
+    if (alladdr ^ mutt_addr_is_user (a1))
+      return (! alladdr);
+  for (; a2 ; a2 = a2->next)
+    if (alladdr ^ mutt_addr_is_user (a2))
+      return (! alladdr);
+  return alladdr;
 }
 
 /* flags
@@ -765,13 +845,17 @@ mutt_pattern_exec (struct pattern_t *pat, pattern_exec_flag flags, CONTEXT *ctx,
     case M_WHOLE_MSG:
       return (pat->not ^ msg_search (ctx, pat->rx, buf, sizeof (buf), pat->op, h->msgno));
     case M_SENDER:
-      return (pat->not ^ match_adrlist (pat->rx, flags & M_MATCH_FULL_ADDRESS, h->env->sender));
+      return (pat->not ^ match_adrlist (pat->rx, flags & M_MATCH_FULL_ADDRESS,
+					pat->alladdr, 1, h->env->sender));
     case M_FROM:
-      return (pat->not ^ match_adrlist (pat->rx, flags & M_MATCH_FULL_ADDRESS, h->env->from));
+      return (pat->not ^ match_adrlist (pat->rx, flags & M_MATCH_FULL_ADDRESS,
+					pat->alladdr, 1, h->env->from));
     case M_TO:
-      return (pat->not ^ match_adrlist (pat->rx, flags & M_MATCH_FULL_ADDRESS, h->env->to));
+      return (pat->not ^ match_adrlist (pat->rx, flags & M_MATCH_FULL_ADDRESS,
+					pat->alladdr, 1, h->env->to));
     case M_CC:
-      return (pat->not ^ match_adrlist (pat->rx, flags & M_MATCH_FULL_ADDRESS, h->env->cc));
+      return (pat->not ^ match_adrlist (pat->rx, flags & M_MATCH_FULL_ADDRESS,
+					pat->alladdr, 1, h->env->cc));
     case M_SUBJECT:
       return (pat->not ^ (h->env->subject && regexec (pat->rx, h->env->subject, 0, NULL, 0) == 0));
     case M_ID:
@@ -780,33 +864,52 @@ mutt_pattern_exec (struct pattern_t *pat, pattern_exec_flag flags, CONTEXT *ctx,
       return (pat->not ^ (h->score >= pat->min && (pat->max == M_MAXRANGE ||
 						   h->score <= pat->max)));
     case M_SIZE:
-      return (pat->not ^ (h->content->length > pat->min && (pat->max == M_MAXRANGE || h->content->length < pat->max)));
+      return (pat->not ^ (h->content->length >= pat->min && (pat->max == M_MAXRANGE || h->content->length <= pat->max)));
     case M_REFERENCE:
       return (pat->not ^ match_reference (pat->rx, h->env->references));
     case M_ADDRESS:
-      return (pat->not ^ (match_adrlist (pat->rx, flags & M_MATCH_FULL_ADDRESS, h->env->from) ||
-			  match_adrlist (pat->rx, flags & M_MATCH_FULL_ADDRESS, h->env->sender) ||
-			  match_adrlist (pat->rx, flags & M_MATCH_FULL_ADDRESS, h->env->to) ||
-			  match_adrlist (pat->rx, flags & M_MATCH_FULL_ADDRESS, h->env->cc)));
-      break;
+      return (pat->not ^ match_adrlist (pat->rx, flags & M_MATCH_FULL_ADDRESS,
+					pat->alladdr, 4, h->env->from,
+					h->env->sender, h->env->to, h->env->cc));
     case M_RECIPIENT:
-      return (pat->not ^ (match_adrlist (pat->rx, flags & M_MATCH_FULL_ADDRESS, h->env->to) ||
-			  match_adrlist (pat->rx, flags & M_MATCH_FULL_ADDRESS, h->env->cc)));
-      break;
+      return (pat->not ^ match_adrlist (pat->rx, flags & M_MATCH_FULL_ADDRESS,
+					pat->alladdr, 2, h->env->to, h->env->cc));
     case M_LIST:
-      return (pat->not ^ (mutt_is_list_recipient (h->env->to) ||
-			  mutt_is_list_recipient (h->env->cc)));
+      return (pat->not ^ mutt_is_list_recipient (pat->alladdr, h->env->to, h->env->cc));
     case M_PERSONAL_RECIP:
-      return (pat->not ^ (match_user (h->env->to) || match_user (h->env->cc)));
-      break;
+      return (pat->not ^ match_user (pat->alladdr, h->env->to, h->env->cc));
     case M_PERSONAL_FROM:
-      return (pat->not ^ (match_user (h->env->from)));
-      break;
+      return (pat->not ^ match_user (pat->alladdr, h->env->from, NULL));
+    case M_COLLAPSED:
+      return (pat->not ^ (h->collapsed && h->num_hidden > 1));
+#ifdef HAVE_PGP
+   case M_PGP_SIGN:
+     return (pat->not ^ (h->pgp & PGPSIGN));
+   case M_PGP_ENCRYPT:
+     return (pat->not ^ (h->pgp & PGPENCRYPT));
+   case M_PGP_KEY:
+     return (pat->not ^ (h->pgp & PGPKEY));
+#endif
   }
-  mutt_error ("error: unknown op %d (report this error).", pat->op);
+  mutt_error (_("error: unknown op %d (report this error)."), pat->op);
   return (-1);
 }
 
+static void quote_simple(char *tmp, size_t len, const char *p)
+{
+  int i = 0;
+  
+  tmp[i++] = '"';
+  while (*p && i < len - 2)
+  {
+    if (*p == '\\' || *p == '"')
+      tmp[i++] = '\\';
+    tmp[i++] = *p++;
+  }
+  tmp[i++] = '"';
+  tmp[i] = 0;
+}
+  
 /* convert a simple search into a real request */
 void mutt_check_simple (char *s, size_t len, const char *simple)
 {
@@ -815,55 +918,45 @@ void mutt_check_simple (char *s, size_t len, const char *simple)
   if (!strchr (s, '~')) /* yup, so spoof a real request */
   {
     /* convert old tokens into the new format */
-    if (strcasecmp ("all", s) == 0 ||
-	!strcmp ("^", s) || !strcmp (".", s)) /* ~A is more efficient */
+    if (mutt_strcasecmp ("all", s) == 0 ||
+	!mutt_strcmp ("^", s) || !mutt_strcmp (".", s)) /* ~A is more efficient */
       strfcpy (s, "~A", len);
-    else if (strcasecmp ("del", s) == 0)
+    else if (mutt_strcasecmp ("del", s) == 0)
       strfcpy (s, "~D", len);
-    else if (strcasecmp ("flag", s) == 0)
+    else if (mutt_strcasecmp ("flag", s) == 0)
       strfcpy (s, "~F", len);
-    else if (strcasecmp ("new", s) == 0)
+    else if (mutt_strcasecmp ("new", s) == 0)
       strfcpy (s, "~N", len);
-    else if (strcasecmp ("old", s) == 0)
+    else if (mutt_strcasecmp ("old", s) == 0)
       strfcpy (s, "~O", len);
-    else if (strcasecmp ("repl", s) == 0)
+    else if (mutt_strcasecmp ("repl", s) == 0)
       strfcpy (s, "~Q", len);
-    else if (strcasecmp ("read", s) == 0)
+    else if (mutt_strcasecmp ("read", s) == 0)
       strfcpy (s, "~R", len);
-    else if (strcasecmp ("tag", s) == 0)
+    else if (mutt_strcasecmp ("tag", s) == 0)
       strfcpy (s, "~T", len);
-    else if (strcasecmp ("unread", s) == 0)
+    else if (mutt_strcasecmp ("unread", s) == 0)
       strfcpy (s, "~U", len);
     else
     {
-      const char *p = s;
-      int i = 0;
-
-      tmp[i++] = '"';
-      while (*p && i < sizeof (tmp) - 2)
-      {
-	if (*p == '\\' || *p == '"')
-	  tmp[i++] = '\\';
-	tmp[i++] = *p++;
-      }
-      tmp[i++] = '"';
-      tmp[i] = 0;
+      quote_simple (tmp, sizeof(tmp), s);
       mutt_expand_fmt (s, len, simple, tmp);
     }
   }
 }
 
-int mutt_pattern_func (int op, char *prompt, HEADER *hdr)
+int mutt_pattern_func (int op, char *prompt)
 {
   pattern_t *pat;
   char buf[LONG_STRING] = "", *simple, error[STRING];
   BUFFER err;
   int i;
 
-  if (mutt_get_field (prompt, buf, sizeof (buf), 0) != 0 || !buf[0])
+  strfcpy (buf, NONULL (Context->pattern), sizeof (buf));
+  if (mutt_get_field (prompt, buf, sizeof (buf), M_PATTERN | M_CLEAR) != 0 || !buf[0])
     return (-1);
 
-  mutt_message ("Compiling search pattern...");
+  mutt_message _("Compiling search pattern...");
   
   simple = safe_strdup (buf);
   mutt_check_simple (buf, sizeof (buf), NONULL (SimpleSearch));
@@ -877,45 +970,58 @@ int mutt_pattern_func (int op, char *prompt, HEADER *hdr)
     return (-1);
   }
 
-  mutt_message ("Executing command on matching messages...");
+  mutt_message _("Executing command on matching messages...");
+
+#define THIS_BODY Context->hdrs[i]->content
 
   if (op == M_LIMIT)
   {
+    Context->vcount    = 0;
+    Context->vsize     = 0;
+    Context->collapsed = 0;
+
     for (i = 0; i < Context->msgcount; i++)
-      Context->hdrs[i]->virtual = -1;
-    Context->vcount = 0;
-    Context->vsize = 0;
-  }
-
-#define this_body Context->hdrs[i]->content
-
-  for (i = 0; i < Context->msgcount; i++)
-    if (mutt_pattern_exec (pat, M_MATCH_FULL_ADDRESS, Context, Context->hdrs[i]))
     {
-      switch (op)
+      /* new limit pattern implicitly uncollapses all threads */
+      Context->hdrs[i]->virtual = -1;
+      Context->hdrs[i]->limited = 0;
+      Context->hdrs[i]->collapsed = 0;
+      Context->hdrs[i]->num_hidden = 0;
+      if (mutt_pattern_exec (pat, M_MATCH_FULL_ADDRESS, Context, Context->hdrs[i]))
       {
-	case M_DELETE:
-	  mutt_set_flag (Context, Context->hdrs[i], M_DELETE, 1);
-	  break;
-	case M_UNDELETE:
-	  mutt_set_flag (Context, Context->hdrs[i], M_DELETE, 0);
-	  break;
-	case M_TAG:
-	  mutt_set_flag (Context, Context->hdrs[i], M_TAG, 1);
-	  break;
-	case M_UNTAG:
-	  mutt_set_flag (Context, Context->hdrs[i], M_TAG, 0);
-	  break;
-	case M_LIMIT:
-	  Context->hdrs[i]->virtual = Context->vcount;
-	  Context->v2r[Context->vcount] = i;
-	  Context->vcount++;
-	  Context->vsize+=this_body->length + this_body->offset -
-	                  this_body->hdr_offset;
-	  break;
+	Context->hdrs[i]->virtual = Context->vcount;
+	Context->hdrs[i]->limited = 1;
+	Context->v2r[Context->vcount] = i;
+	Context->vcount++;
+	Context->vsize+=THIS_BODY->length + THIS_BODY->offset -
+	  THIS_BODY->hdr_offset;
       }
     }
-#undef this_body
+  }
+  else
+  {
+    for (i = 0; i < Context->vcount; i++)
+    {
+      if (mutt_pattern_exec (pat, M_MATCH_FULL_ADDRESS, Context, Context->hdrs[Context->v2r[i]]))
+      {
+	switch (op)
+	{
+	  case M_DELETE:
+	  case M_UNDELETE:
+	    mutt_set_flag (Context, Context->hdrs[Context->v2r[i]], M_DELETE, 
+			  (op == M_DELETE));
+	    break;
+	  case M_TAG:
+	  case M_UNTAG:
+	    mutt_set_flag (Context, Context->hdrs[Context->v2r[i]], M_TAG, 
+			   (op == M_TAG));
+	    break;
+	}
+      }
+    }
+  }
+
+#undef THIS_BODY
 
   mutt_clear_error ();
 
@@ -926,17 +1032,16 @@ int mutt_pattern_func (int op, char *prompt, HEADER *hdr)
       mutt_pattern_free (&Context->limit_pattern);
     if (!Context->vcount)
     {
-      mutt_error ("No messages matched criteria.");
+      Context->vcount = Context->msgcount;
+      mutt_error _("No messages matched criteria.");
       /* restore full display */
       for (i = 0; i < Context->msgcount; i++)
       {
 	Context->hdrs[i]->virtual = i;
 	Context->v2r[i] = i;
       }
-
-      Context->vcount = Context->msgcount;
     }
-    else if (strncmp (buf, "~A", 2) != 0)
+    else if (mutt_strncmp (buf, "~A", 2) != 0)
     {
       Context->pattern = simple;
       simple = NULL; /* don't clobber it */
@@ -961,8 +1066,9 @@ int mutt_search_command (int cur, int op)
   if (op != OP_SEARCH_NEXT && op != OP_SEARCH_OPPOSITE)
   {
     strfcpy (buf, LastSearch, sizeof (buf));
-    if (mutt_get_field ((op == OP_SEARCH) ? "Search for: " : "Reverse search for: ",
-		      buf, sizeof (buf), M_CLEAR) != 0 || !buf[0])
+    if (mutt_get_field ((op == OP_SEARCH) ? _("Search for: ") :
+		      "Reverse search for: ", buf, sizeof (buf),
+		      M_CLEAR | M_PATTERN) != 0 || !buf[0])
       return (-1);
 
     if (op == OP_SEARCH)
@@ -975,11 +1081,11 @@ int mutt_search_command (int cur, int op)
     strfcpy (temp, buf, sizeof (temp));
     mutt_check_simple (temp, sizeof (temp), NONULL (SimpleSearch));
 
-    if (!SearchPattern || strcmp (temp, LastSearchExpn))
+    if (!SearchPattern || mutt_strcmp (temp, LastSearchExpn))
     {
       set_option (OPTSEARCHINVALID);
       strfcpy (LastSearch, buf, sizeof (LastSearch));
-      mutt_message ("Compiling search pattern...");
+      mutt_message _("Compiling search pattern...");
       mutt_pattern_free (&SearchPattern);
       err.data = error;
       err.dsize = sizeof (error);
@@ -993,7 +1099,7 @@ int mutt_search_command (int cur, int op)
   }
   else if (!SearchPattern)
   {
-    mutt_error ("No search pattern.");
+    mutt_error _("No search pattern.");
     return (-1);
   }
 
@@ -1014,10 +1120,10 @@ int mutt_search_command (int cur, int op)
     {
       i = 0;
       if (option (OPTWRAPSEARCH))
-        mutt_message ("Search wrapped to top.");
+        mutt_message _("Search wrapped to top.");
       else 
       {
-        mutt_message ("Search hit bottom without finding match");
+        mutt_message _("Search hit bottom without finding match");
 	return (-1);
       }
     }
@@ -1025,10 +1131,10 @@ int mutt_search_command (int cur, int op)
     {
       i = Context->vcount - 1;
       if (option (OPTWRAPSEARCH))
-        mutt_message ("Search wrapped to bottom.");
+        mutt_message _("Search wrapped to bottom.");
       else 
       {
-        mutt_message ("Search hit top without finding match");
+        mutt_message _("Search hit top without finding match");
 	return (-1);
       }
     }
@@ -1048,16 +1154,16 @@ int mutt_search_command (int cur, int op)
 	return i;
     }
 
-    if (Signals & S_INTERRUPT)
+    if (SigInt)
     {
-      mutt_error ("Search interrupted.");
-      Signals &= ~S_INTERRUPT;
+      mutt_error _("Search interrupted.");
+      SigInt = 0;
       return (-1);
     }
 
     i += incr;
   }
 
-  mutt_error ("Not found.");
+  mutt_error _("Not found.");
   return (-1);
 }
