@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1999-2000 Thomas Roessler <roessler@guug.de>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -13,7 +14,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */ 
 
 #include "mutt.h"
@@ -22,6 +23,10 @@
 #include "mime.h"
 #include "mailbox.h"
 #include "mapping.h"
+#ifdef USE_IMAP
+#include "mx.h"
+#include "imap.h"
+#endif
 
 #include <ctype.h>
 #include <unistd.h>
@@ -38,28 +43,66 @@ static struct mapping_t PostponeHelp[] = {
 
 
 
-#ifdef _PGPPATH
+#ifdef HAVE_PGP
 #include "pgp.h"
-#endif /* _PGPPATH */
-
+#endif /* HAVE_PGP */
 
 
 static short PostCount = 0;
-static time_t LastModify = 0;
 static CONTEXT *PostContext = NULL;
+static short UpdateNumPostponed = 0;
 
-int mutt_num_postponed (void)
+/* Return the number of postponed messages.
+ * if force is 0, use a cached value if it is costly to get a fresh
+ * count (IMAP) - else check.
+ */
+int mutt_num_postponed (int force)
 {
   struct stat st;
   CONTEXT ctx;
 
-  if (!Postponed || stat (Postponed, &st) == -1)
+  static time_t LastModify = 0;
+  static char *OldPostponed = NULL;
+
+  if (UpdateNumPostponed)
+  {
+    UpdateNumPostponed = 0;
+    force = 1;
+  }
+
+  if (Postponed != OldPostponed)
+  {
+    OldPostponed = Postponed;
+    LastModify = 0;
+    force = 1;
+  }
+
+  if (!Postponed)
+    return 0;
+
+#ifdef USE_IMAP
+  /* LastModify is useless for IMAP */
+  if (mx_is_imap (Postponed))
+  {
+    if (force)
+    {
+      PostCount = imap_mailbox_check (Postponed, 0);
+      PostCount = (PostCount < 0) ? 0 : PostCount;
+      dprint(2, (debugfile,
+        "mutt_num_postponed: %d postponed IMAP messages found.\n", PostCount));
+    }
+    return PostCount;
+  }
+#endif
+
+  if (stat (Postponed, &st) == -1)
   {
      PostCount = 0;
      LastModify = 0;
      return (0);
-  } 
-  else if (S_ISDIR (st.st_mode))
+  }
+
+  if (S_ISDIR (st.st_mode))
   {
     /* if we have a maildir mailbox, we need to stat the "new" dir */
 
@@ -90,6 +133,11 @@ int mutt_num_postponed (void)
   return (PostCount);
 }
 
+void mutt_update_num_postponed (void)
+{
+  UpdateNumPostponed = 1;
+}
+
 static void post_entry (char *s, size_t slen, MUTTMENU *menu, int entry)
 {
   CONTEXT *ctx = (CONTEXT *) menu->data;
@@ -174,10 +222,28 @@ int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur, char *fcc, size
   LIST *next;
   char *p;
   int opt_delete;
+#ifdef USE_IMAP
+  char curpath[LONG_STRING];
+  int need_reopen = 0;
+#endif
 
   if (!Postponed)
     return (-1);
 
+#ifdef USE_IMAP
+  /* if we're in an IMAP folder and the postponed folder is also IMAP, we may
+   * need to take steps to avoid opening an additional connection to the same
+   * server. */
+  if ((ctx && ctx->magic == M_IMAP) && mx_is_imap (Postponed))
+  { 
+    strfcpy (curpath, ctx->path, sizeof (curpath));
+    if (imap_select_mailbox (ctx, Postponed) < 0)
+      return -1;
+    need_reopen = 1;
+    PostContext = ctx;
+  }
+  else
+#endif
   if ((PostContext = mx_open_mailbox (Postponed, M_NOSORT, NULL)) == NULL)
   {
     PostCount = 0;
@@ -188,7 +254,12 @@ int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur, char *fcc, size
   if (! PostContext->msgcount)
   {
     PostCount = 0;
-    mx_close_mailbox (PostContext);
+    mx_close_mailbox (PostContext, NULL);
+#ifdef USE_IMAP
+  if (need_reopen)
+    ctx = mx_open_mailbox (curpath, 0, PostContext);
+  else
+#endif
     safe_free ((void **) &PostContext);
     mutt_error _("No postponed messages.");
     return (-1);
@@ -201,16 +272,26 @@ int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur, char *fcc, size
   }
   else if ((h = select_msg ()) == NULL)
   {
-    mx_close_mailbox (PostContext);
+    mx_close_mailbox (PostContext, NULL);
+#ifdef USE_IMAP
+  if (need_reopen)
+    ctx = mx_open_mailbox (curpath, 0, PostContext);
+  else
+#endif
     safe_free ((void **) &PostContext);
     return (-1);
   }
 
-  if (mutt_prepare_edit_message (PostContext, hdr, h) < 0)
+  if (mutt_prepare_template (NULL, PostContext, hdr, h, 0) < 0)
   {
-      mx_fastclose_mailbox (PostContext);
-      safe_free ((void **) &PostContext);
-      return (-1);
+    mx_fastclose_mailbox (PostContext);
+#ifdef USE_IMAP
+    if (need_reopen)
+      ctx = mx_open_mailbox (curpath, 0, NULL);
+    else
+#endif
+    safe_free ((void **) &PostContext);
+    return (-1);
   }
 
   /* finished with this message, so delete it. */
@@ -222,9 +303,14 @@ int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur, char *fcc, size
   /* avoid the "purge deleted messages" prompt */
   opt_delete = quadoption (OPT_DELETE);
   set_quadoption (OPT_DELETE, M_YES);
-  mx_close_mailbox (PostContext);
+  mx_close_mailbox (PostContext, NULL);
   set_quadoption (OPT_DELETE, opt_delete);
 
+#ifdef USE_IMAP
+  if (need_reopen)
+    ctx = mx_open_mailbox (curpath, 0, PostContext);
+  else
+#endif
   safe_free ((void **) &PostContext);
 
   for (tmp = hdr->env->userhdrs; tmp; )
@@ -272,10 +358,13 @@ int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur, char *fcc, size
 
 
 
-#ifdef _PGPPATH
-    else if (mutt_strncmp ("Pgp:", tmp->data, 4) == 0)
+#ifdef HAVE_PGP
+    else if (mutt_strncmp ("Pgp:", tmp->data, 4) == 0 /* this is generated
+						       * by old mutt versions
+						       */
+	     || mutt_strncmp ("X-Mutt-PGP:", tmp->data, 11) == 0)
     {
-      hdr->pgp = mutt_parse_pgp_hdr (tmp->data+4, 1);
+      hdr->pgp = mutt_parse_pgp_hdr (strchr (tmp->data, ':') + 1, 1);
        
       /* remove the pgp field */
       next = tmp->next;
@@ -287,9 +376,31 @@ int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur, char *fcc, size
       mutt_free_list (&tmp);
       tmp = next;
     }
-#endif /* _PGPPATH */
-
+#endif /* HAVE_PGP */
 
+#ifdef MIXMASTER
+    else if (mutt_strncmp ("X-Mutt-Mix:", tmp->data, 11) == 0)
+    {
+      char *t;
+      mutt_free_list (&hdr->chain);
+      
+      t = strtok (tmp->data + 11, " \t\n");
+      while (t)
+      {
+	hdr->chain = mutt_add_list (hdr->chain, t);
+	t = strtok (NULL, " \t\n");
+      }
+      
+      next = tmp->next;
+      if (last) 
+	last->next = tmp->next;
+      else
+	hdr->env->userhdrs = tmp->next;
+      tmp->next = NULL;
+      mutt_free_list (&tmp);
+      tmp = next;
+    }
+#endif
 
     else
     {
@@ -302,7 +413,7 @@ int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur, char *fcc, size
 
 
 
-#ifdef _PGPPATH
+#ifdef HAVE_PGP
 
 int mutt_parse_pgp_hdr (char *p, int set_signas)
 {
@@ -371,93 +482,157 @@ int mutt_parse_pgp_hdr (char *p, int set_signas)
   }
  
   if (set_signas || *pgp_sign_as)
-  {
-    safe_free((void **) &PgpSignAs);
-    PgpSignAs = safe_strdup(pgp_sign_as);
-  }
+    mutt_str_replace (&PgpSignAs, pgp_sign_as);
 
   /* the micalg field must not be empty */
   if (set_signas && *pgp_sign_micalg)
-  {
-    safe_free((void **) &PgpSignMicalg);
-    PgpSignMicalg = safe_strdup(pgp_sign_micalg);
-  }
+    mutt_str_replace (&PgpSignMicalg, pgp_sign_micalg);
 
   return pgp;
 }
-#endif /* _PGPPATH */
+#endif /* HAVE_PGP */
 
-
-
-int mutt_prepare_edit_message (CONTEXT *ctx, HEADER *newhdr, HEADER *hdr)
+int mutt_prepare_template (FILE *fp, CONTEXT *ctx, HEADER *newhdr, HEADER *hdr,
+			       short weed)
 {
-  MESSAGE *msg = mx_open_message (ctx, hdr->msgno);
+  MESSAGE *msg = NULL;
   char file[_POSIX_PATH_MAX];
+  LIST *p, **q;
+  BODY *b;
+  FILE *bfp;
 
-  if (msg == NULL)
+  if (!fp && (msg = mx_open_message (ctx, hdr->msgno)) == NULL)
     return (-1);
 
-  fseek (msg->fp, hdr->offset, 0);
-  newhdr->env = mutt_read_rfc822_header (msg->fp, newhdr, 1);
+  if (!fp) fp = msg->fp;
 
-  if (hdr->content->type == TYPEMESSAGE || hdr->content->type == TYPEMULTIPART)
+  bfp = fp;
+
+  /* parse the message header and MIME structure */
+
+  fseek (fp, hdr->offset, 0);
+  newhdr->offset = hdr->offset;
+  newhdr->env = mutt_read_rfc822_header (fp, newhdr, 1, weed);
+  newhdr->content->length = hdr->content->length;
+  mutt_parse_part (fp, newhdr->content);
+
+  /* weed user-agent, x-mailer - we don't want them here */
+  p = newhdr->env->userhdrs; 
+  q = &newhdr->env->userhdrs;
+
+  while (p)
   {
-    BODY *b;
-
-    mutt_parse_part (msg->fp, hdr->content);
-
-    /* Now that we know what was in the other message, convert to the new
-     * message.
-     */
-    newhdr->content = hdr->content->parts;
-    b = hdr->content->parts;
-    while (b != NULL)
+    if (!strncasecmp (p->data, "x-mailer:", 9) || !strncasecmp (p->data, "user-agent:", 11))
     {
-      file[0] = '\0';
-      if (b->filename)
-	strfcpy (file, b->filename, sizeof (file));
-      else
-	/* avoid Content-Disposition: header with temporary filename */
-	b->use_disp = 0;
-      mutt_adv_mktemp (file, sizeof(file));
-      if (mutt_save_attachment (msg->fp, b, file, 0, NULL) == -1)
-      {
-	mutt_free_envelope (&newhdr->env);
-	mutt_free_body (&newhdr->content);
-	mx_close_message (&msg);
-	return (-1);
-      }
-      safe_free ((void *) &b->filename);
-      b->filename = safe_strdup (file);
-      b->unlink = 1;
-      mutt_stamp_attachment (b);
-      mutt_free_body (&b->parts);
-      b = b->next;
+      *q = p->next;
+      p->next = NULL;
+      mutt_free_list (&p);
     }
-    hdr->content->parts = NULL;
+    else
+      q = &p->next;
+
+    p = *q;
   }
-  else
+
+  safe_free ((void **) &newhdr->env->message_id);
+  safe_free ((void **) &newhdr->env->mail_followup_to);
+
+#ifdef HAVE_PGP
+  /* decrypt pgp/mime encoded messages */
+  if ((hdr->pgp & PGPENCRYPT) && 
+      mutt_is_multipart_encrypted (newhdr->content))
   {
-    mutt_mktemp (file);
-    if (mutt_save_attachment (msg->fp, hdr->content, file, 0, NULL) == -1)
+    newhdr->pgp |= PGPENCRYPT;
+    if (!pgp_valid_passphrase())
+      goto err;
+
+    mutt_message _("Invoking PGP...");
+    if (pgp_decrypt_mime (fp, &bfp, newhdr->content, &b) == -1)
+    {
+ err:
+      mx_close_message (&msg);
+      mutt_free_envelope (&newhdr->env);
+      mutt_free_body (&newhdr->content);
+      return -1;
+    }
+
+    mutt_free_body (&newhdr->content);
+    newhdr->content = b;
+
+    mutt_clear_error ();
+  }
+
+  /* 
+   * remove a potential multipart/signed layer - useful when
+   * resending messages 
+   */
+  
+  if (mutt_is_multipart_signed (newhdr->content))
+  {
+    newhdr->pgp |= PGPSIGN;
+    
+    /* destroy the signature */
+    mutt_free_body (&newhdr->content->parts->next);
+    newhdr->content = mutt_remove_multipart (newhdr->content);
+  }
+#endif
+
+  /* 
+   * We don't need no primary multipart.
+   * Note: We _do_ preserve messages!
+   * 
+   * XXX - we don't handle multipart/alternative in any 
+   * smart way when sending messages.  However, one may
+   * consider this a feature.
+   * 
+   */
+
+  if (newhdr->content->type == TYPEMULTIPART)
+    newhdr->content = mutt_remove_multipart (newhdr->content);
+
+  /* create temporary files for all attachments */
+  for (b = newhdr->content; b; b = b->next)
+  {
+    
+    /* what follows is roughly a receive-mode variant of
+     * mutt_get_tmp_attachment () from muttlib.c
+     */
+
+    file[0] = '\0';
+    if (b->filename)
+    {
+      strfcpy (file, b->filename, sizeof (file));
+      b->d_filename = safe_strdup (b->filename);
+    }
+    else
+      /* avoid Content-Disposition: header with temporary filename */
+      b->use_disp = 0;
+
+    mutt_adv_mktemp (file, sizeof(file));
+    if (mutt_save_attachment (bfp, b, file, 0, NULL) == -1)
     {
       mutt_free_envelope (&newhdr->env);
-      mx_close_message (&msg);
-      return (-1);
+      mutt_free_body (&newhdr->content);
+      if (bfp != fp) fclose (bfp);
+      if (msg) mx_close_message (&msg);
+      return -1;
     }
-    newhdr->content = mutt_make_file_attach (file);
     
-    FREE (&newhdr->content->subtype);
-    FREE (&newhdr->content->xtype);
-    
-    newhdr->content->type = hdr->content->type;
-    newhdr->content->xtype = safe_strdup (hdr->content->xtype);
-    newhdr->content->subtype = safe_strdup (hdr->content->subtype);
-    
-    newhdr->content->use_disp = 0;	/* no content-disposition */
-    newhdr->content->unlink = 1;	/* delete when we are done */
+    mutt_str_replace (&b->filename, file);
+    b->unlink = 1;
+
+    if (mutt_is_text_type (b->type, b->subtype))
+      b->noconv = 1;
+      
+    mutt_stamp_attachment (b);
+
+    mutt_free_body (&b->parts);
+    if (b->hdr) b->hdr->content = NULL; /* avoid dangling pointer */
   }
 
-  mx_close_message (&msg);
+  /* that's it. */
+  if (bfp != fp) fclose (bfp);
+  if (msg) mx_close_message (&msg);
+
   return 0;
 }
