@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */ 
 
 #include "mutt.h"
@@ -92,21 +92,21 @@ static QUERY *run_query (char *s, int quiet)
   fgets (msg, sizeof (msg) - 1, fp);
   while (fgets(buf, sizeof (buf) - 1, fp))
   {
-    if (first == NULL)
-    {
-      FirstColumn = 0;
-      SecondColumn = 0;
-      first = (QUERY *) safe_calloc (1, sizeof (QUERY));
-      cur = first;
-    }
-    else
-    {
-      cur->next = (QUERY *) safe_calloc (1, sizeof (QUERY));
-      cur = cur->next;
-    }
-    p = strtok(buf, "\t\n");
-    if (p)
+    if ((p = strtok(buf, "\t\n")))
     {
+      if (first == NULL)
+      {
+	FirstColumn = 0;
+	SecondColumn = 0;
+	first = (QUERY *) safe_calloc (1, sizeof (QUERY));
+	cur = first;
+      }
+      else
+      {
+	cur->next = (QUERY *) safe_calloc (1, sizeof (QUERY));
+	cur = cur->next;
+      }
+
       l = mutt_strlen (p);
       if (l > SecondColumn)
 	SecondColumn = l;
@@ -131,13 +131,12 @@ static QUERY *run_query (char *s, int quiet)
   if (mutt_wait_filter (thepid))
   {
     dprint (1, (debugfile, "Error: %s\n", msg));
-    if (!quiet)
-      mutt_error (msg);
+    if (!quiet)  mutt_error ("%s", msg);
   }
   else
   {
     if (!quiet)
-      mutt_message (msg);
+      mutt_message ("%s", msg);
   }
   
   return first;
@@ -180,7 +179,7 @@ static void query_entry (char *s, size_t slen, MUTTMENU *m, int num)
 
 static int query_tag (MUTTMENU *menu, int n)
 {
-  return (((ENTRY *) menu->data)[n].tagged = !((ENTRY *) menu->data)[n].tagged);
+  return ((((ENTRY *) menu->data)[n].tagged = !((ENTRY *) menu->data)[n].tagged) ? 1 : -1);
 }
 
 int mutt_query_complete (char *buf, size_t buflen)
