@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */ 
 
 #include "mutt.h"
@@ -34,10 +34,13 @@
 #include <sys/wait.h>
 #include <dirent.h>
 
-#ifdef _PGPPATH
+#ifdef HAVE_PGP
 #include "pgp.h"
 #endif
 
+#ifdef MIXMASTER
+#include "remailer.h"
+#endif
 
 
 static void append_signature (FILE *f)
@@ -231,8 +234,7 @@ static int edit_envelope (ENVELOPE *en)
     mutt_message _("No subject, aborting.");
     return (-1);
   }
-  safe_free ((void **) &en->subject);
-  en->subject = safe_strdup (buf);
+  mutt_str_replace (&en->subject, buf);
 
   return 0;
 }
@@ -311,33 +313,44 @@ LIST *mutt_copy_list (LIST *p)
   return (l);
 }
 
-static int include_forward (CONTEXT *ctx, HEADER *cur, FILE *out)
+void mutt_forward_intro (FILE *fp, HEADER *cur)
 {
   char buffer[STRING];
+  
+  fputs ("----- Forwarded message from ", fp);
+  buffer[0] = 0;
+  rfc822_write_address (buffer, sizeof (buffer), cur->env->from);
+  fputs (buffer, fp);
+  fputs (" -----\n\n", fp);
+}
+
+void mutt_forward_trailer (FILE *fp)
+{
+  fputs ("\n----- End forwarded message -----\n", fp);
+}
+
+
+static int include_forward (CONTEXT *ctx, HEADER *cur, FILE *out)
+{
   int chflags = CH_DECODE, cmflags = 0;
   
 
-#ifdef _PGPPATH
+#ifdef HAVE_PGP
   if ((cur->pgp & PGPENCRYPT) && option (OPTFORWDECODE))
   {
     /* make sure we have the user's passphrase before proceeding... */
     pgp_valid_passphrase ();
   }
-#endif /* _PGPPATH */
+#endif /* HAVE_PGP */
 
+  mutt_forward_intro (out, cur);
 
-
-  fputs ("----- Forwarded message from ", out);
-  buffer[0] = 0;
-  rfc822_write_address (buffer, sizeof (buffer), cur->env->from);
-  fputs (buffer, out);
-  fputs (" -----\n\n", out);
   if (option (OPTFORWDECODE))
   {
-    cmflags |= M_CM_DECODE;
-    if (option (OPTFORWWEEDHEADER))
+    cmflags |= M_CM_DECODE | M_CM_CHARCONV;
+    if (option (OPTWEED))
     {
-      chflags |= CH_WEED;
+      chflags |= CH_WEED | CH_REORDER;
       cmflags |= M_CM_WEED;
     }
   }
@@ -345,18 +358,38 @@ static int include_forward (CONTEXT *ctx, HEADER *cur, FILE *out)
     cmflags |= M_CM_PREFIX;
   mutt_parse_mime_message (ctx, cur);
   mutt_copy_message (out, ctx, cur, cmflags, chflags);
-  fputs ("\n----- End forwarded message -----\n", out);
+  mutt_forward_trailer (out);
   return 0;
 }
 
+void mutt_make_attribution (CONTEXT *ctx, HEADER *cur, FILE *out)
+{
+  char buffer[STRING];
+  if (Attribution)
+  {
+    mutt_make_string (buffer, sizeof (buffer), Attribution, ctx, cur);
+    fputs (buffer, out);
+    fputc ('\n', out);
+  }
+}
+
+void mutt_make_post_indent (CONTEXT *ctx, HEADER *cur, FILE *out)
+{
+  char buffer[STRING];
+  if (PostIndentString)
+  {
+    mutt_make_string (buffer, sizeof (buffer), PostIndentString, ctx, cur);
+    fputs (buffer, out);
+    fputc ('\n', out);
+  }
+}
+
 static int include_reply (CONTEXT *ctx, HEADER *cur, FILE *out)
 {
-  char buffer[STRING];
-  int flags = M_CM_PREFIX | M_CM_DECODE;
+  int cmflags = M_CM_PREFIX | M_CM_DECODE | M_CM_CHARCONV;
+  int chflags = CH_DECODE;
 
-
-
-#ifdef _PGPPATH
+#ifdef HAVE_PGP
   if (cur->pgp)
   {
     if (cur->pgp & PGPENCRYPT)
@@ -365,40 +398,50 @@ static int include_reply (CONTEXT *ctx, HEADER *cur, FILE *out)
       pgp_valid_passphrase ();
     }
   }
-#endif /* _PGPPATH */
+#endif /* HAVE_PGP */
 
-
-
-  if (Attribution)
-  {
-    mutt_make_string (buffer, sizeof (buffer), Attribution, ctx, cur);
-    fputs (buffer, out);
-    fputc ('\n', out);
-  }
+  mutt_make_attribution (ctx, cur, out);
+  
   if (!option (OPTHEADER))
-    flags |= M_CM_NOHEADER;
-  mutt_parse_mime_message (ctx, cur);
-  mutt_copy_message (out, ctx, cur, flags, CH_DECODE);
-  if (PostIndentString)
+    cmflags |= M_CM_NOHEADER;
+  if (option (OPTWEED))
   {
-    mutt_make_string (buffer, sizeof (buffer), PostIndentString, ctx, cur);
-    fputs (buffer, out);
-    fputc ('\n', out);
+    chflags |= CH_WEED;
+    cmflags |= M_CM_WEED;
   }
+
+  mutt_parse_mime_message (ctx, cur);
+  mutt_copy_message (out, ctx, cur, cmflags, chflags);
+  
+  mutt_make_post_indent (ctx, cur, out);
+  
   return 0;
 }
 
-static int default_to (ADDRESS **to, ENVELOPE *env, int group)
+static int default_to (ADDRESS **to, ENVELOPE *env, int flags)
 {
   char prompt[STRING];
   int i = 0;
 
-  if (group && env->mail_followup_to)
+  if (flags && env->mail_followup_to)
   {
-    rfc822_append (to, env->mail_followup_to);
-    return 0;
+    snprintf (prompt, sizeof (prompt), _("Follow-up to %s%s?"),
+	      env->mail_followup_to->mailbox,
+	      env->mail_followup_to->next ? "..." : "");
+
+    if (query_quadoption (OPT_MFUPTO, prompt) == M_YES)
+    {
+      rfc822_append (to, env->mail_followup_to);
+      return 0;
+    }
   }
 
+  /* Exit now if we're setting up the default Cc list for list-reply
+   * (only set if Mail-Followup-To is present and honoured).
+   */
+  if (flags & SENDLISTREPLY)
+    return 0;
+
   if (!option(OPTREPLYSELF) && mutt_addr_is_user (env->from))
   {
     /* mail is from the user, assume replying to recipients */
@@ -450,7 +493,7 @@ static int default_to (ADDRESS **to, ENVELOPE *env, int group)
   return (0);
 }
 
-static int fetch_recips (ENVELOPE *out, ENVELOPE *in, int flags)
+int mutt_fetch_recips (ENVELOPE *out, ENVELOPE *in, int flags)
 {
   ADDRESS *tmp;
   if (flags & SENDLISTREPLY)
@@ -458,6 +501,10 @@ static int fetch_recips (ENVELOPE *out, ENVELOPE *in, int flags)
     tmp = find_mailing_lists (in->to, in->cc);
     rfc822_append (&out->to, tmp);
     rfc822_free_address (&tmp);
+
+    if (in->mail_followup_to &&
+        default_to (&out->cc, in, flags & SENDLISTREPLY) == -1)
+      return (-1); /* abort */
   }
   else
   {
@@ -474,7 +521,7 @@ static int fetch_recips (ENVELOPE *out, ENVELOPE *in, int flags)
   return 0;
 }
 
-static LIST *make_references(ENVELOPE *e)
+LIST *mutt_make_references(ENVELOPE *e)
 {
   LIST *t, *l;
   
@@ -491,12 +538,69 @@ static LIST *make_references(ENVELOPE *e)
   return l;
 }
 
+void mutt_fix_reply_recipients (ENVELOPE *env)
+{
+  if (! option (OPTMETOO))
+  {
+    /* the order is important here.  do the CC: first so that if the
+     * the user is the only recipient, it ends up on the TO: field
+     */
+    env->cc = remove_user (env->cc, (env->to == NULL));
+    env->to = remove_user (env->to, (env->cc == NULL));
+  }
+  
+  /* the CC field can get cluttered, especially with lists */
+  env->to = mutt_remove_duplicates (env->to);
+  env->cc = mutt_remove_duplicates (env->cc);
+  env->cc = mutt_remove_xrefs (env->to, env->cc);
+}
+
+void mutt_make_forward_subject (ENVELOPE *env, CONTEXT *ctx, HEADER *cur)
+{
+  char buffer[STRING];
+
+  /* set the default subject for the message. */
+  mutt_make_string (buffer, sizeof (buffer), NONULL(ForwFmt), ctx, cur);
+  env->subject = safe_strdup (buffer);
+}
+
+void mutt_make_misc_reply_headers (ENVELOPE *env, CONTEXT *ctx,
+				    HEADER *cur, ENVELOPE *curenv)
+{
+  LIST *tmp;
+  char buffer[STRING];
+
+  if (curenv->real_subj)
+  {
+    env->subject = safe_malloc (mutt_strlen (curenv->real_subj) + 5);
+    sprintf (env->subject, "Re: %s", curenv->real_subj);
+  }
+  else
+    env->subject = safe_strdup ("Re: your mail");
+  
+  /* add the In-Reply-To field */
+  if (InReplyTo)
+  {
+    strfcpy (buffer, "In-Reply-To: ", sizeof (buffer));
+    mutt_make_string (buffer + 13, sizeof (buffer) - 13, InReplyTo, ctx, cur);
+    tmp = env->userhdrs;
+    while (tmp && tmp->next)
+      tmp = tmp->next;
+    if (tmp)
+    {
+      tmp->next = mutt_new_list ();
+      tmp = tmp->next;
+    }
+    else
+      tmp = env->userhdrs = mutt_new_list ();
+    tmp->data = safe_strdup (buffer);
+  }
+}
+
 static int
 envelope_defaults (ENVELOPE *env, CONTEXT *ctx, HEADER *cur, int flags)
 {
   ENVELOPE *curenv = NULL;
-  LIST *tmp;
-  char buffer[STRING];
   int i = 0, tag = 0;
 
   if (!cur)
@@ -531,11 +635,11 @@ envelope_defaults (ENVELOPE *env, CONTEXT *ctx, HEADER *cur, int flags)
       for (i = 0; i < ctx->vcount; i++)
       {
 	h = ctx->hdrs[ctx->v2r[i]];
-	if (h->tagged && fetch_recips (env, h->env, flags) == -1)
+	if (h->tagged && mutt_fetch_recips (env, h->env, flags) == -1)
 	  return -1;
       }
     }
-    else if (fetch_recips (env, curenv, flags) == -1)
+    else if (mutt_fetch_recips (env, curenv, flags) == -1)
       return -1;
 
     if ((flags & SENDLISTREPLY) && !env->to)
@@ -544,46 +648,9 @@ envelope_defaults (ENVELOPE *env, CONTEXT *ctx, HEADER *cur, int flags)
       return (-1);
     }
 
-    if (! option (OPTMETOO))
-    {
-      /* the order is important here.  do the CC: first so that if the
-       * the user is the only recipient, it ends up on the TO: field
-       */
-      env->cc = remove_user (env->cc, (env->to == NULL));
-      env->to = remove_user (env->to, (env->cc == NULL));
-    }
-
-    /* the CC field can get cluttered, especially with lists */
-    env->to = mutt_remove_duplicates (env->to);
-    env->cc = mutt_remove_duplicates (env->cc);
-    env->cc = mutt_remove_xrefs (env->to, env->cc);
-
-    if (curenv->real_subj)
-    {
-      env->subject = safe_malloc (mutt_strlen (curenv->real_subj) + 5);
-      sprintf (env->subject, "Re: %s", curenv->real_subj);
-    }
-    else
-      env->subject = safe_strdup ("Re: your mail");
-
-    /* add the In-Reply-To field */
-    if (InReplyTo)
-    {
-      strfcpy (buffer, "In-Reply-To: ", sizeof (buffer));
-      mutt_make_string (buffer + 13, sizeof (buffer) - 13, InReplyTo, ctx, cur);
-      tmp = env->userhdrs;
-      while (tmp && tmp->next)
-	tmp = tmp->next;
-      if (tmp)
-      {
-	tmp->next = mutt_new_list ();
-	tmp = tmp->next;
-      }
-      else
-	tmp = env->userhdrs = mutt_new_list ();
-      tmp->data = safe_strdup (buffer);
-    }
-
+    mutt_fix_reply_recipients (env);
+    mutt_make_misc_reply_headers (env, ctx, cur, curenv);
+    
     if(tag)
     {
       HEADER *h;
@@ -591,25 +658,21 @@ envelope_defaults (ENVELOPE *env, CONTEXT *ctx, HEADER *cur, int flags)
 
       env->references = NULL;
       p = &env->references;
-      
+
       for(i = 0; i < ctx->vcount; i++)
       {
 	while(*p) p = &(*p)->next;
 	h = ctx->hdrs[ctx->v2r[i]];
 	if(h->tagged)
-	  *p = make_references(h->env);
+	  *p = mutt_make_references(h->env);
       }
     }
     else
-      env->references = make_references(curenv);
+      env->references = mutt_make_references(curenv);
 
   }
   else if (flags & SENDFORWARD)
-  {
-    /* set the default subject for the message. */
-    mutt_make_string (buffer, sizeof (buffer), NONULL(ForwFmt), ctx, cur);
-    env->subject = safe_strdup (buffer);
-  }
+    mutt_make_forward_subject (env, ctx, cur);
 
   return (0);
 }
@@ -632,6 +695,7 @@ generate_body (FILE *tempfp,	/* stream for outgoing message */
 
     if (i == M_YES)
     {
+      mutt_message _("Including quoted message...");
       if (!cur)
       {
 	for (i = 0; i < ctx->vcount; i++)
@@ -650,14 +714,17 @@ generate_body (FILE *tempfp,	/* stream for outgoing message */
       }
       else
 	include_reply (ctx, cur, tempfp);
+
     }
   }
   else if (flags & SENDFORWARD)
   {
-    if (query_quadoption (OPT_MIMEFWD, _("Forward MIME encapsulated?")))
+    if ((i = query_quadoption (OPT_MIMEFWD, _("Forward MIME encapsulated?"))) == M_YES)
     {
       BODY *last = msg->content;
 
+      mutt_message _("Preparing forwarded message...");
+      
       while (last && last->next)
 	last = last->next;
 
@@ -687,7 +754,7 @@ generate_body (FILE *tempfp,	/* stream for outgoing message */
 	}
       }
     }
-    else
+    else if (i != -1)
     {
       if (cur)
 	include_forward (ctx, cur, tempfp);
@@ -696,11 +763,13 @@ generate_body (FILE *tempfp,	/* stream for outgoing message */
 	  if (ctx->hdrs[ctx->v2r[i]]->tagged)
 	    include_forward (ctx, ctx->hdrs[ctx->v2r[i]], tempfp);
     }
+    else if (i == -1)
+      return -1;
   }
 
 
 
-#ifdef _PGPPATH
+#ifdef HAVE_PGP
   else if (flags & SENDKEY) 
   {
     BODY *tmp;
@@ -712,7 +781,7 @@ generate_body (FILE *tempfp,	/* stream for outgoing message */
   }
 #endif
 
-
+  mutt_clear_error ();
 
   return (0);
 }
@@ -720,22 +789,56 @@ generate_body (FILE *tempfp,	/* stream for outgoing message */
 void mutt_set_followup_to (ENVELOPE *e)
 {
   ADDRESS *t = NULL;
+  ADDRESS *from;
+
+  /* 
+   * Only generate the Mail-Followup-To if the user has requested it, and
+   * it hasn't already been set
+   */
 
-  /* only generate the Mail-Followup-To if the user has requested it, and
-     it hasn't already been set */
   if (option (OPTFOLLOWUPTO) && !e->mail_followup_to)
   {
-    if (mutt_is_list_recipient (0, e->to, e->cc))
+    if (mutt_is_list_cc (0, e->to, e->cc))
     {
-      /* i am a list recipient, so set the Mail-Followup-To: field so that
-       * i don't end up getting multiple copies of responses to my mail
+      /* 
+       * this message goes to known mailing lists, so create a proper
+       * mail-followup-to header
        */
+
       t = rfc822_append (&e->mail_followup_to, e->to);
       rfc822_append (&t, e->cc);
-      /* the following is needed if $metoo is set, because the ->to and ->cc
-	 may contain the user's private address(es) */
-      e->mail_followup_to = remove_user (e->mail_followup_to, 0);
     }
+
+    /* remove ourselves from the mail-followup-to header */
+    e->mail_followup_to = remove_user (e->mail_followup_to, 0);
+
+    /*
+     * If we are not subscribed to any of the lists in question,
+     * re-add ourselves to the mail-followup-to header.  The 
+     * mail-followup-to header generated is a no-op with group-reply,
+     * but makes sure list-reply has the desired effect.
+     */
+
+    if (e->mail_followup_to && !mutt_is_list_recipient (0, e->to, e->cc))
+    {
+      if (e->from)
+	from = rfc822_cpy_adr (e->from);
+      else
+	from = mutt_default_from ();
+      
+      if (from)
+      {
+	/* Normally, this loop will not even be entered. */
+	for (t = from; t && t->next; t = t->next)
+	  ;
+	
+	t->next = e->mail_followup_to; 	/* t cannot be NULL at this point. */
+	e->mail_followup_to = from;
+      }
+    }
+    
+    e->mail_followup_to = mutt_remove_duplicates (e->mail_followup_to);
+    
   }
 }
 
@@ -765,26 +868,38 @@ static ADDRESS *set_reverse_name (ENVELOPE *env)
   if (tmp)
   {
     tmp = rfc822_cpy_adr_real (tmp);
+    if (!option (OPTREVREAL))
+      FREE (&tmp->personal);
     if (!tmp->personal)
       tmp->personal = safe_strdup (Realname);
   }
   return (tmp);
 }
 
-static ADDRESS *mutt_default_from (void)
+ADDRESS *mutt_default_from (void)
 {
-  ADDRESS *adr = rfc822_new_address ();
+  ADDRESS *adr;
   const char *fqdn = mutt_fqdn(1);
-  
-  /* don't set realname here, it will be set later */
 
-  if (option (OPTUSEDOMAIN))
+  /* 
+   * Note: We let $from override $realname here.  Is this the right
+   * thing to do? 
+   */
+
+  if (From)
+    adr = rfc822_cpy_adr_real (From);
+  else if (option (OPTUSEDOMAIN))
   {
+    adr = rfc822_new_address ();
     adr->mailbox = safe_malloc (mutt_strlen (Username) + mutt_strlen (fqdn) + 2);
     sprintf (adr->mailbox, "%s@%s", NONULL(Username), NONULL(fqdn));
   }
   else
+  {
+    adr = rfc822_new_address ();
     adr->mailbox = safe_strdup (NONULL(Username));
+  }
+  
   return (adr);
 }
 
@@ -793,13 +908,19 @@ static int send_message (HEADER *msg)
   char tempfile[_POSIX_PATH_MAX];
   FILE *tempfp;
   int i;
-
+  
   /* Write out the message in MIME form. */
   mutt_mktemp (tempfile);
   if ((tempfp = safe_fopen (tempfile, "w")) == NULL)
     return (-1);
 
-  mutt_write_rfc822_header (tempfp, msg->env, msg->content, 0);
+#ifdef MIXMASTER
+  mutt_write_rfc822_header (tempfp, msg->env, msg->content, 0, msg->chain ? 1 : 0);
+#endif
+#ifndef MIXMASTER
+  mutt_write_rfc822_header (tempfp, msg->env, msg->content, 0, 0);
+#endif
+  
   fputc ('\n', tempfp); /* tie off the header. */
 
   if ((mutt_write_mime_body (msg->content, tempfp) == -1))
@@ -816,13 +937,18 @@ static int send_message (HEADER *msg)
     return (-1);
   }
 
-  i = mutt_invoke_sendmail (msg->env->to, msg->env->cc, msg->env->bcc,
-		       tempfile, (msg->content->encoding == ENC8BIT));
-  return (i ? -1 : 0);
+#ifdef MIXMASTER
+  if (msg->chain)
+    return mix_send_message (msg->chain, tempfile);
+#endif
+
+  i = mutt_invoke_sendmail (msg->env->from, msg->env->to, msg->env->cc, 
+			    msg->env->bcc, tempfile, (msg->content->encoding == ENC8BIT));
+  return (i);
 }
 
 /* rfc2047 encode the content-descriptions */
-static void encode_descriptions (BODY *b)
+static void encode_descriptions (BODY *b, short recurse)
 {
   BODY *t;
   char tmp[LONG_STRING];
@@ -832,15 +958,43 @@ static void encode_descriptions (BODY *b)
     if (t->description)
     {
       rfc2047_encode_string (tmp, sizeof (tmp), (unsigned char *) t->description);
-      safe_free ((void **) &t->description);
-      t->description = safe_strdup (tmp);
+      mutt_str_replace (&t->description, tmp);
     }
-    if (t->parts)
-      encode_descriptions (t->parts);
+    if (recurse && t->parts)
+      encode_descriptions (t->parts, recurse);
   }
 }
 
-void
+/* rfc2047 decode them in case of an error */
+static void decode_descriptions (BODY *b)
+{
+  BODY *t;
+  char tmp[LONG_STRING];
+  
+  for (t = b; t; t = t->next)
+  {
+    if (t->description)
+    {
+      /* this should really have the same interface as rfc2047_encode_string. */
+      rfc2047_decode (tmp, t->description, sizeof (tmp));
+      mutt_str_replace (&t->description, tmp);
+    }
+    if (t->parts)
+      decode_descriptions (t->parts);
+  }
+}
+
+int mutt_resend_message (FILE *fp, CONTEXT *ctx, HEADER *cur)
+{
+  HEADER *msg = mutt_new_header ();
+  
+  if (mutt_prepare_template (fp, ctx, msg, cur, 1) < 0)
+    return -1;
+  
+  return ci_send_message (SENDRESEND, msg, NULL, ctx, cur);
+}
+
+int
 ci_send_message (int flags,		/* send mode */
 		 HEADER *msg,		/* template to use for new message */
 		 char *tempfile,	/* file specified by -i or -H */
@@ -852,57 +1006,55 @@ ci_send_message (int flags,		/* send mode */
   FILE *tempfp = NULL;
   BODY *pbody;
   int i, killfrom = 0;
-#ifdef _PGPPATH
+
+#ifdef HAVE_PGP
   BODY *save_content = NULL;
+  BODY *clear_content = NULL;
   char *pgpkeylist = NULL;
   /* save current value of "pgp_sign_as" */
   char *signas = NULL;
   char *signmic = NULL;
 #endif
 
-  if (!flags && !msg && quadoption (OPT_RECALL) != M_NO && mutt_num_postponed ())
+  int rv = -1;
+  
+  if (!flags && !msg && quadoption (OPT_RECALL) != M_NO &&
+      mutt_num_postponed (1))
   {
     /* If the user is composing a new message, check to see if there
      * are any postponed messages first.
      */
     if ((i = query_quadoption (OPT_RECALL, _("Recall postponed message?"))) == -1)
-      return;
+      return rv;
 
     if(i == M_YES)
       flags |= SENDPOSTPONED;
   }
   
   
-#ifdef _PGPPATH
+#ifdef HAVE_PGP
   if (flags & SENDPOSTPONED)
   {
     signas = safe_strdup(PgpSignAs);
     signmic = safe_strdup(PgpSignMicalg);
   }
-#endif /* _PGPPATH */
+#endif /* HAVE_PGP */
 
   if (msg)
   {
-    msg->env->to = mutt_expand_aliases (msg->env->to);
-    msg->env->cc = mutt_expand_aliases (msg->env->cc);
-    msg->env->bcc = mutt_expand_aliases (msg->env->bcc);
+    mutt_expand_aliases_env (msg->env);
   }
   else
   {
     msg = mutt_new_header ();
 
-    if (flags == SENDEDITMSG)
-    {
-      if (mutt_prepare_edit_message(ctx, msg, cur) < 0)
-	goto cleanup;
-    }
-    else if (flags == SENDPOSTPONED)
+    if (flags == SENDPOSTPONED)
     {
       if ((flags = mutt_get_postponed (ctx, msg, &cur, fcc, sizeof (fcc))) < 0)
 	goto cleanup;
     }
 
-    if (flags & (SENDPOSTPONED | SENDEDITMSG))
+    if (flags & (SENDPOSTPONED|SENDRESEND))
     {
       if ((tempfp = safe_fopen (msg->content->filename, "a+")) == NULL)
       {
@@ -915,7 +1067,7 @@ ci_send_message (int flags,		/* send mode */
       msg->env = mutt_new_envelope ();
   }
 
-  if (! (flags & (SENDKEY | SENDPOSTPONED | SENDEDITMSG)))
+  if (! (flags & (SENDKEY | SENDPOSTPONED | SENDRESEND)))
   {
     pbody = mutt_new_body ();
     pbody->next = msg->content; /* don't kill command-line attachments */
@@ -925,6 +1077,7 @@ ci_send_message (int flags,		/* send mode */
     msg->content->subtype = safe_strdup ("plain");
     msg->content->unlink = 1;
     msg->content->use_disp = 0;
+    msg->content->disposition = DISPINLINE;
     
     if (!tempfile)
     {
@@ -947,7 +1100,7 @@ ci_send_message (int flags,		/* send mode */
   }
 
   /* this is handled here so that the user can match ~f in send-hook */
-  if (cur && option (OPTREVNAME) && !(flags & (SENDPOSTPONED | SENDEDITMSG)))
+  if (cur && option (OPTREVNAME) && !(flags & (SENDPOSTPONED|SENDRESEND)))
   {
     /* we shouldn't have to worry about freeing `msg->env->from' before
      * setting it here since this code will only execute when doing some
@@ -956,7 +1109,7 @@ ci_send_message (int flags,		/* send mode */
     msg->env->from = set_reverse_name (cur->env);
   }
 
-  if (!msg->env->from && option (OPTUSEFROM) && !(flags & (SENDEDITMSG|SENDPOSTPONED)))
+  if (!msg->env->from && option (OPTUSEFROM) && !(flags & (SENDPOSTPONED|SENDRESEND)))
     msg->env->from = mutt_default_from ();
 
   if (flags & SENDBATCH) 
@@ -968,9 +1121,9 @@ ci_send_message (int flags,		/* send mode */
       process_user_header (msg->env);
     }
   }
-  else if (! (flags & (SENDPOSTPONED | SENDEDITMSG)))
+  else if (! (flags & (SENDPOSTPONED|SENDRESEND)))
   {
-    if ((flags & (SENDREPLY | SENDFORWARD)) &&
+    if ((flags & (SENDREPLY | SENDFORWARD)) && ctx &&
 	envelope_defaults (msg->env, ctx, cur, flags) == -1)
       goto cleanup;
 
@@ -1011,7 +1164,7 @@ ci_send_message (int flags,		/* send mode */
 
 
 
-#ifdef _PGPPATH
+#ifdef HAVE_PGP
     if (! (flags & SENDMAILX))
     {
       if (option (OPTPGPAUTOSIGN))
@@ -1022,13 +1175,16 @@ ci_send_message (int flags,		/* send mode */
 	msg->pgp |= PGPENCRYPT;
       if (option (OPTPGPREPLYSIGN) && cur && cur->pgp & PGPSIGN)
 	msg->pgp |= PGPSIGN;
+      if (option (OPTPGPREPLYSIGNENCRYPTED) && cur && cur->pgp & PGPENCRYPT)
+	msg->pgp |= PGPSIGN;
     }
-#endif /* _PGPPATH */
+#endif /* HAVE_PGP */
 
 
 
-    /* include replies/forwarded messages */
-    if (generate_body (tempfp, msg, flags, ctx, cur) == -1)
+    /* include replies/forwarded messages, unless we are given a template */
+    if (!tempfile && (ctx || !(flags & (SENDREPLY|SENDFORWARD)))
+	&& generate_body (tempfp, msg, flags, ctx, cur) == -1)
       goto cleanup;
 
     if (! (flags & (SENDMAILX | SENDKEY)) && Editor && mutt_strcmp (Editor, "builtin") != 0)
@@ -1036,14 +1192,14 @@ ci_send_message (int flags,		/* send mode */
   }
   /* wait until now to set the real name portion of our return address so
      that $realname can be set in a send-hook */
-  if (msg->env->from && !msg->env->from->personal && !(flags & (SENDEDITMSG | SENDPOSTPONED)))
+  if (msg->env->from && !msg->env->from->personal && !(flags & (SENDRESEND|SENDPOSTPONED)))
     msg->env->from->personal = safe_strdup (Realname);
 
 
 
-#ifdef _PGPPATH
+#ifdef HAVE_PGP
   
-  if (! (flags & SENDKEY))
+  if (! (flags & SENDKEY) && tempfp)
   {
 
 #endif
@@ -1055,7 +1211,7 @@ ci_send_message (int flags,		/* send mode */
     
 
 
-#ifdef _PGPPATH
+#ifdef HAVE_PGP
     
   }
 
@@ -1091,7 +1247,7 @@ ci_send_message (int flags,		/* send mode */
 	mutt_edit_file (Editor, msg->content->filename);
     }
 
-    if (! (flags & (SENDPOSTPONED | SENDEDITMSG | SENDFORWARD | SENDKEY)))
+    if (! (flags & (SENDPOSTPONED | SENDFORWARD | SENDKEY | SENDRESEND)))
     {
       if (stat (msg->content->filename, &st) == 0)
       {
@@ -1111,7 +1267,7 @@ ci_send_message (int flags,		/* send mode */
   /* specify a default fcc.  if we are in batchmode, only save a copy of
    * the message if the value of $copy is yes or ask-yes */
 
-  if (!fcc[0] && (!(flags & SENDBATCH) || (quadoption (OPT_COPY) & 0x1)))
+  if (!fcc[0] && !(flags & (SENDRESEND|SENDPOSTPONED)) && (!(flags & SENDBATCH) || (quadoption (OPT_COPY) & 0x1)))
   {
     /* set the default FCC */
     if (!msg->env->from)
@@ -1128,12 +1284,14 @@ ci_send_message (int flags,		/* send mode */
     }
   }
 
+  
   mutt_update_encoding (msg->content);
 
   if (! (flags & (SENDMAILX | SENDBATCH)))
   {
 main_loop:
 
+    mutt_pretty_mailbox (fcc);
     i = mutt_compose_menu (msg, fcc, sizeof (fcc), cur);
     if (i == -1)
     {
@@ -1146,11 +1304,24 @@ main_loop:
       /* postpone the message until later. */
       if (msg->content->next)
 	msg->content = mutt_make_multipart (msg->content);
+
+      /*
+       * make sure the message is written to the right part of a maildir 
+       * postponed folder.
+       */
+      msg->read = 0; msg->old = 0;
+
+      encode_descriptions (msg->content, 1);
+      mutt_prepare_envelope (msg->env, 0);
+
       if (!Postponed || mutt_write_fcc (NONULL (Postponed), msg, (cur && (flags & SENDREPLY)) ? cur->env->message_id : NULL, 1, fcc) < 0)
       {
 	msg->content = mutt_remove_multipart (msg->content);
+	decode_descriptions (msg->content);
+	mutt_unprepare_envelope (msg->env);
 	goto main_loop;
       }
+      mutt_update_num_postponed ();
       mutt_message _("Message postponed.");
       goto cleanup;
     }
@@ -1182,77 +1353,100 @@ main_loop:
   if (msg->content->next)
     msg->content = mutt_make_multipart (msg->content);
 
-  /* Ok, we need to do it this way instead of handling all fcc stuff in
+  /* 
+   * Ok, we need to do it this way instead of handling all fcc stuff in
    * one place in order to avoid going to main_loop with encoded "env"
    * in case of error.  Ugh.
    */
-#ifdef _PGPPATH
+
+  encode_descriptions (msg->content, 1);
+  
+#ifdef HAVE_PGP
   if (msg->pgp)
   {
-    if (pgp_get_keys (msg, &pgpkeylist) == -1)
-      goto main_loop;
-
     /* save the decrypted attachments */
-    save_content = msg->content;
+    clear_content = msg->content;
 
-    if (pgp_protect (msg, pgpkeylist) == -1)
+    if ((pgp_get_keys (msg, &pgpkeylist) == -1) ||
+	(pgp_protect (msg, pgpkeylist) == -1))
     {
-      if (msg->content->parts)
-      {
-	/* remove the toplevel multipart structure */
-	pbody = msg->content;
-	msg->content = msg->content->parts;
-	pbody->parts = NULL;
-	mutt_free_body (&pbody);
-      }
+      msg->content = mutt_remove_multipart (msg->content);
+      
       if (pgpkeylist)
 	FREE (&pgpkeylist);
+      
+      decode_descriptions (msg->content);
       goto main_loop;
     }
+    encode_descriptions (msg->content, 0);
   }
-#endif /* _PGPPATH */
 
-  if (flags & SENDEDITMSG)
-  {
-   int really_send = mutt_yesorno (_("Message edited. Really send?"), 1);
-   if (really_send != M_YES)
-     goto main_loop;
-  }
+  /* 
+   * at this point, msg->content is one of the following three things:
+   * - multipart/signed.  In this case, clear_content is a child.
+   * - multipart/encrypted.  In this case, clear_content exists
+   *   independently
+   * - application/pgp.  In this case, clear_content exists independently.
+   * - something else.  In this case, it's the same as clear_content.
+   */
+
+#endif /* HAVE_PGP */
 
   if (!option (OPTNOCURSES) && !(flags & SENDMAILX))
     mutt_message _("Sending message...");
 
-  mutt_prepare_envelope (msg->env);
-  encode_descriptions (msg->content);
+  mutt_prepare_envelope (msg->env, 1);
 
   /* save a copy of the message, if necessary. */
+
   mutt_expand_path (fcc, sizeof (fcc));
+
+  
+  /* Don't save a copy when we are in batch-mode, and the FCC
+   * folder is on an IMAP server: This would involve possibly lots
+   * of user interaction, which is not available in batch mode. 
+   * 
+   * Note: A patch to fix the problems with the use of IMAP servers
+   * from non-curses mode is available from Brendan Cully.  However, 
+   * I'd like to think a bit more about this before including it.
+   */
+
+#ifdef USE_IMAP
+  if ((flags & SENDBATCH) && fcc[0] && mx_is_imap (fcc))
+    fcc[0] = '\0';
+#endif
+
   if (*fcc && mutt_strcmp ("/dev/null", fcc) != 0)
   {
     BODY *tmpbody = msg->content;
-#ifdef _PGPPATH
+#ifdef HAVE_PGP
     BODY *save_sig = NULL;
     BODY *save_parts = NULL;
-#endif /* _PGPPATH */
+#endif /* HAVE_PGP */
+
+#ifdef HAVE_PGP
+    if (msg->pgp && option (OPTFCCCLEAR))
+      msg->content = clear_content;
+#endif
 
     /* check to see if the user wants copies of all attachments */
     if (!option (OPTFCCATTACH) && msg->content->type == TYPEMULTIPART)
     {
-#ifdef _PGPPATH
+#ifdef HAVE_PGP
       if (mutt_strcmp (msg->content->subtype, "encrypted") == 0 ||
 	  mutt_strcmp (msg->content->subtype, "signed") == 0)
       {
-	if (save_content->type == TYPEMULTIPART)
+	if (clear_content->type == TYPEMULTIPART)
 	{
 	  if (!(msg->pgp & PGPENCRYPT) && (msg->pgp & PGPSIGN))
 	  {
 	    /* save initial signature and attachments */
 	    save_sig = msg->content->parts->next;
-	    save_parts = msg->content->parts->parts->next;
+	    save_parts = clear_content->parts->next;
 	  }
 
 	  /* this means writing only the main part */
-	  msg->content = save_content->parts;
+	  msg->content = clear_content->parts;
 
 	  if (pgp_protect (msg, pgpkeylist) == -1)
 	  {
@@ -1264,32 +1458,23 @@ main_loop:
 	    goto full_fcc;
 	  }
 
-	  if (msg->pgp & PGPENCRYPT)
-	  {
-	    /* not released in pgp_encrypt_message() */
-	    mutt_free_body (&save_content->parts);
-	    /* make sure we release the right thing later */
-	    save_content->parts = msg->content;
-
-	    encode_descriptions (msg->content);
-	  }
-	  else
-	    save_content = msg->content;
+	  save_content = msg->content;
 	}
       }
       else
-#endif /* _PGPPATH */
+#endif /* HAVE_PGP */
 	msg->content = msg->content->parts;
     }
 
-#ifdef _PGPPATH
+#ifdef HAVE_PGP
 full_fcc:
-#endif /* _PGPPATH */
+#endif /* HAVE_PGP */
     if (msg->content)
       mutt_write_fcc (fcc, msg, NULL, 0, NULL);
+
     msg->content = tmpbody;
 
-#ifdef _PGPPATH
+#ifdef HAVE_PGP
     if (save_sig)
     {
       /* cleanup the second signature structures */
@@ -1301,23 +1486,36 @@ full_fcc:
       msg->content->parts->next = save_sig;
       msg->content->parts->parts->next = save_parts;
     }
-#endif /* _PGPPATH */
+    else if (save_content)
+    {
+      /* destroy the new encrypted body. */
+      mutt_free_body (&save_content);
+    }
+      
+#endif /* HAVE_PGP */
   }
 
-#ifdef _PGPPATH
-  if (msg->pgp & PGPENCRYPT)
-  {
-    /* cleanup structures from the first encryption */
-    mutt_free_body (&save_content);
-    FREE (&pgpkeylist);
-  }
-#endif /* _PGPPATH */
 
-  if (send_message (msg) == -1)
+  if ((i = send_message (msg)) == -1)
   {
     if (!(flags & SENDBATCH))
     {
+#ifdef HAVE_PGP
+      if ((msg->pgp & PGPENCRYPT) || 
+	  ((msg->pgp & PGPSIGN) && msg->content->type == TYPEAPPLICATION))
+      {
+	mutt_free_body (&msg->content); /* destroy PGP data */
+	msg->content = clear_content;	/* restore clear text. */
+      }
+      else if ((msg->pgp & PGPSIGN) && msg->content->type == TYPEMULTIPART)
+      {
+	mutt_free_body (&msg->content->parts->next);		/* destroy sig */
+	msg->content = mutt_remove_multipart (msg->content);	/* remove multipart */
+      }
+#endif
       msg->content = mutt_remove_multipart (msg->content);
+      decode_descriptions (msg->content);
+      mutt_unprepare_envelope (msg->env);
       goto main_loop;
     }
     else
@@ -1326,13 +1524,21 @@ full_fcc:
       goto cleanup;
     }
   }
+  else if (!option (OPTNOCURSES) && ! (flags & SENDMAILX))
+    mutt_message (i == 0 ? _("Mail sent.") : _("Sending in background."));
 
-  if (!option (OPTNOCURSES) && ! (flags & SENDMAILX))
-    mutt_message _("Mail sent.");
+#ifdef HAVE_PGP
+  if (msg->pgp & PGPENCRYPT)
+  {
+    /* cleanup structures from the first encryption */
+    mutt_free_body (&clear_content);
+    FREE (&pgpkeylist);
+  }
+#endif /* HAVE_PGP */
 
   if (flags & SENDREPLY)
   {
-    if (cur)
+    if (cur && ctx)
       mutt_set_flag (ctx, cur, M_REPLIED, 1);
     else if (!(flags & SENDPOSTPONED) && ctx && ctx->tagged)
     {
@@ -1342,11 +1548,14 @@ full_fcc:
     }
   }
 
+
+  rv = 0;
+  
 cleanup:
 
 
 
-#ifdef _PGPPATH
+#ifdef HAVE_PGP
   if (flags & SENDPOSTPONED)
   {
     
@@ -1362,10 +1571,12 @@ cleanup:
       PgpSignMicalg = signmic;
     }
   }
-#endif /* _PGPPATH */
+#endif /* HAVE_PGP */
    
   if (tempfp)
     fclose (tempfp);
   mutt_free_header (&msg);
+  
+  return rv;
 
 }
